#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import os, re

class CodeAnalizer(): # CodeAnalizer for LORZE erasandcad scripts


    def __init__(self):
        self.__scriptlist= [] # list with python scripts
        self.__codedict= {} # dictionairy for codes
        self.__analizedict= {} # dictionairy for analized code


    def GetScriptList(self):
        os.chdir('../Modules') # get python script list / change to script directory
        dirlist= os.listdir('./') # read the directory list

        for i in dirlist: # get all files
            if i.endswith('.py'): # python script found
                self.__scriptlist.append(i) # add to scriptlist


    def ReadScripts(self):
        for i in self.__scriptlist: # read python scripts in the scriptlist / read all files
            pyfile= open(i, 'r') # open script
            self.__codedict[i]= pyfile.read() # read text
            pyfile.close() # close script


    def AnalizeCode(self):  # analize code in codedict
        self.__analizedict= {}  # clear analizedict

        self.__analizedict['0_files']= []  # prepare list for files
        self.__analizedict['zzz_trash']= [] # prepare list for trash

        for pyfile, codetxt in self.__codedict.items():  # each python script in codedict

            #~ attributes= set()  # 1st try with regular expresions
            #~ classpattern= re.compile('class.+:?')
            #~ attrpattern= re.compile('__.+=?')
            #~ methpattern= re.compile('def.+:?')
            #~ print('')
            #~ print(pyfile)
            #~ print('-'* len(pyfile))
            #~ print(classpattern.findall(codetxt))
            #~ print(attrpattern.findall(codetxt))
            #~ print(methpattern.findall(codetxt))

            # 2 files: 1.py, 2.py
            # classes in 1.py: FirstA, FirstB
            # codedict, {'0_files':['1.py', '2.py'], '1.py/class':['FirstA', 'FirstB'], '2.py/class':['SecondA', 'SecondB'], '1.py/class/FirstA/methods':[..], '1.py/class/FirstA/attributes':[..], '1.py/class/FirstB/methods':[..], '1.py/class/FirstB/attributes':[..], .., '/trash':['All otther lines', ..]}

            self.__analizedict['0_files'].append(pyfile) # store filename in list with key '0_files'
            self.__analizedict[pyfile+ '/class']= [] # prepare list for classes

            for i in codetxt.splitlines(): # each line seperatly
                linetxt= i.lstrip() # without whitespaces on the left side

                if linetxt[:5]== 'class': # class found
                    classtxt= linetxt[5:len(linetxt)- 1] # without 'class' & ':'
                    classtxt= classtxt.lstrip().rstrip() # without whitespaces

                    self.__analizedict[pyfile+ '/class'].append(classtxt) # store in dict
                    self.__analizedict[pyfile+ '/class/'+ classtxt+ '/methods']= [] # prepare list for methods
                    self.__analizedict[pyfile+ '/class/'+ classtxt+ '/attributes']= [] # prepare list for attributes

                elif linetxt[:3]== 'def': # method found
                    methodtxt= linetxt[3:] # without 'def'
                    methodtxt= methodtxt.replace('(self, ', '(') # '(self, ' > '('
                    methodtxt= methodtxt.replace('(self,', '(') # '(self,' > '('
                    methodtxt= methodtxt.replace('(self)', '()') # '(self)' > '()'

                    foundend1= methodtxt.find('):') # search '):'
                    foundend2= methodtxt.find(') :') # search ') :'

                    if foundend1!= -1: # '):' is found
                        methodtxt= methodtxt[:foundend1+ 2]
                    elif foundend2!= -1: # ') :' is found
                        methodtxt= methodtxt[:foundend2+ 2]

                    self.__analizedict[pyfile+ '/class/'+ classtxt+ '/methods'].append(methodtxt.lstrip().rstrip()) # store without whitespaces

                elif linetxt[:7]== 'self.__':  # private attribute found
                    attributetxt= linetxt[5:len(linetxt)] # without 'self.'
                    foundequal= attributetxt.find('=') # search 1st '='

                    if foundequal!= -1: # '=' is found
                        attributetxt= attributetxt[:foundequal] # all characters on the left side
                        attributetxt= attributetxt.lstrip().rstrip()

                        if attributetxt not in self.__analizedict[pyfile+ '/class/'+ classtxt+ '/attributes']: # attribute already in list?
                            self.__analizedict[pyfile+ '/class/'+ classtxt+ '/attributes'].append(attributetxt) # store new attribute
                    else: # no '=' found
                        self.ToTrash(linetxt)

                else: # nothing found
                    self.ToTrash(linetxt)


    def ToTrash(self, linetxt): # store linetext to dict as trash
        trashtxt= linetxt.lstrip().rstrip() # without whitespaces
        if trashtxt not in self.__analizedict['zzz_trash']: # trash already in list?
            self.__analizedict['zzz_trash'].append(trashtxt) # store to trash


    def FileOutput(self): # write analizedict sorted to analysis.txt
        keylist= self.__analizedict.keys() # get all keys from dict
        keylist.sort() # sort keys

        os.chdir('../Documentation') # back to documentation directory

        txtfile= open('CodeAnalysis.txt', 'w') # open file to write

        for i in keylist: # each key on a seperate line
            txtfile.write(i+ '\n') # write key
            underline= '-'* len(i) # underline key
            txtfile.write(underline+ '\n') # write underline

            for j in self.__analizedict[i]: # each value in list
                txtfile.write(j+ '\n') # write value

            txtfile.write('\n') # empty line at the end

        txtfile.close() # close file


    def UmlOutput(self): # write analizedict sorted for uml to analysis.txt
        os.chdir('../Develop_Docu') # back to documentation directory
        txtfile= open('CodeAnalysis.txt', 'w') # open file to write

        self.__analizedict['0_files'].sort() # sort scriptlist
        line1= '-'* 60 # ---
        line2= '='* 60 # ===

        for pyscript in self.__analizedict['0_files']: # on each script
            txtfile.write(pyscript+ '\n') # write script
            #~ txtfile.write(line2+ '\n') # write ===

            for classtxt in self.__analizedict[pyscript+ '/class']: # on each class
                txtfile.write(line2+ '\n') # write ===
                txtfile.write(classtxt+ '\n') # write class
                txtfile.write(line1+ '\n') # write ---

                self.__analizedict[pyscript+ '/class/'+ classtxt+ '/attributes'].sort() # sort attributes
                for attributetxt in self.__analizedict[pyscript+ '/class/'+ classtxt+ '/attributes']: # on each attribute
                    txtfile.write(attributetxt+ '\n') # write attribute
                txtfile.write(line1+ '\n') # write ---

                methodlist= self.__analizedict[pyscript+ '/class/'+ classtxt+ '/methods']
                init= methodlist[0] # get method '__init__'
                del methodlist[0] # delete method '__init__'
                methodlist.sort() # sort methodes
                methodlist.insert(0, init) # insert '__init__' on top
                for methodtxt in methodlist: # on each attribute
                    txtfile.write(methodtxt+ '\n') # write methode
                txtfile.write(line2+ '\n') # write ===
                txtfile.write('\n\n') # write 2 empty lines

        txtfile.write('TRASH\n') # write title
        txtfile.write('*****\n') # underline title
        for txt in self.__analizedict['zzz_trash']: # on each line
            txtfile.write(txt+ '\n') # write trash

        txtfile.close() # close file


if __name__== '__main__':
    a= CodeAnalizer()
    a.GetScriptList()
    a.ReadScripts()
    a.AnalizeCode()
    #~ a.FileOutput()
    a.UmlOutput()
    print('')
    print('CodeAnalizer')
    print('------------')
    print('')
    print('The file "CodeAnalysis.txt" as a preparation for the uml illustration has been successfully written')
    print('')
