#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of "LORZE erasandcad"

# "LORZE erasandcad" is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# "LORZE erasandcad" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx, os, zipfile
from pontoninstall import PontonData
from pontoninstall import PontonSysScript
from pontoninstall import PontonTest

class PontonGUI(wx.Frame):
    # GUI for ponton hybrid

    def __init__(self):

        # PontonData
        self.__data= PontonData()

        # subclass
        wx.Frame.__init__(self, None, title= self.__data.Get('pontonguititle'), size= self.__data.Get('pontonguisize'))

        # icon
        self.SetIcon(wx.Icon('pontonicon.png', wx.BITMAP_TYPE_PNG))

        # PontonFind
        self.__find= PontonFind(self)

        # PontonTest
        self.__test= PontonTest()

        # title with picture
        titletxt= wx.StaticText(self, label= self.__data.Get('programtitle'))
        titlefont= wx.Font(14, wx.SWISS, wx.NORMAL, wx.BOLD)
        titletxt.SetFont(titlefont)
        titlepic= wx.StaticBitmap(self, bitmap= wx.Bitmap(self.__data.Get('programpic')))

        # text infos
        pytxt= wx.StaticText(self, label= 'Python installed version: '+ self.__test.GetPyVer())
        wxtxt= wx.StaticText(self, label= 'wxPython installed version: '+ self.__test.GetWxPyVer())

        # boxes
        sysbox= wx.StaticBox(self, label= u'System informations')

        # buttons
        buttoninstall= wx.Button(self, label= u'Install')
        buttoninstall.SetToolTip(wx.ToolTip(u'Install '+ self.__data.Get('pontonguititle')+ ' on your system'))

        buttonremove= wx.Button(self, label= u'Remove')
        buttonremove.SetToolTip(wx.ToolTip(u'Remove existing installations of '+ self.__data.Get('programname')+ ' from your system'))

        buttonexit= wx.Button(self, label= u'Exit')
        buttonexit.SetToolTip(wx.ToolTip(u'Quit the installation'))

        buttoninfo= wx.Button(self, label= u'Info about python & wxpython')
        buttoninfo.SetToolTip(wx.ToolTip(u'Administration of python & wxpython'))

        # bindings
        buttoninstall.Bind(wx.EVT_BUTTON, self.OnInstall)
        buttonremove.Bind(wx.EVT_BUTTON, self.OnRemove)
        buttonexit.Bind(wx.EVT_BUTTON, self.OnExit)
        buttoninfo.Bind(wx.EVT_BUTTON, self.OnInfo)

        # sizers
        vbox= wx.BoxSizer(wx.VERTICAL)
        grid= wx.GridSizer(1, 3, 5, 5)
        syssizer= wx.StaticBoxSizer(sysbox, wx.VERTICAL)

        # layout
        vbox.Add(titletxt, 0, wx.ALL|wx.ALIGN_CENTRE, 5)
        vbox.Add(titlepic, 0, wx.ALL|wx.ALIGN_CENTRE, 5)

        syssizer.Add(pytxt, 0, wx.ALL, 5)
        syssizer.Add(wxtxt, 0, wx.ALL, 5)
        syssizer.Add(buttoninfo, 0, wx.EXPAND|wx.ALL, 5)
        vbox.Add(syssizer, 0, wx.EXPAND|wx.ALL, 5)

        grid.AddMany([(buttoninstall, 0, wx.EXPAND), (buttonremove, 0, wx.EXPAND), (buttonexit, 0, wx.EXPAND)])
        vbox.Add(grid, 0, wx.EXPAND|wx.ALL, 5)

        self.SetSizer(vbox)
        self.Centre()
        self.Show()


    def OnInstall(self, event):
        instdlg= PontonInstDlg(self)
        instdlg.ShowModal()
        appdir= instdlg.GetAppDir()
        instdlg.Destroy()

        if appdir!= '':

            # zipfile
            source= zipfile.ZipFile('pontonsrc.zip')

            # extract zipfile to destination (appdir)
            source.extractall(appdir)

            # Uninstall info file
            uninstallbool= self.MakeUninstallFile(appdir)

            # make all .py and .sh executable
            executablebool= self.MakeExecutable(appdir)

            # prepare text for message
            text= u'The installation was created.'

            if uninstallbool== True and executablebool== True:
                text+= u' There was no errors.'
            elif uninstallbool== False and executablebool== True:
                text+= u' There were errors while creating the file uninstall list.'
            elif uninstallbool== True and executablebool== False:
                text+= u' There was error in granting the execution rights.'
            else:
                text+= u' There were errors when creating the Uninstall list and in granting the execution rights.'

            # show message
            dlg= wx.MessageDialog(self, text, u'Information', style= wx.OK)
            dlg.ShowModal()

            # starter dialogue
            dlg= PontonStarterDlg(self, appdir)
            dlg.ShowModal()
            starterstatus= dlg.GetStatus()
            dlg.Destroy()

            if starterstatus== True:
                dlg= wx.MessageDialog(self, u'The starter was created.', u'Information', style= wx.OK)
                dlg.ShowModal()
            else:
                dlg= wx.MessageDialog(self, u'The starter could not be created. Please manually create the starter. Make sure that lrzgui.py will be startet in the "Modules" Directory (for example: LORZE/Modules)', u'Information', style= wx.OK)
                dlg.ShowModal()

            dlg= wx.MessageDialog(self, u'The installation is complete. The program will be terminated. To uninstall or reinstall "'+ self.__data.Get('programname')+ '" run the setup programm "'+ self.__data.Get('pontonstartername')+ '" again.', u'Finish', style= wx.OK)
            dlg.ShowModal()

            self.Close()



    def OnRemove(self, event):
        # get directory dictionairy from pontonfind
        dirdic= self.__find.SearchExistInst()

        # list with directorys
        dirlist= dirdic.keys()

        if len(dirlist)== 0:
            # no existing installation found
            dlg= wx.MessageDialog(self, u'There was no installation found.', u'Information', style= wx.OK)
            dlg.ShowModal()

        else:
            # existing installations found
            dlg= PontonListDlg(self, u'Select Installation', dirlist, dirlist[0])
            dlg.ShowModal()
            appdir= dlg.GetChoice()
            dlg.Destroy()
            if appdir!= '':
                # OK clicked
                question= wx.MessageDialog(self, u'Are you sure to remove this installation: '+ appdir+ '?', u'Confirmation', wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
                answer= question.ShowModal()

                if answer== wx.ID_YES:
                    # remove installation
                    for dirtxt, version in dirdic.items():
                        if dirtxt== os.path.join(appdir, 'Modules'):
                            verstring= version
                    self.RemoveApp(appdir, version)


    def OnExit(self, event):
        self.Close()


    def RemoveApp(self, appdir, version):
        # remove application from disk / distring= path to root directory of the application / version= string with application version to uninstall / read uninstall info file
        dirlist, filelist= self.ReadUninstallFile(appdir)

        # remove entry from wx.config
        self.__find.RemoveAppDir(appdir, version)

        if dirlist== [] or filelist== []:
            # no uninstall info file found
            dlg= wx.MessageDialog(self, u'There was no uninstall info file found. Please remove the installation manually.', u'Information', style= wx.OK)
            dlg.ShowModal()

        else:
            # uninstall info file exist
            cwd= os.getcwd()

            # logtext
            log= ''

            # chance work directory to installation
            try:
                os.chdir(appdir)
                log+= 'OK|chdir\n   '+ appdir+ '\n'
            except:
                log+= 'FAIL|chdir\n     '+ appdir+ '\n'

            for i in filelist:
                # remove files
                try:
                    os.remove(i)
                    log+= 'OK|remove\n   '+ i+ '\n'
                except:
                    log+= 'FAIL|remove\n     '+ i+ '\n'
                if i.endswith('.py'):
                    # python script
                    try:
                        # delete .pyc
                        os.remove(i+ 'c')
                    except:
                        pass

            # chance back to cwd
            try:
                os.chdir(cwd)
                log+= 'OK|chdir\n   '+ cwd+ '\n'
            except:
                log+= 'FAIL|chdir\n   '+ cwd+ '\n'

            # remove directory structure
            for i in dirlist:
                # remove directorys
                try:
                    os.removedirs(i)
                    log+= 'OK|removedirs\n   '+ i+ '\n'
                except:
                    log+= '.'+ i+ '\n'

            # check if application directory already exist
            if os.path.isdir(appdir):
                # path already exist
                log+= 'FAIL|could not remove '+ appdir
                msgtext= u'The uninstall was successful. The directory '+ appdir+ 'could not removed, please delete that directory manually. You like to read the log?'

            else:
                # path not longer exist
                log+='OK|all directorys removed ..'
                msgtext= u'The uninstall was successful. You like to read the log?'

            question= wx.MessageDialog(None, msgtext, u'Confirmation', wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
            answer= question.ShowModal()

            if answer== wx.ID_YES:
                # show log
                dlg= PontonTextDlg(self, u'Log text: Remove '+ appdir, log)
                dlg.ShowModal()
                dlg.Destroy()


    def MakeUninstallFile(self, appdir):
        # read all files and directorys in appdir / os walk
        files= os.walk(appdir)

        # clear filesdic, dictionairy for structure,
        text= ''

        # read structure
        for i in files:
            directorytxt, subdirlist, fileslist= i

            # directory
            text+= 'DIRECTORY: '+ directorytxt+ '\n'

            # files
            for i in fileslist:
                pathtxt= os.path.join(directorytxt, i)
                text+= 'FILE: '+ pathtxt+ '\n'

        # write uninstall info text file
        try:
            uninstpath= os.path.join(appdir, self.__data.Get('uninstfile'))
            txtfile= open(uninstpath, 'w')
            txtfile.write(text)
            txtfile.close()

            # succesful saved, return true
            return(True)

        except:
            dlg= wx.MessageDialog(self, u'There was an error while writing the uninstall info file: '+ self.__data.Get('uninstfile'), u'Error', style= wx.OK|wx.ICON_ERROR)
            dlg.ShowModal()

            # failed, return false
            return(False)


    def ReadUninstallFile(self, appdir):
        # read uninstall info text file / return 2 list, dirlist and filelist
        try:
            # make complete path
            uninstpath= os.path.join(appdir, self.__data.Get('uninstfile'))

            # read file and split text to lines
            txtfile= open(uninstpath, 'r')
            text= txtfile.read()
            txtfile.close()
            linetxt= text.splitlines()

            # clear filelist, dirlist
            filelist, dirlist= [], []

            # read linetxt
            for i in linetxt:
                if i.startswith('DIRECTORY: '):
                    dirlist.append(i[11:])
                elif i.startswith('FILE: '):
                    filelist.append(i[6:])

            return(dirlist, filelist)

        except:
            return([], [])


    def MakeExecutable(self, appdir):
        # make all *.py and *.sh executable / read uninstallfile, success= true (no error)
        dirlist, filelist= self.ReadUninstallFile(appdir)
        success= True

        if filelist== []:
            success= False

        for i in filelist:
            if i.endswith('.py') or i.endswith('.sh'):
                try:
                    os.chmod(i, 0o777)
                except:
                    # error
                    success= False

        return(success)


    def OnInfo(self, event):
        dlg= wx.MessageDialog(self, u'Python and wxPython you have to manage in their operating system. Further informations you see in your operating system documentation or on www.python.org, www.wxpython.org', u'Information', style= wx.OK)
        dlg.ShowModal()


class PontonListDlg(wx.Dialog):
    # dialog with a listbox choice

    def __init__(self, parent, titel, choicelist, default):
        # PontonData
        self.__data= PontonData()

        # subclass
        wx.Dialog.__init__(self, parent, wx.ID_ANY, titel, size=self.__data.Get('listseldlgsize'), style= wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)

        # choicelist and choiceindex
        self.__choicelist= choicelist
        self.__choiceindex= None

        # listbox
        self.__list= wx.ListBox(self, size= (-1, -1), choices= self.__choicelist)

        # search index of default and select default
        for i in range(len(self.__choicelist)):
            if self.__choicelist[i]== default:
                self.__list.SetSelection(i)

        buttonok= wx.Button(self, label= u'OK')
        buttoncancel= wx.Button(self, label= u'Cancel')

        # bindings
        buttonok.Bind(wx.EVT_BUTTON, self.OnOk)
        buttoncancel.Bind(wx.EVT_BUTTON, self.OnCancel)
        self.__list.Bind(wx.EVT_LISTBOX_DCLICK, self.OnOk)
        self.__list.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)

        # layout
        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(self.__list, 1, wx.EXPAND|wx.ALL, 5)
        hbox= wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(buttonok, 1, wx.EXPAND|wx.ALL, 5)
        hbox.Add(buttoncancel, 1, wx.EXPAND|wx.ALL, 5)
        vbox.Add(hbox, 0, wx.EXPAND)

        self.__list.SetFocus()

        self.SetSizer(vbox)
        self.Centre()


    def OnOk(self, event):
        self.OkDialogue()


    def OnCancel(self, event):
        self.CancelDialogue()


    def GetChoice(self):
        if self.__choiceindex== None:
            return('')
        else:
            return(self.__choicelist[self.__choiceindex])


    def CancelDialogue(self):
        self.__choiceindex= None
        self.Close()


    def OkDialogue(self):
        self.__choiceindex= self.__list.GetSelection()
        self.Close()


    def OnKeyDown(self, event):
        if event.GetKeyCode()== wx.WXK_ESCAPE:
            self.CancelDialogue()
        elif event.GetKeyCode()== wx.WXK_RETURN:
            self.OkDialogue()
        elif event.GetKeyCode()== wx.WXK_NUMPAD_ENTER:
            self.OkDialogue()


class PontonInstDlg(wx.Dialog):
    #Ponton hybrid install dialogue

    def __init__(self, parent):

        # PontonData
        self.__data= PontonData()

        # subclass
        wx.Dialog.__init__(self, parent, wx.ID_ANY, u'Install '+ self.__data.Get('programname'), size= self.__data.Get('installdlgsize'), style= wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)

        self.__find= PontonFind(self)

        # box
        dirbox= wx.StaticBox(self, label= u'Program directory')

        # directory text
        self.__dirtxt= wx.StaticText(self, label= '')

        # buttons
        buttonchangedir= wx.Button(self, label= u'Change directory')
        buttonchangedir.SetToolTip(wx.ToolTip(u'Select another program directory for '+ self.__data.Get('programname')))

        buttonupdate= wx.Button(self, label= u'Update '+ self.__data.Get('programname'))
        buttonupdate.SetToolTip(wx.ToolTip(u'Select an existing '+ self.__data.Get('programname')+ ' installation to update'))

        buttonok= wx.Button(self, label= u'OK')
        buttonok.SetToolTip(wx.ToolTip(u'Install '+ self.__data.Get('programname')+ ' in the choosen program directory'))

        buttoncancel= wx.Button(self, label= u'Cancel')
        buttoncancel.SetToolTip(wx.ToolTip(u'Return to previous window'))

        # bindings
        buttonchangedir.Bind(wx.EVT_BUTTON, self.OnChangeDir)
        buttonupdate.Bind(wx.EVT_BUTTON, self.OnUpdate)
        buttonok.Bind(wx.EVT_BUTTON, self.OnOK)
        buttoncancel.Bind(wx.EVT_BUTTON, self.OnCancel)

        # layout
        vbox= wx.BoxSizer(wx.VERTICAL)
        dirsizer= wx.StaticBoxSizer(dirbox, wx.VERTICAL)
        grid= wx.GridSizer(1, 2, 5, 5)

        dirsizer.Add(self.__dirtxt, 0, wx.ALL, 5)
        vbox.Add(dirsizer, 0, wx.EXPAND|wx.ALL, 5)
        vbox.Add(buttonchangedir, 0, wx.EXPAND|wx.ALL, 5)
        vbox.Add(buttonupdate, 0, wx.EXPAND|wx.ALL, 5)

        grid.AddMany([(buttonok, 0, wx.EXPAND), (buttoncancel, 0, wx.EXPAND)])
        vbox.Add(grid, 0, wx.EXPAND|wx.ALL, 5)

        self.SetSizer(vbox)
        self.Centre()

        self.SetAppDir(self.__find.GetDefaultDir())


    def OnChangeDir(self, event):
        dlg = wx.DirDialog(self, u'Choose a directory', defaultPath= self.__appdir, style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dlg.ShowModal() == wx.ID_OK:
            self.SetAppDir(dlg.GetPath())
        dlg.Destroy()


    def OnOK(self, event):
        self.Close()


    def OnCancel(self, event):
        self.__appdir= ''
        self.Close()


    def GetAppDir(self):
        return(self.__appdir)


    def SetAppDir(self, appdir):
        self.__appdir= os.path.normpath(appdir)
        self.__dirtxt.SetLabel(self.__appdir)


    def OnUpdate(self, event):
        # get dirlist from pontonfind
        dirlist= self.__find.SearchAppDir()

        if len(dirlist)== 1:
            # only 1 existing installation found
            self.SetAppDir(dirlist[0])

        else:
            # many installations found
            dlg= PontonListDlg(self, u'Select Installation', dirlist, dirlist[0])
            dlg.ShowModal()
            appdir= dlg.GetChoice()
            dlg.Destroy()
            if appdir!= '':
                self.SetAppDir(appdir)


class PontonFind():
    # object to find installation and folders

    def __init__(self, parent):
        #parent
        self.__parent= parent

        # default program directory
        dir= os.getcwd()
        if dir.endswith('Install'):
            # cut '/Install'
            self.__defaultdir= dir[:len(dir)- 8]
        elif dir.endswith('ponton5hybrid'):
            # cut '/Install'
            dirtxt= dir[:len(dir)- 14]
            self.__defaultdir= os.path.join(dirtxt, 'LORZE')
        else:
            self.__defaultdir= dir

        # PontonData
        self.__data= PontonData()

        self.__founddic= self.SearchExistCfg()


    def SearchExistCfg(self):
        # search installed versions with the wx.Config, clear foundcfg, dictionairy, key= version, value= config file
        founddic= {}

        # get filedic from PontonData
        configdic= self.__data.Get('configdic')

        for version, configfile in configdic.items():
            # set config file
            cfg= wx.Config(configfile)

            # read specified key, interger, default= 0
            value= cfg.ReadInt(self.__data.Get('cfgfindkey'), 0)

            if value!= 0:
                # wx.Config from lorze is found
                founddic[version]= configfile

        return(founddic)


    def SearchAppDir(self):
        # search directory to install, return list with paths, ['path', ..], read found versions
        verslist= self.__founddic.keys()

        # clear dirstring
        dirstring= ''

        if self.__data.Get('appversion') in verslist:
            # same version found, get dir from config
            cfg= wx.Config(self.__founddic[self.__data.Get('appversion')])
            dirstring= cfg.Read(self.__data.Get('cfgdirkey'), '')

        else:
            # search previous version
            for i in self.__data.Get('prevverions'):
                if i in verslist:
                    # found version
                    cfg= wx.Config(self.__founddic[i])
                    dirstring= cfg.Read(self.__data.Get('cfgdirkey'), '')
                    break

        # testing
        #~ dirstring= '/home/user/desktop/LORZE/Modules, /home/user/desktop/CAD/LORZE/Modules, home/user/desktop/CAD/LorzeTest/OwnStructure'
        #~ dirstring= ''

        if dirstring== '':
            # no installed versions found / show info box
            dlg= wx.MessageDialog(self.__parent, u'There was no installation found.', u'Information', style= wx.OK)
            dlg.ShowModal()

            # set directory to default
            dirstring= os.path.normpath(self.__defaultdir)

        # split dirstring into list, seperator ist ','
        dirlist= dirstring.split(',')

        for i in range(len(dirlist)):
            # remove whitespaces
            dirlist[i].strip()

            if dirlist[i].endswith('Modules'):
                # cut '/Modules'
                lnew= len(dirlist[i])- 8
                dirlist[i]= dirlist[i][:lnew]

        return(dirlist)


    def SearchExistInst(self):
        # search directorys of installations / clear dic: key= directory, value= version
        instdic= {}

        # read dictionairy of found config files
        for version, cfgfile in self.__founddic.items():
            # read directory string from config file
            cfg= wx.Config(cfgfile)
            dirstring= cfg.Read(self.__data.Get('cfgdirkey'), '')

            if dirstring!= '':
                # split dirstring into list, seperator ist ','
                dirlist= dirstring.split(',')

                for i in range(len(dirlist)):
                    # remove whitespaces
                    dirlist[i].strip()

                    if dirlist[i].endswith('Modules'):
                        # cut '/Modules'
                        lnew= len(dirlist[i])- 8
                        dirlist[i]= dirlist[i][:lnew]

                    # directory= key, version= value
                    instdic[dirlist[i]]= version

        return(instdic)


    def GetDefaultDir(self):
        return(self.__defaultdir)


    def RemoveAppDir(self, appdir, version):
        # remove a directory from the path entry in the config file
        cfg= wx.Config(self.__founddic[version])

        # read dirstring
        dirstring= cfg.Read(self.__data.Get('cfgdirkey'), '')

        if dirstring!= '':
            # split dirstring into list, seperator ist ','
            dirlist= dirstring.split(',')

            # clear dirstring
            dirstring= ''

            for i in dirlist:
                # remove whitespaces
                j= i.strip()

                if j!= os.path.join(appdir, 'Modules'):
                    # directory not matching with the removed application
                    if dirstring== '':
                        # 1st entry
                        dirstring= j
                    else:
                        # not 1st entry
                        dirstring+= ','+ j

            # write config file entry
            cfg.Write(self.__data.Get('cfgdirkey'), dirstring)

            # true, directory successful removed
            return(dirstring)

        else:
            # false, application not found
            return('')


class PontonTextDlg(wx.Dialog):
    #Ponton hybrid install dialogue

    def __init__(self, parent, title, text):
        # PontonData
        self.__data= PontonData()

        # subclass
        wx.Dialog.__init__(self, parent, wx.ID_ANY, title, size= self.__data.Get('textdlgsize'), style= wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)

        textctrl = wx.TextCtrl(self, value= text, style=wx.TE_MULTILINE)
        buttonok= wx.Button(self, label= u'OK')

        textctrl.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        buttonok.Bind(wx.EVT_BUTTON, self.OnOk)

        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(textctrl, 1, wx.EXPAND|wx.ALL, 5)
        vbox.Add(buttonok, 0, wx.EXPAND|wx.ALL, 5)

        textctrl.SetFocus()

        self.SetSizer(vbox)
        self.Centre()


    def OnOk(self, event):
        self.Close()


    def OnKeyDown(self, event):
        if event.GetKeyCode()== wx.WXK_ESCAPE:
            self.Close()
        elif event.GetKeyCode()== wx.WXK_RETURN:
            self.Close()
        elif event.GetKeyCode()== wx.WXK_NUMPAD_ENTER:
            self.Close()


class PontonStarterDlg(wx.Dialog):
    #Ponton hybrid install dialogue

    def __init__(self, parent, appdir):
        # PontonData
        self.__data= PontonData()

        # subclass
        wx.Dialog.__init__(self, parent, wx.ID_ANY, u'Starter for '+ self.__data.Get('programname'), size= self.__data.Get('starterdlgsize'), style= wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)

        # root directory of application
        self.__appdir= appdir

        # PontonTest
        self.__test= PontonTest()

        # read platform info
        self.__ostype, pkgtype, archtype= self.__test.GetPlatform()

        # combobox entrys in dependence on operating system / clear list
        self.__starterlist= []

        # set combobox entrys
        if self.__ostype== 'linux':
            self.__starterlist= [u'Desktop Starter', u'Program link']
        elif self.__ostype== 'win':
            self.__starterlist= [u'Shortcut']
        elif self.__ostype== 'mac':
            self.__starterlist= [u'Program link']

        # combobox
        self.__startercombo= wx.ComboBox(self, size= (-1, -1), choices= self.__starterlist, style=wx.CB_READONLY)
        self.__startercombo.SetStringSelection(self.__starterlist[0])

        # directory text
        self.__dirtxt= wx.StaticText(self, label= '')

        # buttons
        buttonchangedir= wx.Button(self, label= u'Change directory')
        buttonchangedir.SetToolTip(wx.ToolTip(u'Select another directory for the '+ self.__data.Get('programname')+ ' starter'))

        buttonok= wx.Button(self, label= u'OK')
        buttonok.SetToolTip(wx.ToolTip(u'Create the program starter for '+ self.__data.Get('programname')))

        buttoncancel= wx.Button(self, label= u'Cancel')
        buttoncancel.SetToolTip(wx.ToolTip(u'Finish the installation without a program starter'))

        # boxes
        typbox= wx.StaticBox(self, label= u'Type of program launcher')
        dirbox= wx.StaticBox(self, label= u'Destination directory')

        # bindings
        buttonchangedir.Bind(wx.EVT_BUTTON, self.OnChangeDir)
        buttonok.Bind(wx.EVT_BUTTON, self.OnOK)
        buttoncancel.Bind(wx.EVT_BUTTON, self.OnCancel)

        # layout
        vbox= wx.BoxSizer(wx.VERTICAL)
        typsizer= wx.StaticBoxSizer(typbox, wx.VERTICAL)
        dirsizer= wx.StaticBoxSizer(dirbox, wx.VERTICAL)
        grid= wx.GridSizer(1, 2, 5, 5)

        typsizer.Add(self.__startercombo, 0, wx.EXPAND|wx.ALL, 5)

        dirsizer.Add(self.__dirtxt, 0, wx.EXPAND|wx.ALL, 5)
        dirsizer.Add(buttonchangedir, 0, wx.EXPAND|wx.ALL, 5)
        grid.AddMany([(buttonok, 0, wx.EXPAND), (buttoncancel, 0, wx.EXPAND)])

        vbox.Add(typsizer, 0, wx.EXPAND|wx.ALL, 5)
        vbox.Add(dirsizer, 0, wx.EXPAND|wx.ALL, 5)
        vbox.Add(grid, 0, wx.EXPAND|wx.ALL, 5)

        self.SetSizer(vbox)
        self.Centre()

        # defult directory
        self.SetStartDir(self.GetDefaultDir())

        # set status
        self.__status= False

        # set starterpaths
        self.SetStarterPaths()


    def OnChangeDir(self, event):
        # change directory
        dlg = wx.DirDialog(self, u'Choose a directory', defaultPath= self.__startdir, style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dlg.ShowModal() == wx.ID_OK:
            self.SetStartDir(dlg.GetPath())
        dlg.Destroy()


    def OnOK(self, event):
        # ok and make starter, read selectet starter
        typ= self.__startercombo.GetValue()

        if typ== u'Desktop Starter':
            self.MakeStartBashScript()
            self.MakeFreeDesktopEntry()
        elif typ== u'Program link':
            self.MakeStartBashScript()
            self.MakeSymlink()
        elif typ== u'Shortcut':
            self.MakeShortcut()

        # true for done
        self.__status= True
        self.Close()


    def OnCancel(self, event):
        # cancel, false for no action
        self.__status= False
        self.Close()


    def SetStartDir(self, startdir):
        # set startdir in attribute and gui
        self.__startdir= os.path.normpath(startdir)
        self.__dirtxt.SetLabel(self.__startdir)


    def GetDefaultDir(self):
        # return default directory in dependence on operating system
        home= os.environ['HOME']

        # test different desktop paths
        found= ''
        for i in self.__data.Get('desktops'):
            if os.path.isdir(os.path.join(home, i)):
                found= os.path.join(home, i)

        if found!= '':
            return(found)
        else:
            return(home)


    def GetStatus(self):
        return(self.__status)


    def MakeSymlink(self):
        # make symlink in startdir / store the current directory, change to destination directory for the starter
        cwd= os.getcwd()
        os.chdir(self.__startdir)

        # create text for installed application
        command= 'ln -s '+ self.GetStarterPaths('startscriptpath')+ ' '+ self.__data.Get('startername')

        # testing
        #~ print(command)

        # shell command
        os.system(command)

        # create text for ponton
        command= 'ln -s '+ self.GetStarterPaths('pontonstartscriptpath')+ ' '+ self.__data.Get('pontonstartername')

        # testing
        #~ print(command)

        # shell command
        os.system(command)

        # change back to ponton install directory
        os.chdir(cwd)


    def MakeStartBashScript(self):
        # create a startscript in the root direcory to start / set introtext
        text=  '#!/bin/bash\n\n'
        text+= '# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.\n'
        text+= '# http://erasand.jimdo.com/python-programme/lorze/\n# (C) 2012, Andreas Ulrich\n\n'
        text+= '# This file is part of "LORZE erasandcad"\n\n'
        text+= '# "LORZE erasandcad" is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.\n\n'
        text+= '# "LORZE erasandcad" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\n'
        text+= '# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.\n\n'

        #write startscript for installed application
        try:
            txtfile= open(self.GetStarterPaths('startscriptpath'), 'w')

            cmd=  'cd '+ self.GetStarterPaths('moduledir')+ '\n'
            cmd+= './'+ self.__data.Get('startmodule')+ '\n'

            txtfile.write(text+ cmd)

            txtfile.close()

            # set all rights and executable for owener, group and others
            os.chmod(self.GetStarterPaths('startscriptpath'), 0o777)

        except:
            dlg= wx.MessageDialog(self, u'There was an error while writing '+ self.GetStarterPaths('startscriptpath'), u'Error', style= wx.OK|wx.ICON_ERROR)
            dlg.ShowModal()

        #write startscript for ponton
        try:
            txtfile= open(self.GetStarterPaths('pontonstartscriptpath'), 'w')

            cmd=  'cd '+ self.GetStarterPaths('pontondir')+ '\n'
            cmd+= './pontongui.sh\n'

            txtfile.write(text+ cmd)

            txtfile.close()

            # set all rights and executable for owener, group and others
            os.chmod(self.GetStarterPaths('pontonstartscriptpath'), 0o777)

        except:
            dlg= wx.MessageDialog(self, u'There was an error while writing '+ self.GetStarterPaths('pontonStartscriptPath'), u'Error', style= wx.OK|wx.ICON_ERROR)
            dlg.ShowModal()


    def MakeShortcut(self):
        # create under windows a program shortcut / store the current directory, change to destination directory for the starter
        cwd= os.getcwd()
        os.chdir(self.__startdir)

        # text for command installed application
        command= cwd+ '\Shortcut.exe /F:'+ self.__data.Get('startername')+ '.lnk /A:C /T:'+ os.path.normpath(self.GetStarterPaths('modulepath'))+' /W:'+ os.path.normpath(self.GetStarterPaths('moduledir'))+ ' /R:1 /I'+ self.GetStarterPaths('iconpath')

        # testing
        #~ print(command)

        # windows console command
        os.system(command)

        # text for command ponton
        command= cwd+ '\Shortcut.exe /F:'+ self.__data.Get('pontonstartername')+ '.lnk /A:C /T:'+ os.path.normpath(self.GetStarterPaths('pontoninstpath'))+' /W:'+ os.path.normpath(self.GetStarterPaths('pontondir'))+ ' /R:1 /I'+ self.GetStarterPaths('pontoniconpath')

        # testing
        #~ print(command)

        # windows console command
        os.system(command)

        # change back to ponton install directory
        os.chdir(cwd)


    def MakeFreeDesktopEntry(self):
        # create a free desktop starter for the installed application / entry text
        entrytext=  '[Desktop Entry]\n'
        entrytext+= 'Version=1.0\n'
        entrytext+= 'Type=Application\n'
        entrytext+= 'Name='+ self.__data.Get('startername')+ '\n'
        entrytext+= 'GenericName='+ self.__data.Get('startercategory')+ '\n'
        entrytext+= 'Comment='+ self.__data.Get('startercomment')+ '\n'
        entrytext+= 'Exec='+ self.GetStarterPaths('startscriptpath')+ '\n'
        entrytext+= 'Icon='+ self.GetStarterPaths('iconpath')+ '\n'

        # write desktop file installed application
        filename= os.path.normpath(self.GetStarterPaths('starterpath')+ '.desktop')
        try:
            desktopfile= open(filename, 'w')
            desktopfile.write(entrytext)
            desktopfile.close()
        except:
            dlg= wx.MessageDialog(self, u'There was an error while writing '+ filename, u'Error', style= wx.OK|wx.ICON_ERROR)
            dlg.ShowModal()

        # create a free desktop starter for ponton / entry text
        entrytext=  '[Desktop Entry]\n'
        entrytext+= 'Version=1.0\n'
        entrytext+= 'Type=Application\n'
        entrytext+= 'Name='+ self.__data.Get('pontonstartername')+ '\n'
        entrytext+= 'GenericName='+ self.__data.Get('pontonstartercategory')+ '\n'
        entrytext+= 'Comment='+ self.__data.Get('pontonstartercomment')+ '\n'
        entrytext+= 'Exec='+ self.GetStarterPaths('pontonstartscriptpath')+ '\n'
        entrytext+= 'Icon='+ self.GetStarterPaths('pontoniconpath')+ '\n'

        # write desktop file ponton
        filename= os.path.normpath(self.GetStarterPaths('pontonstarterpath')+ '.desktop')
        try:
            desktopfile= open(filename, 'w')
            desktopfile.write(entrytext)
            desktopfile.close()
        except:
            dlg= wx.MessageDialog(self, u'There was an error while writing '+ filename, u'Error', style= wx.OK|wx.ICON_ERROR)
            dlg.ShowModal()


    def SetStarterPaths(self):
        # set paths to starterpathdic, key= text, value= path / clear dic
        self.__starterpaths= {}

        self.__starterpaths['moduledir']= os.path.join(self.__appdir, self.__data.Get('startdir'))

        self.__starterpaths['modulepath']= os.path.join(self.__appdir, self.__data.Get('startdir'), self.__data.Get('startmodule'))

        self.__starterpaths['startscriptpath']= os.path.join(self.__appdir, self.__data.Get('startscript'))

        self.__starterpaths['starterpath']= os.path.join(self.__startdir, self.__data.Get('startername'))

        self.__starterpaths['iconpath']= os.path.join(self.__appdir, self.__data.Get('startericon'))

        self.__starterpaths['pontondir']= os.path.join(self.__appdir, 'Install')

        self.__starterpaths['pontoninstpath']= os.path.join(self.__appdir, 'Install', 'pontongui.bat')

        self.__starterpaths['pontonstartscriptpath']= os.path.join(self.__appdir, 'PONTON.sh')

        self.__starterpaths['pontonstarterpath']= os.path.join(self.__startdir, 'LORZEinstaller')

        self.__starterpaths['pontoniconpath']= os.path.join(self.__appdir, 'Install', 'pontonicon.png')


    def GetStarterPaths(self, key):
        # returns paths from starterpathdic, key= text
        return(self.__starterpaths[key])


class Develop(wx.Frame):
    # object for testing
    def __init__(self):
        self.__data= PontonData()

        wx.Frame.__init__(self, None, title= u'Develop '+ self.__data.Get('pontonguititle'), size= (300, 600))

        self.__find= PontonFind(self)

        dir= os.getcwd()
        self.__defaultdir= dir[:len(dir)- 8]

        guibox= wx.StaticBox(self, label= u'Dialogues')
        buttongui= wx.Button(self, label= u'PontonGUI')
        buttoninst= wx.Button(self, label= u'InstallDialogue')
        buttonlist= wx.Button(self, label= u'ListDialogue')
        buooninfo= wx.Button(self, label= u'How to make a ponton release binary')
        buttontext= wx.Button(self, label= u'TextDialogue')
        buttonstart= wx.Button(self, label= u'StarterDialogue')

        buttongui.Bind(wx.EVT_BUTTON, self.OnGui)
        buttoninst.Bind(wx.EVT_BUTTON, self.OnInst)
        buttonlist.Bind(wx.EVT_BUTTON, self.OnList)
        buooninfo.Bind(wx.EVT_BUTTON, self.OnInfo)
        buttontext.Bind(wx.EVT_BUTTON, self.OnOsWalk)
        buttonstart.Bind(wx.EVT_BUTTON, self.OnStarter)

        guisizer= wx.StaticBoxSizer(guibox, wx.VERTICAL)
        guisizer.Add(buttongui, 0, wx.EXPAND)
        guisizer.Add(buttoninst, 0, wx.EXPAND)
        guisizer.Add(buttonlist, 0, wx.EXPAND)
        guisizer.Add(buooninfo, 0, wx.EXPAND)
        guisizer.Add(buttontext, 0, wx.EXPAND)
        guisizer.Add(buttonstart, 0, wx.EXPAND)

        pontonfindbox= wx.StaticBox(self, label= u'PontonFind')
        pontonfind1= wx.Button(self, label= u'GetDefaultDir')
        pontonfind2= wx.Button(self, label= u'SearchExistCfg')
        pontonfind3= wx.Button(self, label= u'SearchExistInst')
        pontonfind4= wx.Button(self, label= u'SearchAppDir')
        pontonfind5= wx.Button(self, label= u'RemoveAppDir')

        pontonfind1.Bind(wx.EVT_BUTTON, self.OnGetDefaultDir)
        pontonfind2.Bind(wx.EVT_BUTTON, self.OnSearchExistCfg)
        pontonfind3.Bind(wx.EVT_BUTTON, self.OnSearchExistInst)
        pontonfind4.Bind(wx.EVT_BUTTON, self.OnSearchAppDir)
        pontonfind5.Bind(wx.EVT_BUTTON, self.OnRemoveAppDir)

        pontonfindsizer= wx.StaticBoxSizer(pontonfindbox, wx.VERTICAL)
        pontonfindsizer.Add(pontonfind1, 0, wx.EXPAND)
        pontonfindsizer.Add(pontonfind2, 0, wx.EXPAND)
        pontonfindsizer.Add(pontonfind3, 0, wx.EXPAND)
        pontonfindsizer.Add(pontonfind4, 0, wx.EXPAND)
        pontonfindsizer.Add(pontonfind5, 0, wx.EXPAND)

        buttonexit= wx.Button(self, label= u'Exit')
        buttonexit.Bind(wx.EVT_BUTTON, self.OnExit)

        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(guisizer, 0, wx.EXPAND)
        vbox.Add(pontonfindsizer, 0, wx.EXPAND)
        vbox.Add(buttonexit, 0, wx.EXPAND)

        self.SetSizer(vbox)
        self.Center()
        self.Show()


    def OnInst(self, event):
        dlg= PontonInstDlg(self)
        dlg.ShowModal()
        appdir= dlg.GetAppDir()
        dlg.Destroy()
        print(appdir)



    def OnList(self, event):
        dir= os.getcwd()
        choicelist=[]
        choicelist.append(dir[:len(dir)- 8])
        choicelist.append('./test1')
        choicelist.append('./apps/test2')
        dlg= PontonListDlg(self, 'Testing', choicelist, choicelist[0])
        dlg.ShowModal()
        appdir= dlg.GetChoice()
        dlg.Destroy()
        print(appdir)


    def OnExit(self, event):
        self.Close()


    def OnGui(self, event):
        gui= PontonGUI()


    def OnOsWalk(self, event):
        files= os.walk('../')

        # clear text
        text= ''

        # read structure
        for i in files:
            directorytxt, subdirlist, fileslist= i

            # directory
            text+= 'DIRECTORY: '+ directorytxt+ '\n'

            # files
            for i in fileslist:
                pathtxt= os.path.join(directorytxt, i)
                text+= 'FILE: '+ pathtxt+ '\n'

        dlg= PontonTextDlg(self, 'devlelop: os.walk', text)
        dlg.ShowModal()
        dlg.Destroy()


    def OnInfo(self, event):
        text=  'PONTONSRC.ZIP\n'
        text+= 'Make a source direcory named "pontonsrc"\n'
        text+= 'Copy all needed directorys and files into "pontonsrc"\n'
        text+= 'Copy the installer (directory "Install" with all files and subdirectorys) into "pontonsrc"\n'
        text+= 'Copy "setupBASH.sh" & "setupWIN.bat" into "pontonsrc"\n'
        text+= 'Pack all directorys and files in "pontonsrc" to "pontonsrc.zip"\n\n'
        text+= 'APP_ver.ZIP\n'
        text+= 'Make a release direcory named "APP_ver" (APP_ver is a placeholder for the name and version of the application)\n'
        text+= 'Copy "setupBASH.sh" & "setupWIN.bat" into "APP_ver"\n'
        text+= 'Copy the installer (directory "Install" with all files and subdirectorys) into "APP_ver"\n'
        text+= 'Copy "pontonsrc.zip" into the subdirectory "Install" of "APP_ver"\n'
        text+= 'Pack all directorys and files in "APP_ver" to "APP_ver.zip"\n'
        text+= 'Save the file APP_ver.zip, this ist release binary. Make a md5 hash for safety donwloads.\n\n'
        text+= 'Now you can delete the directorys "pontonsrc" and "APP_ver" with their entire contents.\n\n'
        text+= '(c) 2012, December, Andreas Ulrich'

        dlg= PontonTextDlg(self, 'Information about the sourcefile pontonsrz.zip', text)
        dlg.ShowModal()
        dlg.Destroy()


    def OnGetDefaultDir(self, event):
        dlg= wx.MessageDialog(self, 'string\n'+ self.__find.GetDefaultDir(), u'PontonFind.GetDefaultDir()', style= wx.OK)
        dlg.ShowModal()


    def OnSearchExistCfg(self, event):
        dic= self.__find.SearchExistCfg()
        text= 'dictionairy\n'
        for key, value in dic.items():
            text+= key+ ' : '+ value+ '\n'
        dlg= wx.MessageDialog(self, text, u'PontonFind.SearchExistCfg()', style= wx.OK)
        dlg.ShowModal()


    def OnSearchExistInst(self, event):
        dic= self.__find.SearchExistInst()
        text= 'dictionairy\n'
        for key, value in dic.items():
            text+= key+ ' : '+ value+ '\n'
        dlg= wx.MessageDialog(self, text, u'PontonFind.SearchExistInst()', style= wx.OK)
        dlg.ShowModal()


    def OnSearchAppDir(self, event):
        dirlist= self.__find.SearchAppDir()
        text= 'list\n'
        for i in dirlist:
            text+= i+ '\n'
        dlg= wx.MessageDialog(self, text, u'PontonFind.SearchAppDir()', style= wx.OK)
        dlg.ShowModal()


    def OnRemoveAppDir(self, event):
        text= 'removing "foo": '
        text+= str(self.__find.RemoveAppDir('foo', '0.2'))+ '\n'
        text+= 'removing "/home/tygrys/Dropbox/Andy/ERASAND/LORZE": '
        text+= str(self.__find.RemoveAppDir('/home/tygrys/Dropbox/Andy/ERASAND/LORZE', '0.2'))+ '\n'
        dlg= wx.MessageDialog(self, text, u'PontonFind.RemoveAppDir()', style= wx.OK)
        dlg.ShowModal()


    def OnStarter(self, event):
        dlg= PontonStarterDlg(self, '/home/tygrys')
        print('PontonStarterDlg(self, /home/tygrys)')
        dlg.ShowModal()
        print('GetDefaultDir', dlg.GetDefaultDir())
        print('GetStarterPaths(iconpath)', dlg.GetStarterPaths('iconpath'))
        print('GetStarterPaths(pontoniconpath)', dlg.GetStarterPaths('pontoniconpath'))
        print('GetStarterPaths(moduledir)', dlg.GetStarterPaths('moduledir'))
        print('GetStarterPaths(pontondir)', dlg.GetStarterPaths('pontondir'))
        print('GetStarterPaths(modulepath)', dlg.GetStarterPaths('modulepath'))
        print('GetStarterPaths(pontoninstpath)', dlg.GetStarterPaths('pontoninstpath'))
        print('GetStarterPaths(starterpath)', dlg.GetStarterPaths('starterpath'))
        print('GetStarterPaths(pontonstarterpath)', dlg.GetStarterPaths('pontonstarterpath'))
        print('GetStarterPaths(startscriptpath)', dlg.GetStarterPaths('startscriptpath'))
        print('GetStarterPaths(pontonstartscriptpath)', dlg.GetStarterPaths('pontonstartscriptpath'))
        print('GetStatus', dlg.GetStatus())
        dlg.MakeStartBashScript()
        dlg.MakeFreeDesktopEntry()
        dlg.MakeShortcut()
        dlg.MakeSymlink()
        dlg.Destroy()
        print('DELETE THE CREATET FILES IN THE HOME DIRECTORY AND ON THE DESKTOP.')


if __name__== '__main__':
    app= wx.App()

    # run: 'PontonGui' for installation, 'Test' for testing
    run= 'PontonGui'

    if run== 'PontonGui':
        # gui for installation
        gui= PontonGUI()

    elif run== 'Test':
        # testing
        dev= Develop()

    app.MainLoop()

