#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of "LORZE erasandcad"

# "LORZE erasandcad" is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# "LORZE erasandcad" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import sys, os


class PontonData():
    # object with list of files of known releases

    def __init__(self):
        # dic with options
        self.__pontondata= {}

        # gui: window title
        self.__pontondata['pontonguititle']= 'LORZE erasandcad / ponton5h INSTALLER'

        # gui: window size
        self.__pontondata['pontonguisize']= (400, 420)

        # gui: install dialogue size
        self.__pontondata['installdlgsize']= (500, 180)

        # gui: remove dialogue size
        self.__pontondata['removedlgsize']= (300, 200)

        # gui: list dialouge size
        self.__pontondata['listseldlgsize']= (500, 200)

        # gui: text dialouge size
        self.__pontondata['textdlgsize']= (400, 500)

        # gui: starter dialouge size
        self.__pontondata['starterdlgsize']= (400, 240)

        # gui: program title
        self.__pontondata['programtitle']= u'LORZE erasandcad 0.2 / alpha 2'

        # gui: program title
        self.__pontondata['programname']= u'LORZE erasandcad'

        # gui: picture
        self.__pontondata['programpic']= 'pontonprogrampic.png'

        # config file: dictionairy
        self.__pontondata['configdic']= {'0.1':'lorzecfg', '0.2':'lorze_0-2'}

        # actual program version
        self.__pontondata['appversion']= '0.2'

        # previous program versions
        self.__pontondata['prevverions']= ('0.1')

        # key in config file for identification
        self.__pontondata['cfgfindkey']= 'framesizew'

        # key in config file for program directory
        self.__pontondata['cfgdirkey']= 'instpaths'

        # uninstall info file name
        self.__pontondata['uninstfile']= 'Install/uninstfile.txt'

        # startscript desktops
        self.__pontondata['desktops']= (u'Desktop', u'desktop', u'DESKTOP', u'arbeitsfläche', u'Arbeitsfläche', u'ARBEITSFLÄCHE')

        # startscript names
        self.__pontondata['startscript']= 'LORZE.sh'
        self.__pontondata['startdir']= 'Modules'
        self.__pontondata['startmodule']= 'lrzgui.py'

        # data for freedesktop entry
        self.__pontondata['startername']= 'LORZE'
        self.__pontondata['pontonstartername']= 'LORZEinstaller'
        self.__pontondata['startercategory']= 'CAD'
        self.__pontondata['pontonstartercategory']= 'Utility'
        self.__pontondata['startercomment']= 'erasandcad LORZE 0.2 / alpha 2'
        self.__pontondata['pontonstartercomment']= 'ponton5h INSTALLER'
        self.__pontondata['startericon']= 'Icons/LORZE_64.png'


    def Get(self, optionkey):
        return(self.__pontondata[optionkey])


class PontonRequired():
    # object with all installation datas

    def __init__(self):
        # python version
        self.__pyver= '2.7'

        # wxpython version
        self.__wxpyver= '2.8'

        # python sources, dictionairys, keys= 'apt-get', 'yum', 'hdid32', 'hdid64'
        self.__srcpython= {}
        self.__srcpython['apt-get']= 'sudo apt-get install python\n'

        self.__srcpython['yum']= 'sudo yum install python\n'

        self.__srcpython['hdid32']=  'hdid http://www.python.org/ftp/python/2.7.3/python-2.7.3-macosx10.3.dmg\n'
        self.__srcpython['hdid32']+= 'md5sum python-2.7.3-macosx10.3.dmg > dat1.txt\n'
        self.__srcpython['hdid32']+= 'SUM="$(grep -i 80461c3c60fae64122b51eb20341b453 dat1.txt)"\n'
        self.__srcpython['hdid32']+= 'if [ -n "$SUM" ]\n'
        self.__srcpython['hdid32']+= '    then\n'
        self.__srcpython['hdid32']+= '        cd /Volumes/python-2.7.3-macosx10.3\n'
        self.__srcpython['hdid32']+= '        sudo installer -pkg python-2.7.3.pkg -target /\n'
        self.__srcpython['hdid32']+= 'fi\n'

        self.__srcpython['hdid64']=  'hdid http://www.python.org/ftp/python/2.7.3/python-2.7.3-macosx10.6.dmg\n'
        self.__srcpython['hdid64']+= 'md5sum python-2.7.3-macosx10.6.dmg > dat1.txt\n'
        self.__srcpython['hdid64']+= 'SUM="$(grep -i 15c434a11abe7ea5575ef451cfd60f67 dat1.txt)"\n'
        self.__srcpython['hdid64']+= 'if [ -n "$SUM" ]\n'
        self.__srcpython['hdid64']+= '    then\n'
        self.__srcpython['hdid64']+= '        cd /Volumes/python-2.7.3-macosx10.6\n'
        self.__srcpython['hdid64']+= '        sudo installer -pkg python-2.7.3.pkg -target /\n'
        self.__srcpython['hdid64']+= 'fi\n'

        self.__srcpython['win32']=  'wget -c http://www.python.org/ftp/python/2.7.3/python-2.7.3.msi\n'
        self.__srcpython['win32']+= 'md5.exe python-2.7.3.msi > dat1.txt\n'
        self.__srcpython['win32']+= 'findstr /M "c846d7a5ed186707d3675564a9838cc2" dat1.txt > dat2.txt\n'
        self.__srcpython['win32']+= 'for /f "tokens=*" %%i in (dat2.txt) do msiexec /i python-2.7.3.msi\n'

        self.__srcpython['win64']=  'wget -c http://www.python.org/ftp/python/2.7.3/python-2.7.3.amd64.msi\n'
        self.__srcpython['win64']+= 'md5-x64.exe python-2.7.3.amd64.msi > dat1.txt\n'
        self.__srcpython['win64']+= 'findstr /M "d11d4aeb7e5425bf28f28ab1c7452886" dat1.txt > dat2.txt\n'
        self.__srcpython['win64']+= 'for /f "tokens=*" %%i in (dat2.txt) do msiexec /i python-2.7.3.amd64.msi\n'

        # wxpython sources, dictioairy, keys= 'apt-get', 'yum', 'hdid'
        self.__srcwxpython= {}
        self.__srcwxpython['apt-get']= 'sudo apt-get install python-wxgtk2.8 python-wxtools python-wxversion\n'

        self.__srcwxpython['yum']= 'sudo yum install python-wxgtk2.8 python-wxtools python-wxversion\n'

        self.__srcwxpython['hdid']=  'hdid http://downloads.sourceforge.net/wxpython/wxPython2.8-osx-unicode-2.8.12.1-universal-py2.7.dmg\n'
        self.__srcwxpython['hdid']+= 'md5sum wxPython2.8-osx-unicode-2.8.12.1-universal-py2.7.dmg > dat1.txt\n'
        self.__srcwxpython['hdid']+= 'SUM="$(grep -i e77eafae443566abb4da1d4875f7fc24 dat1.txt)"\n'
        self.__srcwxpython['hdid']+= 'if [ -n "$SUM" ]\n'
        self.__srcwxpython['hdid']+= '    then\n'
        self.__srcwxpython['hdid']+= '        cd /Volumes/wxPython2.8-osx-unicode-2.8.12.1-universal-py2.7\n'
        self.__srcwxpython['hdid']+= '        sudo installer -pkg wxPython2.8-osx-unicode-2.8.12.1.pkg -target /\n'
        self.__srcwxpython['hdid']+= 'fi\n'

        self.__srcwxpython['win32']=  'wget -c http://downloads.sourceforge.net/wxpython/wxPython2.8-win32-unicode-2.8.12.1-py27.exe\n'
        self.__srcwxpython['win32']+= 'md5.exe wxPython2.8-win32-unicode-2.8.12.1-py27.exe > dat1.txt\n'
        self.__srcwxpython['win32']+= 'findstr /M "895ba07ea6c05fcdbc273a921bad787b" dat1.txt > dat2.txt\n'
        self.__srcwxpython['win32']+= 'for /f "tokens=*" %%i in (dat2.txt) do wxPython2.8-win32-unicode-2.8.12.1-py27.exe\n'

        self.__srcwxpython['win64']=  'wget -c http://downloads.sourceforge.net/wxpython/wxPython2.8-win64-unicode-2.8.12.1-py27.exe\n'
        self.__srcwxpython['win64']+= 'md5-x64.exe wxPython2.8-win64-unicode-2.8.12.1-py27.exe > dat1.txt\n'
        self.__srcwxpython['win64']+= 'findstr /M "7875f2f09f01e4fa247ec0ee0c849ffc" dat1.txt > dat2.txt\n'
        self.__srcwxpython['win64']+= 'for /f "tokens=*" %%i in (dat2.txt) do echo wxPython2.8-win64-unicode-2.8.12.1-py27.exe\n'

        # shell commands to restart ponton
        self.__srcrestart= {}
        self.__srcrestart['bash']=  '\ncd ..\n'
        self.__srcrestart['bash']+= './setupBASH.sh\n'

        self.__srcrestart['bat']=  '\ncd ..\n'
        self.__srcrestart['bat']+= 'setupWIN.bat\n'

        # shell commands to start the gui
        self.__srcguicopy= {}
        self.__srcguicopy['bash']=  'mkdir ~/ponton5hybrid\n'
        self.__srcguicopy['bash']+= 'cp pontongui.py ~/ponton5hybrid/pontongui.py\n'
        self.__srcguicopy['bash']+= 'cp pontonprogrampic.png ~/ponton5hybrid/pontonprogrampic.png\n'
        self.__srcguicopy['bash']+= 'cp pontoninstall.py ~/ponton5hybrid/pontoninstall.py\n'
        self.__srcguicopy['bash']+= 'cp pontongui.sh ~/ponton5hybrid/pontongui.sh\n'
        self.__srcguicopy['bash']+= 'cp pontonsrc.zip ~/ponton5hybrid/pontonsrc.zip\n'
        self.__srcguicopy['bash']+= 'cd ~/ponton5hybrid\n'
        self.__srcguicopy['bash']+= './pontongui.sh\n'

        self.__srcguicopy['bat']=  'md c:\ponton5hybrid\n'
        self.__srcguicopy['bat']+= 'copy pontongui.py c:\ponton5hybrid\pontongui.py\n'
        self.__srcguicopy['bat']+= 'copy pontonprogrampic.png c:\ponton5hybrid\pontonprogrampic.png\n'
        self.__srcguicopy['bat']+= 'copy pontoninstall.py c:\ponton5hybrid\pontoninstall.py\n'
        self.__srcguicopy['bat']+= 'copy pontongui.bat c:\ponton5hybrid\pontongui.bat\n'
        self.__srcguicopy['bat']+= 'copy pontonsrc.zip c:\ponton5hybrid\pontonsrc.zip\n'
        self.__srcguicopy['bat']+= 'copy Shortcut.exe c:\ponton5hybrid\Shortcut.exe\n'
        self.__srcguicopy['bat']+= 'cd c:\ponton5hybrid\n'
        self.__srcguicopy['bat']+= 'pontongui.bat\n'


    def GetPyVer(self):
        # get pythen version
        return(self.__pyver)


    def GetWxPyVer(self):
        # get wxpython version
        return(self.__wxpyver)


    def GetSourcesPy(self):
        # return dic with python sources
        return(self.__srcpython)


    def GetSourcesWxPy(self):
        # return dic with python sources
        return(self.__srcwxpython)


    def GetSourcesRestart(self):
        # return dic with python sources
        return(self.__srcrestart)


    def GetSourcesGuiCopy(self):
        # return dic with python sources
        return(self.__srcguicopy)


class PontonSysScript():
    # object to write system scripts

    def __init__(self):
        # end file of setupBASH.sh or setupWIN.bat
        self.__finishbash= 'pontonfinish.sh'
        self.__finishbat= 'pontonfinish.bat'


    def WriteFinishBash(self, bashtxt):
        try:
            txtfile= open(self.__finishbash, 'w')

            text=  '#!/bin/bash\n\n'
            text+= '# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.\n'
            text+= '# http://erasand.jimdo.com/python-programme/lorze/\n# (C) 2012, Andreas Ulrich\n\n'
            text+= '# This file is part of "LORZE erasandcad"\n\n'
            text+= '# "LORZE erasandcad" is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.\n\n'
            text+= '# "LORZE erasandcad" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\n'
            text+= '# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.\n\n'

            txtfile.write(text+ bashtxt)

            txtfile.close()

            # set all rights and executable for owener, group and others
            os.chmod(self.__finishbash, 0o777)

        except:
            print('An error was found while writing '+ self.__finishbash)


    def WriteFinishBat(self, battxt):
        try:
            txtfile= open(self.__finishbat, 'w')

            text=  '@echo off\n\n'
            text+= ':: LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.\n'
            text+= ':: http://erasand.jimdo.com/python-programme/lorze/\n# (C) 2012, Andreas Ulrich\n\n'
            text+= ':: This file is part of "LORZE erasandcad"\n\n'
            text+= ':: "LORZE erasandcad" is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.\n\n'
            text+= ':: "LORZE erasandcad" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\n'
            text+= ':: You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.\n\n'

            txtfile.write(text+ battxt)

            txtfile.close()

        except:
            print('An error was found while writing '+ self.__finishbat)


    def GetFinishBash(self):
        return(self.__finishbash)


    def GetFinishBat(self):
        return(self.__finishbat)


class PontonTest():
    # object to test the running system

    def __init__(self):
        # information about the system
        #~ self.__infofile= 'pontonDAT.txt'

        # information list, ('operating system', 'package system', 'architecture')

        # read system information
        self.__info= self.GetPlatform()


    def GetPlatform(self):
        # get os system / python 2.7 documentation with possible os: linux2, win32, cygwin, darwin, os2, os2emx, riscos, atheos / return tupel, ('operatingsystem', 'packagesystem', 'bit')

        archinfo= ('none', 'none', 'none')

        if sys.platform.startswith('linux'):
            # operating system linux
            if os.system('apt-get > /dev/null')== 0:
                # package system apt-get
                archinfo=('linux', 'apt-get', 'none')
            elif os.system('yum > /dev/null')== 0:
                # package system yum
                archinfo=('linux', 'yum', 'none')

        elif sys.platform.startswith('win'):
            # operating system windows
            if '64 bit' in sys.version:
                # 64 bit python
                archinfo=('win', 'none', '64')
            elif '32 bit' in sys.version:
                # 32 bit python
                archinfo=('win', 'none', '32')

        elif sys.platform.startswith('darwin'):
            # operating system mac os
            if '64 bit' in sys.version:
                # 64 bit python
                self.__archinfo=('mac', 'hdid', '64')
            elif '32 bit' in sys.version:
                # 32 bit python
                archinfo=('mac', 'hdid', '32')

        return(archinfo)


    def GetPyVer(self):
        # return string, 'x.y'
        pyver= sys.version
        pyver= pyver[:3]

        return(pyver)


    def GetWxPyVer(self):
        # return string, 'x.y', '0.0'= no wxpython found
        try:
            import wx
            wxpyver= wx.VERSION
        except:
            wxpyver= (0, 0)

        wxpyver= str(wxpyver[0])+ '.'+ str(wxpyver[1])

        return(wxpyver)


    def GetSystemInfo(self):
        # return tuple: ('python version', 'wxpython version', 'operating system', 'package system', 'bit architecture')
        return(self.GetPyVer(), self.GetWxPyVer(), self.__info[0], self.__info[1], self.__info[2])


class PontonInstall():
    # object to install

    def __init__(self):
        # PontonTest
        self.__test= PontonTest()

        # PontonRequired
        self.__req= PontonRequired()

        # PontonSysScript
        self.__script= PontonSysScript()

        # string for next task / 'wx_inst' for wxpython installation', 'lorze_inst' for LORZE installation, '' for abort installation, 'pywx_inst' for python & wxpython installation
        self.__todo= ''

        # analaize what is next to do
        self.WhatToDo()

        if self.__todo== 'wx_inst':
            # install required wxpython
            self.WxInst()
        elif self.__todo== 'lorze_inst':
            # install lorze
            self.LorzeInst()
        elif self.__todo== 'pywx_inst':
            # install required python & wxpython
            self.PyWxInst()
        else:
            # cancel
            self.Cancel()


    def WhatToDo(self):
        # anlize what is to do / get system infos and required infos
        pyver, wxver, ostype, pkgtype, archtype= self.__test.GetSystemInfo()
        reqpy= self.__req.GetPyVer()
        reqwx= self.__req.GetWxPyVer()

        # testing
        #~ reqpy, reqwx= '2.7', '2.8'
        #~ pyver, wxver= '2.7', '2.8' # should be lorze_inst
        #~ pyver, wxver= '3.2', '0.0' # should ask, result pywx_inst or ''
        #~ pyver, wxver= '2.9', '0.0' # should be wx_inst
        #~ pyver, wxver= '2.7', '2.6' # should ask, result wx_inst, lorze_inst or ''
        #~ pyver, wxver= '2.6', '2.2' # should ask, result pywx_inst, lorze_inst or ''
        #~ pyver, wxver= '2.5', '0.0' # should ask, result pywx_inst or ''


        if pyver== reqpy or pyver> reqpy:
            # running python version is equal or higher than required

            if wxver== reqwx or wxver> reqwx:
                # running wxpython version is equal or higher than required: install LORZE
                self.__todo= 'lorze_inst'

            elif wxver== '0.0' and pyver< '3':
                # no wxpython found: install wxpython
                self.__todo= 'wx_inst'

            elif wxver== '0.0' and pyver>= '3':
                # no wxpython found, python version is 3 or higher
                choice= ''
                while choice!= '1' and choice!= '2':
                    print('')
                    print('------------------------------------------------------------')
                    print('  Lorze does not work currently with python 3')
                    print('')
                    print('  Recommended version of Python is '+ reqpy)
                    print('  Recommended version of Wxpython is '+ reqwx)
                    print('')
                    print('  What do you prefer ?')
                    print('------------------------------------------------------------')
                    print('  [1] Install recommended versions')
                    print('  [2] installation cancel')
                    print('------------------------------------------------------------')
                    choice= raw_input('  Your choice .. ')
                if choice== '1':
                    # update to new version
                    self.__todo= 'pywx_inst'
                else:
                    # cancel
                    self.__todo= ''

            elif wxver< reqwx:
                # running wxpython version is lower than required / ak what to do
                choice= ''
                while choice!= '1' and choice!= '2' and choice!= '3':
                    print('')
                    print('------------------------------------------------------------')
                    print('  Installed version of Wxpython is '+ wxver)
                    print('')
                    print('  Recommended version of Wxpython is '+ reqwx)
                    print('')
                    print('  What do you prefer ?')
                    print('------------------------------------------------------------')
                    print('  [1] Update to recommended version')
                    print('  [2] Keep existing version and install LORZE')
                    print('  [3] installation cancel')
                    print('------------------------------------------------------------')
                    choice= raw_input('  Your choice .. ')
                if choice== '1':
                    # update to new version
                    self.__todo= 'wx_inst'
                elif choice== '2':
                    # keep existing version
                    self.__todo= 'lorze_inst'
                else:
                    # cancel
                    self.__todo= ''

        elif pyver< reqpy:
            # running python version is lower than required

            if wxver != '0.0':
                # a wxpython version is installed / ask what to do
                choice= ''
                while choice!= '1' and choice!= '2' and choice!= '3':
                    print('')
                    print('------------------------------------------------------------')
                    print('  Installed version of Python is '+ pyver)
                    print('  Installed version of Wxpython is '+ wxver)
                    print('')
                    print('  Recommended version of Python is '+ reqpy)
                    print('  Recommended version of Wxpython is '+ reqwx)
                    print('')
                    print('  What do you prefer ?')
                    print('------------------------------------------------------------')
                    print('  [1] Update to recommended versions')
                    print('  [2] Keep existing version and install LORZE')
                    print('  [3] installation cancel')
                    print('------------------------------------------------------------')
                    choice= raw_input('  Your choice .. ')
                if choice== '1':
                    # update to new version
                    self.__todo= 'pywx_inst'
                elif choice== '2':
                    # keep existing version
                    self.__todo= 'lorze_inst'
                else:
                    # cancel
                    self.__todo= ''

            else:
                # no wxpython is installed / ask what to do
                choice= ''
                while choice!= '1' and choice!= '2' and choice!= '3':
                    print('')
                    print('------------------------------------------------------------')
                    print('  Installed version of Python is '+ pyver)
                    print('  It has been found no version of wxpython.')
                    print('')
                    print('  Recommended version of Python is '+ reqpy)
                    print('  Recommended version of Wxpython is '+ reqwx)
                    print('')
                    print('  What do you prefer ?')
                    print('------------------------------------------------------------')
                    print('  [1] Update to recommended versions')
                    print('  [2] installation cancel')
                    print('------------------------------------------------------------')
                    choice= raw_input('  Your choice .. ')
                if choice== '1':
                    # update to new version
                    self.__todo= 'pywx_inst'
                elif choice== '2':
                    # cancel
                    self.__todo= ''


    def GetToDo(self):
        return(self.__todo)


    def WxInst(self):
        # Write install routine in finish for install wxpython / get system info
        pyver, wxver, ostype, pkgtype, archtype= self.__test.GetSystemInfo()

        # testing
        #~ ostype, pkgtype= 'linux', 'apt-get'
        #~ ostype, pkgtype= 'linux', 'yum'
        #~ ostype, archtype= 'win', '32'
        #~ ostype, archtype= 'win', '64'
        #~ ostype= 'mac'

        text= 'error'

        if ostype== 'linux':
            # linux operating system
            if pkgtype== 'apt-get':
                # package managment apt-get
                command= self.__req.GetSourcesWxPy()['apt-get']+ self.__req.GetSourcesRestart()['bash']
                self.__script.WriteFinishBash(command)
                text= 'Linux & Apt-Get'

            elif pkgtype== 'yum':
                # package managment apt-get
                command= self.__req.GetSourcesWxPy()['yum']+ self.__req.GetSourcesRestart()['bash']
                self.__script.WriteFinishBash(command)
                text= 'Linux & Yum'

        elif ostype== 'win':
            # windows operating system
            if archtype== '32':
                # 32 bit windows
                command= self.__req.GetSourcesWxPy()['win32']+ self.__req.GetSourcesRestart()['bat']
                self.__script.WriteFinishBat(command)
                text= 'Windows 32-bit'

            elif archtype== '64':
                # 32 bit windows
                command= self.__req.GetSourcesWxPy()['win64']+ self.__req.GetSourcesRestart()['bat']
                self.__script.WriteFinishBat(command)
                text= 'Windows 64-bit'

        elif ostype== 'mac':
            # windows operating system
            command= self.__req.GetSourcesWxPy()['hdid']+ self.__req.GetSourcesRestart()['bash']
            self.__script.WriteFinishBash(command)
            text= 'MAC OS X'

        print('------------------------------------------------------------')
        print('  Installing wxPython .. '+ text)
        print('------------------------------------------------------------')


    def LorzeInst(self):
        # Write start routine in finish for gui / get system info
        pyver, wxver, ostype, pkgtype, archtype= self.__test.GetSystemInfo()

        #testing
        #~ ostype= 'linux'
        #~ ostype= 'win'

        text= 'error'

        if ostype== 'linux':
            # linux operating system
            command= self.__req.GetSourcesGuiCopy()['bash']
            self.__script.WriteFinishBash(command)
            text= 'Linux'

        elif ostype== 'win':
            # windows operating system
            command= self.__req.GetSourcesGuiCopy()['bat']
            self.__script.WriteFinishBat(command)
            text= 'Windows'

        elif ostype== 'mac':
            # windows operating system
            command= self.__req.GetSourcesGuiCopy()['bash']
            self.__script.WriteFinishBash(command)
            text= 'Mac OS'

        print('------------------------------------------------------------')
        print('  Installing Lorze .. '+ text)
        print('------------------------------------------------------------')


    def PyWxInst(self):
        # Write install routine in finish for install python & wxpython / get system info
        pyver, wxver, ostype, pkgtype, archtype= self.__test.GetSystemInfo()

        # testing
        #~ ostype, pkgtype= 'linux', 'apt-get'
        #~ ostype, pkgtype= 'linux', 'yum'
        #~ ostype, archtype= 'win', '32'
        #~ ostype, archtype= 'win', '64'
        #~ ostype ,archtype= 'mac', '32'
        #~ ostype ,archtype= 'mac', '64'

        text= 'error'

        if ostype== 'linux':
            # linux operating system
            if pkgtype== 'apt-get':
                # package managment apt-get
                command= self.__req.GetSourcesPy()['apt-get']+ '\n'+ self.__req.GetSourcesWxPy()['apt-get']+ self.__req.GetSourcesRestart()['bash']
                self.__script.WriteFinishBash(command)
                text= 'Linux & Apt-Get'

            elif pkgtype== 'yum':
                # package managment apt-get
                command= self.__req.GetSourcesPy()['yum']+ '\n'+ self.__req.GetSourcesWxPy()['yum']+ self.__req.GetSourcesRestart()['bash']
                self.__script.WriteFinishBash(command)
                text= 'Linux & Yum'

        elif ostype== 'win':
            # windows operating system
            if archtype== '32':
                # 32 bit windows
                command= self.__req.GetSourcesPy()['win32']+ '\n'+ self.__req.GetSourcesWxPy()['win32']+ self.__req.GetSourcesRestart()['bat']
                self.__script.WriteFinishBat(command)
                text= 'Windows 32-bit'

            elif archtype== '64':
                # 32 bit windows
                command= self.__req.GetSourcesPy()['win64']+ '\n'+ self.__req.GetSourcesWxPy()['win64']+ self.__req.GetSourcesRestart()['bat']
                self.__script.WriteFinishBat(command)
                text= 'Windows 64-bit'

        elif ostype== 'mac':
            # windows operating system
            if archtype== '32':
                # mac os x ppc 10.3 - 10.6
                command= self.__req.GetSourcesPy()['hdid32']+ '\n'+ self.__req.GetSourcesWxPy()['hdid']+ self.__req.GetSourcesRestart()['bash']
                self.__script.WriteFinishBash(command)
                text= 'MAC OS X 32-bit'

            elif archtype== '64':
                # mac os x 10.6 - 10.7
                command= self.__req.GetSourcesPy()['hdid64']+ '\n'+ self.__req.GetSourcesWxPy()['hdid']+ self.__req.GetSourcesRestart()['bash']
                self.__script.WriteFinishBash(command)
                text= 'MAC OS X 64-bit'

        print('------------------------------------------------------------')
        print('  Installing Python & wxPython .. '+ text)
        print('------------------------------------------------------------')


    def Cancel(self):
        # Write only echo in finish
        self.__script.WriteFinishBash('echo ".."')
        self.__script.WriteFinishBat('echo ..')

        print('------------------------------------------------------------')
        print('  Cancel ..')
        print('------------------------------------------------------------')


if __name__== '__main__':
    # run: 'Install' for installation, 'Test' for testing
    run= 'Install'

    if run== 'Install':
        # install program
        inst= PontonInstall()

    elif run== 'Test':
        # test PontonInstall
        inst= PontonInstall()
        req= PontonRequired()
        script= PontonSysScript()
        test= PontonTest()
        data= PontonData()

        # menu
        choice= '0'
        while choice!= '':
            print('======================================================================')
            print('Test PontonInstall')
            print('======================================================================')
            print('[ENTER]: Exit')
            print('----------------------------------------------------------------------')
            print('01: PontonInstall GetToDo          04: PontonInstall Cancel')
            print('02: PontonInstall WxInst           05: PontonInstall LorzeInst')
            print('03: PontonInstall PyWxInst')
            print('----------------------------------------------------------------------')
            print('11: PontonRequired GetPyVer        14: PontonRequired GetSourcesWxPy')
            print('12: PontonRequired GetWxPyVer      15: PontonRequired GetSourcesGuiCopy')
            print('13: PontonRequired GetSourcesPy')
            print('----------------------------------------------------------------------')
            print('21: PontonSysScript GetFinishBash  23: PontonSysScript WriteFinishBash')
            print('22: PontonSysScript GetFinishBat   24: PontonSysScript WriteFinishBat')
            print('----------------------------------------------------------------------')
            print('31: PontonTest GetPlatform         33: PontonTest GetWxPyVer')
            print('32: PontonTest GetPyVer            34: PontonTest GetSystemInfo')
            print('----------------------------------------------------------------------')
            print('41: PontonData Get')
            print('----------------------------------------------------------------------')
            choice= raw_input('Your choice .. ')
            print('======================================================================')
            if choice== '01':
                print(' .GetToDo() ',inst.GetToDo())

            elif choice== '02':
                inst.WxInst()

            elif choice== '03':
                inst.PyWxInst()

            elif choice== '04':
                inst.Cancel()

            elif choice== '05':
                inst.LorzeInst()

            elif choice== '11':
                print(req.GetPyVer())

            elif choice== '12':
                print(req.GetWxPyVer())

            elif choice== '13':
                for key, cmd in req.GetSourcesPy().items():
                    print(key+ ': '+ cmd)

            elif choice== '14':
                for key, cmd in req.GetSourcesWxPy().items():
                    print(key+ ': '+ cmd)

            elif choice== '15':
                for key, cmd in req.GetSourcesGuiCopy().items():
                    print(key+ ': '+ cmd)

            elif choice== '21':
                print(script.GetFinishBash())

            elif choice== '22':
                print(script.GetFinishBat())

            elif choice== '23':
                script.WriteFinishBash('echo "Test .."')
                print('Run the File '+ script.GetFinishBash()+ ' in the terminal ..')

            elif choice== '24':
                script.WriteFinishBat('echo Test ..')
                print('Run the File '+ script.GetFinishBat()+ ' with a dos prompt ..')

            elif choice== '31':
                print(test.GetPlatform())

            elif choice== '32':
                print(test.GetPyVer())

            elif choice== '33':
                print(test.GetWxPyVer())

            elif choice== '34':
                sysinf= test.GetSystemInfo()
                print(sysinf)

            elif choice== '41':
                for i in ('pontonguititle', 'pontonguisize', 'installdlgsize', 'removedlgsize', 'listseldlgsize', 'textdlgsize', 'starterdlgsize', 'programtitle', 'programname', 'programpic', 'configdic', 'appversion', 'prevverions', 'cfgfindkey', 'cfgdirkey', 'uninstfile', 'desktops', 'startscript', 'startdir', 'startmodule', 'startername', 'pontonstartername', 'startercategory', 'pontonstartercategory', 'startercomment', 'pontonstartercomment', 'startericon'):
                    print("Get('"+ i+ "')", data.Get(i))

        print('======================================================================')
