#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.


class LorzeAttribute():
    # Element attribute helper object for LORZE erasandcad

    def __init__(self, options, log, valchck):
        # lorze options
        self.__options= options

        # log object
        self.__log= log

        # default attributes, text
        self.__drawdefcolor= ''
        self.__drawdeflayer= ''
        self.__drawdefstyle= ''
        self.__drawdefwidth= ''

        # global color dictionairys: color name= key / valuedict: 'black':'#000000' / labeldict: 'black':'dark'
        self.__colorvaluedict= {}
        self.__colorlabeldict= {}

        # global layer dictionairy: layer name= key / labeldict: 'demo':'info'
        self.__layerlabeldict= {}

        # global line style dictionairys: line style name= key / valuedict: 'dashed':('wx.USER_DASH', [40, 50, 40]) / labeldict: 'dashed':'dashed group A'
        self.__stylevaluedict= {}
        self.__stylelabeldict= {}

        # global line width dictionairys: line width name= key / valuedict:  '0,25 mm':0.25 / labeldict: '0,25 mm':Rotring white'
        self.__widthvaluedict= {}
        self.__widthlabeldict= {}

        self.__valchk= valchck

        self.AddColorEntry(self.__options.GetDrawDefColor()[0], self.__options.GetDrawDefColor()[1], self.__options.GetDrawDefColor()[2])
        self.AddLayerEntry(self.__options.GetDrawDefLayer()[0], self.__options.GetDrawDefLayer()[1])
        self.AddStyleEntry(self.__options.GetDrawDefStyle()[0], self.__options.GetDrawDefStyle()[1], self.__options.GetDrawDefStyle()[2])
        self.AddWidthEntry(self.__options.GetDrawDefWidth()[0], self.__options.GetDrawDefWidth()[1], self.__options.GetDrawDefWidth()[2])

        self.SetDefaultColor(self.__options.GetDrawDefColor()[0])
        self.SetDefaultLayer(self.__options.GetDrawDefLayer()[0])
        self.SetDefaultStyle(self.__options.GetDrawDefStyle()[0])
        self.SetDefaultWidth(self.__options.GetDrawDefWidth()[0])


    def SetDefaultColor(self, color):
        # set default color, string for lorzelog
        logtext= 'LorzeAttribute.SetDefaultColor('+ color+ ')'

        if color in self.__colorvaluedict.keys():
            self.__log.AddEntry('OK|'+ logtext+ '|Old= '+ self.__drawdefcolor)
            self.__drawdefcolor= color
        else:
            self.__log.AddEntry('FAIL|colour not in dictionairy|'+ logtext)


    def SetDefaultLayer(self, layer):
        # set default layer, string for lorzelog
        logtext= 'LorzeAttribute.SetDefaultLayer('+ layer+ ')'

        if layer in self.__layerlabeldict.keys():
            self.__log.AddEntry('OK|'+ logtext+ '|Old= '+ self.__drawdeflayer)
            self.__drawdeflayer= layer
        else:
            self.__log.AddEntry('FAIL|layer not in dictionairy|'+ logtext)


    def SetDefaultStyle(self, style):
        # set default line style, string for lorzelog
        logtext= 'LorzeAttribute.SetDefaultStyle('+ style+ ')'

        if style in self.__stylevaluedict.keys():
            self.__log.AddEntry('OK|'+ logtext+ '|Old= '+ self.__drawdefstyle)
            self.__drawdefstyle= style
        else:
            self.__log.AddEntry('FAIL|line style not in dictionairy|'+ logtext)


    def SetDefaultWidth(self, width):
        # set default line width, string for lorzelog
        logtext= 'LorzeAttribute.SetDefaultWidth('+ width+ ')'

        if width in self.__widthvaluedict.keys():
            self.__log.AddEntry('OK|'+ logtext+ '|Old= '+ self.__drawdefwidth)
            self.__drawdefwidth= width
        else:
            self.__log.AddEntry('FAIL|line width not in dictionairy|'+ logtext)


    def GetDefaultColor(self):
        return(self.__drawdefcolor)


    def GetDefaultLayer(self):
        return(self.__drawdeflayer)


    def GetDefaultStyle(self):
        return(self.__drawdefstyle)


    def GetDefaultWidth(self):
        return(self.__drawdefwidth)


    def AddColorEntry(self, name, value, label):
        # add new colour to colour dictionairy, name= text, value= html color, label= text, string for lorzelog
        logtext= 'LorzeAttribute.AddColorEntry('+ name+ ', '+ value+ ', '+ label+ ')'

        if name not in self.__colorvaluedict:
            self.__colorvaluedict[name]= self.__valchk.RGBtxtToHTMLcol(value)
            self.__colorlabeldict[name]= label
            self.__log.AddEntry('OK|'+ logtext)
        else:
            self.__log.AddEntry('FAIL|colour entry already exist|'+ logtext)


    def AddLayerEntry(self, name, label):
        # add new layer to colourdictionairy, name= text, label= text, string for lorzelog
        logtext= 'LorzeAttribute.AddLayerEntry('+ name+ ', '+ label+ ')'

        if name not in self.__layerlabeldict:
            self.__layerlabeldict[name]= label
            self.__log.AddEntry('OK|'+ logtext)
        else:
            self.__log.AddEntry('FAIL|layer entry already exist|'+ logtext)


    def AddStyleEntry(self, name, value, label):
        # add new layer to colourdictionairy, name= text, value= text, convert to tupel ('wx.SOLID', []), label= text, string for lorzelog
        logtext= 'LorzeAttribute.AddStyleEntry('+ name+ ', '+ value+ ', '+ label+ ')'

        if name not in self.__stylevaluedict:
            self.__stylevaluedict[name]= self.__valchk.LorzeTextToWxStyle(value)
            self.__stylelabeldict[name]= label
            self.__log.AddEntry('OK|'+ logtext)
        else:
            self.__log.AddEntry('FAIL|line style entry already exist|'+ logtext)


    def AddWidthEntry(self, name, value, label):
        # add new layer to colourdictionairy, name= text, value= text, convert to float, label= text, string for lorzelog
        logtext= 'LorzeAttribute.AddWidthEntry('+ name+ ', '+ value+ ', '+ label+ ')'

        if name not in self.__widthvaluedict:
            self.__widthvaluedict[name]= float(value)
            self.__widthlabeldict[name]= label
            self.__log.AddEntry('OK|'+ logtext)
        else:
            self.__log.AddEntry('FAIL|line width entry already exist|'+ logtext)


    def GetColorEntry(self, name):
        # return colour values from colour dictionairy, value, label, convert value to RGB Text before return
        if name in self.__colorvaluedict:
            return(self.__valchk.HTMLcolToRGBtxt(self.__colorvaluedict[name]), self.__colorlabeldict[name])
        else:
            return(None, None)


    def GetLayerEntry(self, name):
        # return layer values from layer dictionairy, label
        if name in self.__layerlabeldict:
            return(self.__layerlabeldict[name])
        else:
            return(None)


    def GetStyleEntry(self, name):
        # return style values from style dictionairy, value, label, convert value to Lorze text before return
        if name in self.__stylevaluedict:
            return(self.__valchk.WxStyleToLorzeText(self.__stylevaluedict[name]), self.__stylelabeldict[name])
        else:
            return(None, None)


    def GetWidthEntry(self, name):
        # return width values from width dictionairy, value, label, convert value to text before return
        if name in self.__widthvaluedict:
            return(str(self.__widthvaluedict[name]), self.__widthlabeldict[name])
        else:
            return(None, None)


    def GetAllAttrNames(self):
        # return all attribute names, colors, layers, styles, widths
        return(self.__colorvaluedict.keys(), self.__layerlabeldict.keys(), self.__stylevaluedict.keys(), self.__widthvaluedict.keys())


    def AmountToList(self, amount):
        # convert amount in to list, define list
        alist= []

        # get from amounts
        for i in amount:
            alist.append(i)

        # return list
        return(alist)


    def GetWxColor(self, color):
        if color in self.__colorvaluedict:
            return(self.__colorvaluedict[color])
        else:
            return(self.__drawdefcolor)


    def GetWxStyle(self, style):
        if style in self.__stylevaluedict:
            return(self.__stylevaluedict[style])
        else:
            return(self.__drawdefstyle)


    def GetWxWidth(self, width):
        if width in self.__widthvaluedict:
            return(self.__widthvaluedict[width])
        else:
            return(self.__drawdefwidth)


if __name__== '__main__':
    from lrzoptions import LorzeOptions
    from lrzlog import LorzeLog
    from lrzvalchck import LorzeValueCheck

    options= LorzeOptions()
    log= LorzeLog()
    valchck= LorzeValueCheck()
    attr= LorzeAttribute(options, log, valchck)

    print('')
    print('GetDefaultColor', attr.GetDefaultColor())
    print('GetDefaultLayer', attr.GetDefaultLayer())
    print('GetDefaultStyle', attr.GetDefaultStyle())
    print('GetDefaultWidth', attr.GetDefaultWidth())

    color= [('Red', '255, 0, 0', 'base color'), ('Blue', '0, 0, 255', 'base color'), ('Green', '0, 128, 0', 'base color'), ('Yellow', '255, 255, 0', 'base color'), ('Gray', '128, 128, 128', 'base color'), ('White', '0, 0, 0', 'base color')]
    layer= [('Layer 1', 'default'), ('Layer 2', 'default'), ('Layer 3', 'default'), ('Layer 4', 'default'), ('Layer 5', 'default'), ('Layer 6', 'default')]
    style= [('Solid', 'SOLID', 'standard'), ('Dot', 'DOT', 'standard'), ('Long dash', 'LONG_DASH', 'standard'), ('Short dash', 'SHORT_DASH', 'standard'), ('Dot dash', 'DOT_DASH', 'standard'), ('Axis 13mm', '4, 2, 1, 2, 4', 'standard')]
    width= [('0.13mm', '0.13', 'Rotring'), ('0.18mm', '0.18', 'Rotring'), ('0.25mm', '0.25', 'Rotring'), ('0.35mm', '0.35', 'Rotring'), ('0.50mm', '0.50', 'Rotring'), ('0.70mm', '0.70', 'Rotring')]
    for i in color:
        attr.AddColorEntry(i[0], i[1], i[2])
    for i in layer:
        attr.AddLayerEntry(i[0], i[1])
    for i in style:
        attr.AddStyleEntry(i[0], i[1], i[2])
    for i in width:
        attr.AddWidthEntry(i[0], i[1], i[2])

    print('')
    print('GetAllAttrNames')
    for i in attr.GetAllAttrNames():
        print(i)
    print('')
    print('Trying to add again Green, Layer 6, Dot dash, 0.7mm / visit the LorzeLog')
    attr.AddColorEntry(color[2][0], color[2][1], color[2][2])
    attr.AddLayerEntry(layer[5][0], layer[5][1])
    attr.AddStyleEntry(style[5][0], style[5][1], style[5][2])
    attr.AddWidthEntry(width[5][0], width[5][1], width[5][2])
    print('')
    print('SetDefault Attributes to Red, Layer 1, Axis 13mm, 0.25mm')
    attr.SetDefaultColor('Red')
    attr.SetDefaultLayer('Layer 1')
    attr.SetDefaultStyle('Axis 13mm')
    attr.SetDefaultWidth('0.25mm')
    print('GetDefaultColor', attr.GetDefaultColor())
    print('GetDefaultLayer', attr.GetDefaultLayer())
    print('GetDefaultStyle', attr.GetDefaultStyle())
    print('GetDefaultWidth', attr.GetDefaultWidth())
    print('')
    print('SetDefault Attributes to error, error, error, error')
    attr.SetDefaultColor('error')
    attr.SetDefaultLayer('error')
    attr.SetDefaultStyle('error')
    attr.SetDefaultWidth('error')
    print('GetDefaultColor', attr.GetDefaultColor())
    print('GetDefaultLayer', attr.GetDefaultLayer())
    print('GetDefaultStyle', attr.GetDefaultStyle())
    print('GetDefaultWidth', attr.GetDefaultWidth())
    print('')
    print('GetColorEntry Blue', attr.GetColorEntry('Blue'))
    print('GetLayerEntry Layer 3', attr.GetLayerEntry('Layer 3'))
    print('GetStyleEntry Long dash', attr.GetStyleEntry('Long dash'))
    print('GetWidthEntry 0.50mm', attr.GetWidthEntry('0.50mm'))
    print('')
    print('GetColorEntry error', attr.GetColorEntry('error'))
    print('GetLayerEntry error', attr.GetLayerEntry('error'))
    print('GetStyleEntry error', attr.GetStyleEntry('error'))
    print('GetWidthEntry error', attr.GetWidthEntry('error'))
    print('')
    print('GetWxColor Gray', attr.GetWxColor('Gray'))
    print('GetWxStyle Short dash', attr.GetWxStyle('Short dash'))
    print('GetWxWidth 0.35mm)', attr.GetWxWidth('0.35mm'))
    print('GetWxColor error', attr.GetWxColor('error'))
    print('GetWxStyle error', attr.GetWxStyle('error'))
    print('GetWxWidth error)', attr.GetWxWidth('error'))
    print('LorzeLog')
    for i in log.GetLog():
        print(i)
