#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of "LORZE erasandcad"

# "LORZE erasandcad" is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# "LORZE erasandcad" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx, locale
from lrzdlgoptions import SortedListCtrl

# name for translations texts
_= wx.GetTranslation


class LorzeDlgAttribut(wx.Dialog):
    #Lorze Atrtribut Dialog to the GUI of LORZE erasandcad

    def __init__(self, parent, options, valchck, dlghelper, gdipanel):
        # parent class
        self.__parent= parent

        # LorzeOptions() of parent class
        self.__options= options

        # get settings for dialog from options
        dlgborder= self.__options.GetDlgBorder()
        wdlg, hdlg= self.DialogueSize()

        # subclass
        wx.Dialog.__init__(self, self.__parent, wx.ID_ANY, _(u'Attributes'), size= (wdlg, hdlg), style= wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)

        # LorzeValueCheck
        self.__valchck= valchck

        # Dialog Helper
        self.__dlg= dlghelper

        # GdiPanel
        self.__gdi= gdipanel

        # default drawing list, label & combobox
        self.__drawlist= []
        self.__drawcombo= wx.ComboBox(self, size= (-1, -1), choices= self.__drawlist, style=wx.CB_READONLY)

        # info texts
        self.__colortxt= wx.StaticText(self)
        self.__layertxt= wx.StaticText(self)
        self.__styletxt= wx.StaticText(self)
        self.__widthtxt= wx.StaticText(self)

        colortitle= wx.StaticText(self, label=_(u'Colour'))
        layertitle= wx.StaticText(self, label=_(u'Layer'))
        styletitle= wx.StaticText(self, label=_(u'Line type'))
        widthtitle= wx.StaticText(self, label=_(u'Line width'))

        # boxes for infotexts, attribute list, options, default drawing
        infobox= wx.StaticBox(self, label= _(u'Chosen attributes'))
        optionbox= wx.StaticBox(self, label= _(u'Filter options'))
        drawbox= wx.StaticBox(self, label= _(u'Drawings'))

        # notebook
        nbook= wx.Notebook(self)
        widths= self.__options.GetAttrListWidths()

        # panels
        self.__color= LorzeAttrPanel(nbook, dlgborder, 'color', (widths[0], widths[1], widths[2], widths[3]))
        self.__layer= LorzeAttrPanel(nbook, dlgborder, 'layer', (widths[4], widths[5], widths[6], widths[7]))
        self.__style= LorzeAttrPanel(nbook, dlgborder, 'style', (widths[8], widths[9], widths[10], widths[11]))
        self.__width= LorzeAttrPanel(nbook, dlgborder, 'width', (widths[12], widths[13], widths[14], widths[15]))

        nbook.AddPage(self.__color, _(u'Colour'))
        nbook.AddPage(self.__layer, _(u'Layer'))
        nbook.AddPage(self.__style, _(u'Line type'))
        nbook.AddPage(self.__width, _(u'Line width'))

        # option: from actual drawing, from all drawings, from defaults, from selection
        self.__optdrawings= wx.RadioButton(self, label= _(u'Drawings'), style=wx.RB_GROUP)
        self.__optselected= wx.RadioButton(self, label= _(u'Selected'))
        self.__optdefault= wx.RadioButton(self, label= _(u'Defaults'))
        self.__optdictionairy= wx.RadioButton(self, label= _(u'Dictionairy'))

        # dialog control buttons
        buttonok= wx.Button(self, label= _(u'OK'))
        buttoncancel= wx.Button(self, label= _(u'Cancel'))

        buttonok.Bind(wx.EVT_BUTTON, self.OnOk)
        buttoncancel.Bind(wx.EVT_BUTTON, self.OnCancel)
        self.Bind(wx.EVT_CLOSE, self.OnDialogueClose)

        # layout
        vbox= wx.BoxSizer(wx.VERTICAL)

        # drawing
        drawsizer= wx.StaticBoxSizer(drawbox, wx.VERTICAL)
        drawsizer.Add(self.__drawcombo, 0, wx.ALL, dlgborder)
        vbox.Add(drawsizer, 0, wx.EXPAND|wx.ALL, dlgborder)

        # infotexts
        infosizer= wx.StaticBoxSizer(infobox, wx.VERTICAL)
        infogrid= wx.GridSizer(2, 4, 0, dlgborder)
        infogrid.AddMany([(colortitle, 0, wx.EXPAND), (layertitle, 0, wx.EXPAND), (styletitle, 0, wx.EXPAND), (widthtitle, 0, wx.EXPAND), (self.__colortxt, 0, wx.EXPAND), (self.__layertxt, 0, wx.EXPAND), (self.__styletxt, 0, wx.EXPAND), (self.__widthtxt, 0, wx.EXPAND)])
        infosizer.Add(infogrid, 0, wx.EXPAND)
        vbox.Add(infosizer, 0, wx.EXPAND|wx.ALL, dlgborder)

        vbox.Add(nbook, 2, wx.EXPAND|wx.ALL, dlgborder)

        # filter options
        optionsizer= wx.StaticBoxSizer(optionbox, wx.VERTICAL)
        optiongrid= wx.GridSizer(1, 4, dlgborder, dlgborder)
        optiongrid.AddMany([(self.__optdrawings, 0, wx.EXPAND), (self.__optselected, 0, wx.EXPAND), (self.__optdefault, 0, wx.EXPAND), (self.__optdictionairy, 0, wx.EXPAND)])
        optionsizer.Add(optiongrid, 0, wx.EXPAND)
        vbox.Add(optionsizer, 0, wx.EXPAND|wx.ALL, dlgborder)

        # dialog buttons
        hbox2= wx.BoxSizer(wx.HORIZONTAL)
        hbox2.Add(buttonok, 0, wx.ALL, dlgborder)
        hbox2.Add(buttoncancel, 0, wx.ALL, dlgborder)
        vbox.Add(hbox2, 0, wx.EXPAND|wx.ALL, dlgborder)

        self.SetSizer(vbox)
        self.Centre()


    def OnDialogueClose(self, event):
        # last commando before close the dialogue
        dlgw, dlgh= self.GetSize()
        self.__options.SetDlgAttrSize(dlgw, dlgh)

        w0, w1, w2, w3= self.__color.GetWidths()
        w4, w5, w6, w7= self.__layer.GetWidths()
        w8, w9, w10, w11= self.__style.GetWidths()
        w12, w13, w14, w15= self.__width.GetWidths()
        self.__options.SetAttrListWidths(w0, w1, w2, w3, w4, w5, w6, w7, w8, w9, w10, w11, w12, w13, w14, w15)

        event.Skip()


    def OnActDraw(self, event):
        # attributes from actual drawing
        pass


    def OnAllDraw(self, event):
        # attributes from all drawings
        pass


    def OnSelected(self, event):
        # attributes from selected elements
        pass


    def OnDefault(self, event):
        # attributes from defaults
        pass


    def OnOk(self, event):
        # ok clicked, get all attributes
        pass


    def OnCancel(self, event):
        # cancel clicked, leave without changes
        pass


    def DialogueSize(self):
        wdlg, hdlg= self.__options.GetDlgAttrSize()

        # get size from parent
        wparent, hparent= self.__parent.GetSize()

        # dialog size control
        if hdlg> hparent:
            # correct height to parent size
            hdlg= hparent

        if wdlg> wparent:
            # correct width to parent size
            wdlg= wparent

        return(wdlg, hdlg)


class LorzeAttrPanel(wx.Panel):
    # Lorze attribut panel for the attribut dialog

    def __init__(self, parent, dlgborder, title, widths):
        # dlgborder is the space between the widgets / title can be 'color', 'layer', 'style' or 'width' / widths is a tuple (w0, w1, w2, w3)= column with for the list control / parent class
        self.__parent= parent
        self.__title= title

        # subclass
        wx.Panel.__init__(self, self.__parent)

        titledict= {'color':_(u'Colour'), 'layer':_(u'Layer'), 'style':_(u'Line type'), 'width':_(u'Line width')}

        # sorted list conntrol
        self.__list= SortedListCtrl(self)
        self.__list.InsertColumn(0, titledict[self.__title]+ _(u' name'), width= widths[0])
        self.__list.InsertColumn(1, _(u'Description'), width= widths[1])
        self.__list.InsertColumn(2, titledict[self.__title]+ _(u' value'), width= widths[2])
        self.__list.InsertColumn(3, _(u' Used by'), width= widths[3])

        # preview panel
        self.__prev= wx.Panel(self)

        # buttons
        new= wx.Button(self, label= _(u'New entry'))

        # bindings
        self.__list.Bind(wx.EVT_LISTBOX_DCLICK, self.OnNew)
        new.Bind(wx.EVT_BUTTON, self.OnNew)

        # layout
        vbox= wx.BoxSizer(wx.VERTICAL)
        hbox= wx.BoxSizer(wx.HORIZONTAL)

        hbox.Add(self.__prev, 1, wx.EXPAND|wx.ALL, dlgborder)
        hbox.Add(new, 0, wx.EXPAND|wx.ALL, dlgborder)

        #~ grid=wx.GridSizer(1, 3, dlgborder, dlgborder)
        #~ grid.AddMany([(), (edit, 0, wx.EXPAND), (delete, 0, wx.EXPAND)])

        vbox.Add(self.__list, 2, wx.EXPAND|wx.ALL, dlgborder)
        vbox.Add(hbox, 0, wx.EXPAND|wx.ALL, dlgborder)
        #~ vbox.Add(grid, 0, wx.EXPAND|wx.ALL, dlgborder)

        self.SetSizer(vbox)


    def OnNew(self, event):
        # get a new entry
        pass


    def GetWidths(self):
        return(self.__list.GetColumnWidth(0), self.__list.GetColumnWidth(1), self.__list.GetColumnWidth(2), self.__list.GetColumnWidth(3))


    def GetSortedListDict(self):
        # Returns a dictionairy for SortedListCtrl
        sortedlistdict= {}
        #~ index= 0
        #~ for i in self.__orderlist:
            #~ sortedlistdict[index]= (self.__labeldict[i], self.__valuedict[i])
            #~ index+= 1
        return(sortedlistdict)


        #~ elif seloption== 'drawdefcolour':
            #~ # get color as html string from color selection dialogue
            #~ colour= self.__dlg.ColorDialog(self.__valuedict['drawdefcolour'])
            #~ if colour!= '':
                #~ # clicked ok, get colour html string
                #~ self.__valuedict['drawdefcolour']= colour
#~
        #~ elif seloption== 'drawdeflayer':
            #~ # get text from text entry dialogue
            #~ text= self.__dlg.TextDialog(self.__labeldict['drawdeflayer'], _(u'Please give the name of the default layer'), self.__valuedict['drawdeflayer'])
            #~ if text!= '':
                #~ # clicked ok, get string
                #~ self.__valuedict['drawdeflayer']= text
#~
        #~ elif seloption== 'drawdefstyle':
            #~ # read penstyle and user definition, style1= wx.PenStyle, style2= user definied list
            #~ stylelist= self.__valuedict['drawdefstyle'].split('/')
            #~ if len(stylelist)== 2:
                #~ style1= stylelist[0].lstrip().rstrip()
                #~ style2= self.__valchck.StringToList(stylelist[1], ',', False)
            #~ else:
                #~ style1= stylelist[0].lstrip().rstrip()
                #~ style2= []
#~
            #~ # get string from list selection dialogue
            #~ text= self.__dlg.ListSelection(self.__labeldict['drawdefstyle'], self.__penstyledict.keys(), style1)
            #~ if text!= '':
                #~ # clicked ok
                #~ if text== 'wx.USER_DASH':
                    #~ # user definied line types, get list for style, set style2 as default
                    #~ styletext= self.__dlg.TextDialog(_(u'User definied line type'), _(u'Please give the values for the style defnition, for example 5, 1, 5)'), self.__valchck.ListToString(style2))
                    #~ if styletext!= '':
                        #~ # clicked ok, get list of integers
                        #~ stylelist= self.__valchck.StringToList(styletext, ',', False)
                        #~ text2= ' / '+ self.__valchck.ListToString(stylelist)
                    #~ else:
                        #~ text2= ' / 10, 5'
                #~ else:
                    #~ text2= ''
#~
                #~ self.__valuedict['drawdefstyle']= text+ text2
#~
        #~ elif seloption== 'drawdefsize':
            #~ # get text from text entry dialogue
            #~ text= self.__dlg.TextDialog(self.__labeldict['drawdefsize'], _(u'Please give the default pen strength'), self.__valuedict['drawdefsize'])
            #~ if text!= '':
                #~ # clicked ok, get float
                #~ self.__valuedict['drawdefsize']= self.__valchck.Float(text, str(self.__options.GetSmartCursMarkPenSize()), 0)


class TestFrame(wx.Frame):
    # object for testing
    def __init__(self):
        wx.Frame.__init__(self, None, title= u'Testcode LorzeDlgAttribut', size= (600, 600))

        self.__options= LorzeOptions()
        self.__valchck= LorzeValueCheck()
        self.__dlg= LorzeDlgHelper(self)
        self.__geohelp= LorzeGeoHelp(self.__options)
        self.__log= LorzeLog()
        self.__attribute= LorzeAttribute(self.__options, self.__log, self.__valchck)
        self.__drawman= LorzeDrawManager(self.__attribute, self.__log, self.__geohelp)
        self.__selection= LorzeSelection(self.__drawman, self.__geohelp)
        self.__gdi= GraphicPanel(self, self.__options, self.__geohelp, self.__log, self.__valchck, self.__attribute, self.__drawman, self.__selection)

        buttontest= wx.Button(self, label= u'Test')
        buttonexit= wx.Button(self, label= u'Exit')

        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(buttontest)
        vbox.Add(buttonexit)
        self.SetSizer(vbox)

        buttontest.Bind(wx.EVT_BUTTON, self.OnTest)
        buttonexit.Bind(wx.EVT_BUTTON, self.OnExit)

        self.Center()
        self.Show()


    def OnTest(self, event):
        dlg= LorzeDlgAttribut(self, self.__options, self.__valchck, self.__dlg, self.__gdi)
        dlg.ShowModal()
        dlg.Destroy()


    def OnExit(self, event):
        self.Close()


    def GetGdiSize(self):
        # dummy for the method, return framesize
        return(self.GetSize())


if __name__== '__main__':
    from lrzoptions import LorzeOptions
    from lrzselect import LorzeSelection
    from lrzgeohelp import LorzeGeoHelp
    from lrzdrawing import LorzeDrawing
    from lrzlog import LorzeLog
    from lrzvalchck import LorzeValueCheck
    from lrzattribute import LorzeAttribute
    from lrzdrawmanager import LorzeDrawManager
    from lrzdlghelper import LorzeDlgHelper
    from lrzgdipnl import GraphicPanel

    app= wx.App()

    # internationalization
    wxloc= wx.Locale()
    wxloc.AddCatalogLookupPathPrefix('./in18')
    # get system language ('xx_XX', 'CHARSET')
    wxlang= locale.getdefaultlocale()
    wxlang= wxlang[0][:2]
    # select translations
    if wxlang== 'de':
        wxloc.AddCatalog('lorze_de')

    frame= TestFrame()
    app.MainLoop()

