#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx, locale
import wx.html as html

# name for translations texts
_= wx.GetTranslation


class LorzeHelp(wx.Dialog):
    # Lorze help dialogue

    def __init__(self, parent, options):
        # parent class
        self.__parent= parent

        # LorzeOptions() of parent class
        self.__options= options

        # get settings for dialog from options
        dlgborder= self.__options.GetDlgBorder()
        wdlg, hdlg= self.DialogueSize()

        # subclass
        wx.Dialog.__init__(self, self.__parent, wx.ID_ANY, _(u'LORZE erasandcad HELP'), size= (wdlg, hdlg), style= wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)

        # help window
        htmlhelp= html.HtmlWindow(self, wx.ID_ANY, style=wx.NO_BORDER)

        # select language
        wxlang= locale.getdefaultlocale()
        wxlang= wxlang[0][:2]
        if wxlang== 'de':
            htmlhelp.LoadFile('../Documentation/HTML/DE_hilfe.html')
        else:
            htmlhelp.LoadPage('../Documentation/HTML/EN_help.html')

        # buttons
        buttonok= wx.Button(self, label= _(u'OK'))

        # dialog bindings
        buttonok.Bind(wx.EVT_BUTTON, self.OnExit)
        self.Bind(wx.EVT_CLOSE, self.OnDialogueClose)

        # layout
        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(htmlhelp, 2, wx.EXPAND|wx.ALL, dlgborder)
        vbox.Add(buttonok, 0, wx.ALIGN_RIGHT|wx.ALL, dlgborder)

        self.SetSizer(vbox)
        self.Centre()


    def DialogueSize(self):
        wdlg, hdlg= self.__options.GetDlgHelpSize()

        # get size from parent
        wparent, hparent= self.__parent.GetSize()

        # dialog size control
        if hdlg> hparent:
            # correct height to parent size
            hdlg= hparent

        if wdlg> wparent:
            # correct width to parent size
            wdlg= wparent

        return(wdlg, hdlg)


    def OnDialogueClose(self, event):
        # last commando before close the dialogue, read dialogue size
        wdlg, hdlg= self.GetSize()

        # set sizes to options
        self.__options.SetDlgHelpSize(wdlg, hdlg)

        event.Skip()


    def OnExit(self, event):
        self.Close()


class TestFrame(wx.Frame):
    # object for testing
    def __init__(self):
        wx.Frame.__init__(self, None, title= u'Test Dialog Load/ Save Object')

        self.__options= LorzeOptions()

        buttontest= wx.Button(self, label= u'Help')
        buttonexit= wx.Button(self, label= u'Exit')
        buttontest.Bind(wx.EVT_BUTTON, self.OnTest)
        buttonexit.Bind(wx.EVT_BUTTON, self.OnExit)

        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(buttontest)
        vbox.Add(buttonexit)

        self.SetSizer(vbox)
        self.Center()
        self.Show()


    def OnTest(self, event):
        dlg= LorzeHelp(self, self.__options)
        dlg.ShowModal()
        dlg.Destroy()


    def OnExit(self, event):
        self.Close()


if __name__== '__main__':
    from lrzoptions import LorzeOptions

    app= wx.App()

    # internationalization
    wxloc= wx.Locale()
    wxloc.AddCatalogLookupPathPrefix('./in18')
    # get system language ('xx_XX', 'CHARSET')
    wxlang= locale.getdefaultlocale()
    wxlang= wxlang[0][:2]
    # select translations
    if wxlang== 'de':
        wxloc.AddCatalog('lorze_de')

    frame= TestFrame()
    app.MainLoop()
