#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx, os, pickle, locale

# name for translations texts
_= wx.GetTranslation


class LorzeDlgHelper():
    # standard dialogues for LORZE ersandcad

    def __init__(self, parent):
        # parent
        self.__parent= parent

    def OpenDialog(self, label, filedir, filename, wildcard):
        # dialog for open files, show dialog, return path or '', parent: wx.Frame,  label: string, title of dialog, filedir & filename: strings, default filename and directory, wildcard: string, at example: 'Text (*.txt)|*.txt|All (*.*)|*.*'
        dlg= wx.FileDialog(self.__parent, label, filedir, filename, wildcard, wx.FD_OPEN)

        if dlg.ShowModal()== wx.ID_OK:
            # clicked ok, set path, destroy dialog, return path
            path= dlg.GetPath()
            dlg.Destroy()
            return(path)

        else:
            # not clicked ok, destroy dialog, return ''
            dlg.Destroy()
            return('')


    def SaveDialog(self, label, filedir, filename, wildcard, defext):
        # dialog for save files with extension & overwrite control, show dialog, return path or '', parent: wx.Frame, label: string, title of dialog, filedir & filename: strings, default filename and directory, wildcard: string, at example: 'Text (*.txt)|*.txt|All (*.*)|*.*', defext: string, extension must have, at example '.txt'
        dlg= wx.FileDialog(self.__parent, label, filedir,filename, wildcard, wx.FD_SAVE)

        if dlg.ShowModal()== wx.ID_OK:
            # clicked ok, set path, destroy dialog
            path= dlg.GetPath()
            dlg.Destroy()

            if path.endswith(defext)== False:
                # check extension, correct extension, syntax for the os
                path= os.path.normpath(path+ defext)

            if os.path.exists(path):
                # file already exist, overwrite?, show dialog and get answer
                question= wx.MessageDialog(None, _(u'The file already exists: ')+ path+ _(u'. Do you want to overwrite ?'), _(u'Confirmation'), wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
                answer= question.ShowModal()

                if answer== wx.ID_NO:
                    # do not overwrite, return ''
                    return('')
                else:
                    # overwrite, return path
                    return(path)

            # file dont exist, all ok
            else:
                return(path)

        else:
            # not clicked ok, destroy dialog
            dlg.Destroy()
            return('')


    def ColorDialog(self, default):
        # dialog to select color, return dialog in html format or '',  default= default color in html syntax, default color
        defaultdata= wx.ColourData()
        defaultdata.SetColour(default)

        # dialog
        dlg = wx.ColourDialog(self.__parent, data= defaultdata)
        dlg.GetColourData().SetChooseFull(True)

        # show dialog
        if dlg.ShowModal() == wx.ID_OK:
            # clicked ok
            data = dlg.GetColourData()
            return(data.GetColour().GetAsString(wx.C2S_HTML_SYNTAX))
        else:
            # not clicked ok
            return('')


    def TextDialog(self, titel, label, default):
        # dialog to entry text, return string or '', title= titel of the dialogue, label= description for the text input, default= detaulf text
        dlg= wx.TextEntryDialog(self.__parent, label, titel)
        dlg.SetValue(default)

        if dlg.ShowModal() == wx.ID_OK:
            # clicked ok
            text= dlg.GetValue()
        else:
            # not clicked ok
            text= ''

        dlg.Destroy()
        return(text)


    def ListSelection(self, titel, choicelist, default):
        # show combobox dialogue
        dlg= DlgListSel(self.__parent, titel, choicelist, default)
        dlg.ShowModal()

        # get choice, close dialogue, return choice, '' for cancel
        choice= dlg.GetChoice()
        dlg.Destroy()
        return(choice)


class DlgListSel(wx.Dialog):
    # dialog with a listbox choice

    def __init__(self, parent, titel, choicelist, default):
        # subclass
        wx.Dialog.__init__(self, parent, wx.ID_ANY, titel, size=(200, 300), style= wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)

        # choicelist and choiceindex
        self.__choicelist= choicelist
        self.__choiceindex= None

        # listbox
        self.__list= wx.ListBox(self, size= (-1, -1), choices= self.__choicelist)

        # search index of default and select default
        for i in range(len(self.__choicelist)):
            if self.__choicelist[i]== default:
                self.__list.SetSelection(i)

        buttonok= wx.Button(self, label= _(u'OK'))
        buttoncancel= wx.Button(self, label= _(u'Cancel'))

        # bindings
        buttonok.Bind(wx.EVT_BUTTON, self.OnOk)
        buttoncancel.Bind(wx.EVT_BUTTON, self.OnCancel)
        self.__list.Bind(wx.EVT_LISTBOX_DCLICK, self.OnOk)
        self.__list.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)

        # layout
        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(self.__list, 1, wx.EXPAND|wx.ALL, 5)
        hbox= wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(buttonok, 1, wx.EXPAND|wx.ALL, 5)
        hbox.Add(buttoncancel, 1, wx.EXPAND|wx.ALL, 5)
        vbox.Add(hbox, 0, wx.EXPAND)

        self.__list.SetFocus()

        self.SetSizer(vbox)
        self.Centre()


    def OnOk(self, event):
        self.OkDialogue()


    def OnCancel(self, event):
        self.CancelDialogue()


    def GetChoice(self):
        if self.__choiceindex== None:
            return('')
        else:
            return(self.__choicelist[self.__choiceindex])


    def CancelDialogue(self):
        self.__choiceindex= None
        self.Close()


    def OkDialogue(self):
        self.__choiceindex= self.__list.GetSelection()
        self.Close()


    def OnKeyDown(self, event):
        # check is esc pressed
        if event.GetKeyCode()== wx.WXK_ESCAPE:
            self.CancelDialogue()
        elif event.GetKeyCode()== wx.WXK_RETURN:
            self.OkDialogue()
        elif event.GetKeyCode()== wx.WXK_NUMPAD_ENTER:
            self.OkDialogue()


class TestFrame(wx.Frame):
    # object for testing
    def __init__(self):
        wx.Frame.__init__(self, None, title= u'Test Dialog Load/ Save Object')

        buttonload= wx.Button(self, label= u'Load dialogue')
        buttonsave= wx.Button(self, label= u'Save dialogue')
        buttoncolour= wx.Button(self, label= u'Color dialogue')
        buttontext= wx.Button(self, label= u'Text dialogue')
        buttonlist= wx.Button(self, label= u'List dialogue')
        buttonexit= wx.Button(self, label= u'Exit')

        buttonload.Bind(wx.EVT_BUTTON, self.OnLoad)
        buttonsave.Bind(wx.EVT_BUTTON, self.OnSave)
        buttoncolour.Bind(wx.EVT_BUTTON, self.OnColour)
        buttontext.Bind(wx.EVT_BUTTON, self.OnText)
        buttonlist.Bind(wx.EVT_BUTTON, self.OnList)
        buttonexit.Bind(wx.EVT_BUTTON, self.OnExit)

        self.__dlg= LorzeDlgHelper(self)

        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(buttonload)
        vbox.Add(buttonsave)
        vbox.Add(buttoncolour)
        vbox.Add(buttontext)
        vbox.Add(buttonlist)
        vbox.Add(buttonexit)

        self.SetSizer(vbox)
        self.Center()
        self.Show()


    def OnLoad(self, event):
        print(self.__dlg.OpenDialog('Load dialog', '/', 'Test.txt',
         'Text (*.txt)|*.txt|All (*.*)|*.*'))


    def OnSave(self, event):
        print(self.__dlg.SaveDialog('Save dialog', '/', 'Test.txt',
         'Text (*.txt)|*.txt|All (*.*)|*.*', '.txt'))


    def OnColour(self, event):
        print(self.__dlg.ColorDialog('#FFFFFF'))


    def OnText(self, event):
        print(self.__dlg.TextDialog('Test', 'Testeingabe ..', '123'))


    def OnList(self, event):
        print(self.__dlg.ListSelection('List-Test', ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i'], 'e'))


    def OnExit(self, event):
        self.Close()


if __name__== '__main__':
    app= wx.App()

    # internationalization
    wxloc= wx.Locale()
    wxloc.AddCatalogLookupPathPrefix('./in18')
    # get system language ('xx_XX', 'CHARSET')
    wxlang= locale.getdefaultlocale()
    wxlang= wxlang[0][:2]
    # select translations
    if wxlang== 'de':
        wxloc.AddCatalog('lorze_de')

    frame= TestFrame()
    app.MainLoop()

