#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx, os, pickle, locale
from lrzloadsave import LorzeLoadSave

# name for translations texts
_= wx.GetTranslation


class LorzeDlgLoadSave():
    # load and save dialogs for LORZE ersandcad

    def __init__(self, parent, options):

        # parent class
        self.__parent= parent

        # LorzeOptions() of parent class
        self.__options= options

        # string: path
        self.__path= None

        # data container
        self.__data= None

        # string: filename
        self.__filename= ''

        # string: file directory
        self.__filedir= ''

        # string: caption
        self.__label= ''

        # string: wildcard, at example: 'Text (*.txt)|*.txt|All (*.*)|*.*'
        self.__wildcard= ''

        # string: default extension
        self.__defext= ''

    def OpenDialog(self):
        # dialog for open files, show dialog
        dlg= wx.FileDialog(self.__parent, self.__label, self.__filedir, self.__filename, self.__wildcard, wx.FD_OPEN)

        if dlg.ShowModal()== wx.ID_OK:
            # clicked ok, set path
            self.__path= dlg.GetPath()

            # destroy dialog
            dlg.Destroy()

            # ok
            return(True)

        else:
            # not clicked ok, destroy dialog
            dlg.Destroy()

            # cancel
            return(False)


    def SaveDialog(self):
        # dialog for save files with extension & overwrite control, show dialog
        dlg= wx.FileDialog(self.__parent, self.__label, self.__filedir, self.__filename, self.__wildcard, wx.FD_SAVE)

        if dlg.ShowModal()== wx.ID_OK:
            # clicked ok, set path
            self.__path= dlg.GetPath()

            # destroy dialog
            dlg.Destroy()

            if self.__path.endswith(self.__defext)== False:
                # check extension, correct extension
                self.__path+= self.__defext

                # syntax for the os
                self.__path= os.path.normpath(self.__path)

            if os.path.exists(self.__path):
                # file already exist, overwrite?
                question= wx.MessageDialog(None,
                 _(u'The file already exists: ')+ self.__path+
                 _(u'. Do you want to overwrite ?'), _(u'Confirmation'),
                    wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)

                # show dialog and get answer
                answer= question.ShowModal()

                if answer== wx.ID_NO:
                    # do not overwrite
                    return(False)

                else:
                    # overwrite
                    return(True)

            # file dont exist, all ok
            else:
                return(True)

        else:
            # not clicked ok, destroy dialog
            dlg.Destroy()
            return(False)


    def LoadDrawing(self):
        # load LORZE drawing, with dialog, set wildcards
        self.__wildcard= _(u'LORZE drawings')+ ' (*.lrzdrw)|*.lrzdrw|'+ u'All files'+ ' (*.*)|*.*'

        # title
        self.__label= _(u'Load drawing')

        # show dialog with flag
        dlgflag= self.OpenDialog()

        if dlgflag== True:
            # all ok, helper object
            ls= LorzeLoadSave()

            # set path
            ls.SetPath(self.__path)

            # load with flag
            loadflag= ls.Load()
            # for testing load routine: ls.LoadDevelop()

            if loadflag== False:
                # error while loading
                dlg = wx.MessageDialog(self.__parent, _(u'Error opening the file: ')+ self.__path, _(u'File error'), style= wx.OK|wx.ICON_ERROR)

                # show message & destroy dialog
                dlg.ShowModal()
                dlg.Destroy()

                # return error
                return(False)

            else:
                # no error, read loaded datas
                self.__data= ls.GetData()

                # return no error
                return(True)


    def SaveAsDrawing(self):
        # save LORZE drawing, with dialog, default extension= .lrzdrw
        self.__defext= '.lrzdrw'

        # wildcards
        self.__wildcard= _(u'LORZE drawings')+ ' (*.lrzdrw)|*.lrzdrw'

        # title
        self.__label= _(u'Save drawing')

        # show dialog with flag
        dlgflag= self.SaveDialog()

        if dlgflag== True:
            # all ok, helper object
            ls= LorzeLoadSave()

            # set datas
            ls.SetData(self.__data)

            # set path
            ls.SetPath(self.__path)

            # save with flag
            saveflag= ls.Save()
            # for testing save routine: ls.SaveDevelop()

            if saveflag== False:
                # error while saving
                dlg = wx.MessageDialog(self.__parent, _(u'Error saving the file: ')+ self.__path, _(u'File error'), style= wx.OK|wx.ICON_ERROR)

                # show message & destroy dialog
                dlg.ShowModal()
                dlg.Destroy()

                # return error
                return(False)

            else:
                # no error, return no error
                return(True)


    def SetData(self, data):
        # set datas
        self.__data= data


    def GetData(self):
        # get datas
        return(self.__data)


    def SetPath(self, path):
        # set path
        self.__path= path

        # set filename and filedir from path
        self.__filename= os.path.basename(self.__path)
        self.__filedir= os.path.dirname(self.__path)


    def GetPath(self):
        # get path (string)
        return(self.__path)


class TestFrame(wx.Frame):
    # object for testing

    def __init__(self):
        from lrzoptions import LorzeOptions

        wx.Frame.__init__(self, None, title= u'Test Dialog Load/ Save Object')
        self.__options= LorzeOptions()

        loadbutton= wx.Button(self, label= u'Load')
        savebutton= wx.Button(self, label= u'Save as ..')

        loadbutton.Bind(wx.EVT_BUTTON, self.OnLoad)
        savebutton.Bind(wx.EVT_BUTTON, self.OnSaveAs)

        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(loadbutton)
        vbox.Add(savebutton)

        self.SetSizer(vbox)
        self.Center()
        self.Show()


    def OnLoad(self, event):
        lsdlg= LorzeDlgLoadSave(self, self.__options)
        lsdlg.SetPath= ''
        dlgflag= lsdlg.LoadDrawing()
        print(lsdlg.GetPath())
        print(lsdlg.GetData())
        print(str(dlgflag))


    def OnSaveAs(self, event):
        lsdlg= LorzeDlgLoadSave(self, self.__options)
        lsdlg.SetData(self.__options)
        lsdlg.SetPath= ''
        dlgflag= lsdlg.SaveAsDrawing()
        print(lsdlg.GetPath())
        print(lsdlg.GetData())
        print(str(dlgflag))


if __name__== '__main__':
    app= wx.App()

    # internationalization
    wxloc= wx.Locale()
    wxloc.AddCatalogLookupPathPrefix('./in18')
    # get system language ('xx_XX', 'CHARSET')
    wxlang= locale.getdefaultlocale()
    wxlang= wxlang[0][:2]
    # select translations
    if wxlang== 'de':
        wxloc.AddCatalog('LORZE_de')

    frame= TestFrame()
    app.MainLoop()

