#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx, re, locale, sys
import wx.lib.mixins.listctrl as listmix

# name for translations texts
_= wx.GetTranslation


#~ class EditableListCtrl(wx.ListCtrl, listmix.TextEditMixin):
    #~ # class from http://www.blog.pythonlibrary.org/2011/01/04/wxpython-wx-listctrl-tips-and-tricks/
    #~ # TextEditMixin allows any column to be edited.
    #~ def __init__(self, parent, ID=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0):
        #~ #Constructor
        #~ wx.ListCtrl.__init__(self, parent, ID, pos, size, style)
        #~ listmix.TextEditMixin.__init__(self)


class SortedListCtrl(wx.ListCtrl, listmix.ColumnSorterMixin):
    # class from http://zetcode.com/wxpython/
    def __init__(self, parent):
        wx.ListCtrl.__init__(self, parent, -1, style=wx.LC_REPORT)
        sortdic= parent.GetSortedListDict()
        listmix.ColumnSorterMixin.__init__(self, len(sortdic))
        self.itemDataMap = sortdic


    def GetListCtrl(self):
        return self


class LorzeDlgOptions(wx.Dialog):
    #Lorze Options Dialog to the GUI of LORZE erasandcad

    def __init__(self, parent, options, valchck, dlghelper, gdipanel):
        # parent class
        self.__parent= parent

        # LorzeOptions() of parent class
        self.__options= options

        # LorzeValueCheck
        self.__valchck= valchck

        # DialogHelper
        self.__dlg= dlghelper

        # GdiPanel
        self.__gdi= gdipanel

        # bool: Flag for gui restart
        self.__restart= False

        # get settings for dialog from options
        dlgborder= self.__options.GetDlgBorder()
        wdlg, hdlg= self.DialogueSize()

        # subclass
        wx.Dialog.__init__(self, self.__parent, wx.ID_ANY, _(u'Program options'), size= (wdlg, hdlg), style= wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)

        # datas for list control: orderlist= order to show options
        self.__orderlist= ['sensitive', 'scsize', 'scpensize', 'scmarkpensize', 'sccolpoint', 'sccolsegm', 'sccolelem', 'selcol', 'prevcol', 'prevpensize', 'prevpenstyle', 'dezplcs', 'dlgborder', 'framesize', 'dlgoptsize', 'optlistwidths', 'dlgattrsize', 'attrlistwidths', 'dlghelpsize', 'gdiborder', 'cursorwin', 'cursorselect', 'cursordraw', 'stcoordw', 'sbrange', 'sbsize', 'wheelenlarge', 'wheelreduce', 'maxpenstylescale', 'maxuserscale']

        # labels of the options: dictionairy
        self.__labeldict= {'framesize':_(u'Size: Program window'), 'gdiborder':_(u'Border: Graphic area'), 'dezplcs':_(u'Number of dezimalplaces'), 'cursorwin':_(u'Cursor: Menus & dialogues'), 'cursordraw':_(u'Cursor: Drawing commands'), 'cursorselect':_(u'Cursor: Selections'), 'dlgborder':_(u'Size: Border between widgets'), 'dlgoptsize':_(u'Size: Option dialogue'), 'sbrange':_(u'Size: Scrollbar range'), 'sbsize':_(u'Size: Scrollbar slider'), 'wheelenlarge':_(u'Mouse wheel: Magnification'), 'wheelreduce':_(u'Mouse wheel: Reduce'), 'sensitive':_(u'Size: Smart cursor'), 'maxpenstylescale':_(u'Maximal zoom: Line types'), 'maxuserscale':_(u'Maximal zoom: Drawing'), 'scsize':_(u'Size: Cursor mark symbol'), 'scpensize':_(u'Size: Cursor mark pen'), 'sccolpoint':_(u'Colour: Point mark'), 'sccolsegm':_(u'Colour: Segment mark'), 'sccolelem':_(u'Colour: Element mark'), 'scmarkpensize':_(u'Size: Marked elements'), 'prevcol':_(u'Colour: Element preview'), 'prevpensize':_(u'Size: Preview pen'), 'prevpenstyle':_(u'Style: Preview pen'), 'stcoordw':_(u'Size: Width of coordinate fields'), 'selcol':_(u'Colour: Selected elements'), 'dlgattrsize':_(u'Size: Attribute dialogue'), 'optlistwidths':_(u'Size: Width columns option dialogue'), 'attrlistwidths':_(u'Size: Width columns attribute dialogue'), 'defaultattributes':_(u'Defaults: Drawing attributes'), 'dlghelpsize':_(u'Size: Help dialogue')}

        # set defaults from options
        self.SetDefaults()

        # list control
        self.__list= SortedListCtrl(self)

        widths= self.__options.GetOptListWidths()
        self.__list.InsertColumn(0, _(u'Label'), width= widths[0])
        self.__list.InsertColumn(1, _(u'Value'), width= widths[1])

        self.SetListCtrlNew()

        # dialog buttons
        buttonedit= wx.Button(self, label= _(u'Edit option'))
        buttonok= wx.Button(self, label= _(u'OK'))
        buttoncancel= wx.Button(self, label= _(u'Cancel'))
        buttonreset= wx.Button(self, label= _(u'Reset'))

        # dialog bindings
        buttonedit.Bind(wx.EVT_BUTTON, self.OnEdit)
        buttonok.Bind(wx.EVT_BUTTON, self.OnOk)
        buttoncancel.Bind(wx.EVT_BUTTON, self.OnCancel)
        buttonreset.Bind(wx.EVT_BUTTON, self.OnReset)
        self.Bind(wx.EVT_CLOSE, self.OnDialogueClose)

        # listcontrol bindings
        self.__list.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemSelected)

        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(self.__list, 1, wx.EXPAND|wx.ALL, dlgborder)

        gbox=wx.GridSizer(1, 4, dlgborder, dlgborder)
        gbox.AddMany([(buttonedit, 0, wx.EXPAND, dlgborder), (buttonok, 0, wx.EXPAND, dlgborder), (buttoncancel, 0, wx.EXPAND, dlgborder),  (buttonreset, 0, wx.EXPAND, dlgborder)])
        vbox.Add(gbox, 0, wx.EXPAND|wx.ALL, dlgborder)

        self.SetSizer(vbox)
        self.Centre()


    def OnOk(self, event):
        # set all options to LorzeOptions, restart the gui
        # frametitle: ist not editable
        size= self.__valchck.StringToList(self.__valuedict['framesize'], '/', False)
        self.__options.SetFrameSize(size[0], size[1])

        self.__options.SetGDIBorder(self.__valuedict['gdiborder'])
        self.__options.SetDezPlcs(self.__valuedict['dezplcs'])
        self.__options.SetCursorWin(self.__valuedict['cursorwin'])
        self.__options.SetCursorDraw(self.__valuedict['cursordraw'])
        self.__options.SetCursorSelect(self.__valuedict['cursorselect'])
        self.__options.SetDlgBorder(self.__valuedict['dlgborder'])

        size= self.__valchck.StringToList(self.__valuedict['dlgoptsize'], '/', False)
        self.__options.SetDlgOptSize(size[0], size[1])

        self.__options.SetSbRange(self.__valuedict['sbrange'])
        self.__options.SetSbSize(self.__valuedict['sbsize'])
        self.__options.SetWheelEnlarge(self.__valuedict['wheelenlarge'])
        self.__options.SetWheelReduce(self.__valuedict['wheelreduce'])
        self.__options.SetSensitive(self.__valuedict['sensitive'])
        self.__options.SetMaxPenStyleScale(self.__valuedict['maxpenstylescale'])
        self.__options.SetMaxUserScale(self.__valuedict['maxuserscale'])
        self.__options.SetSmartCursSize(self.__valuedict['scsize'])
        self.__options.SetSmartCursPenSize(self.__valuedict['scpensize'])
        self.__options.SetSmartCursColPoint(self.__valuedict['sccolpoint'])
        self.__options.SetSmartCursColSegm(self.__valuedict['sccolsegm'])
        self.__options.SetSmartCursColElem(self.__valuedict['sccolelem'])
        self.__options.SetSmartCursMarkPenSize(self.__valuedict['scmarkpensize'])
        self.__options.SetPreviewColor(self.__valuedict['prevcol'])
        self.__options.SetPreviewPenSize(self.__valuedict['prevpensize'])
        self.__options.SetPreviewPenStyle(self.__valuedict['prevpenstyle'])
        self.__options.SetStCoordW(self.__valuedict['stcoordw'])
        self.__options.SetSelCol(self.__valuedict['selcol'])

        size= self.__valchck.StringToList(self.__valuedict['dlgattrsize'], '/', False)
        self.__options.SetDlgAttrSize(size[0], size[1])

        widths= self.__valchck.StringToList(self.__valuedict['optlistwidths'], ',', False)
        self.__options.SetOptListWidths(widths[0], widths[1])

        widths= self.__valchck.StringToList(self.__valuedict['attrlistwidths'], ',', False)
        self.__options.SetAttrListWidths(widths[0], widths[1], widths[2], widths[3], widths[4], widths[5], widths[6], widths[7], widths[8], widths[9], widths[10], widths[11], widths[12], widths[13], widths[14], widths[15])

        size= self.__valchck.StringToList(self.__valuedict['dlghelpsize'], '/', False)
        self.__options.SetDlgHelpSize(size[0], size[1])

        # True: restart GUI
        self.__restart= True
        self.Close()


    def OnCancel(self, event):
        # set no options, False, no restart
        self.__restart= False

        self.Close()


    def OnReset(self, event):
        # reset option, confirmation
        dlg = wx.MessageDialog(None, _(u'Are yo sure to put the settings back? This command can not be undone..'), _(u'Confirmation'), wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
        answer= dlg.ShowModal()

        if answer== wx.ID_YES:
            # yes: reset options
            self.__options.Reset()

            # set new defaults and set new the listcontrol
            self.SetDefaults()
            self.SetListCtrlNew()

            # set resetted sizes to parent frame
            self.__parent.SetSize(self.__options.GetFrameSize())
            self.__parent.Centre()

            # set resetted sizes
            wdlg, hdlg= self.DialogueSize()
            self.SetSize((wdlg, hdlg))
            self.Centre()

            widths= self.__options.GetOptListWidths()
            self.__list.SetColumnWidth(0, widths[0])
            self.__list.SetColumnWidth(1, widths[1])


    def GetRestart(self):
        # get restart (bool)
        return(self.__restart)


    def SetDefaults(self):
        self.__valuedict= {}

        w, h= self.__options.GetFrameSize()
        self.__valuedict['framesize']= str(w)+ '/' +str(h)

        self.__valuedict['gdiborder']= self.__options.GetGDIBorder()
        self.__valuedict['dezplcs']= str(self.__options.GetDezPlcs())
        self.__valuedict['cursorwin']= self.__options.GetCursorWin()
        self.__valuedict['cursordraw']= self.__options.GetCursorDraw()
        self.__valuedict['cursorselect']= self.__options.GetCursorSelect()
        self.__valuedict['dlgborder']= str(self.__options.GetDlgBorder())

        w, h= self.__options.GetDlgOptSize()
        self.__valuedict['dlgoptsize']= str(w)+ '/' +str(h)

        self.__valuedict['sbrange']= str(self.__options.GetSbRange())
        self.__valuedict['sbsize']= str(self.__options.GetSbSize())
        self.__valuedict['wheelenlarge']= str(self.__options.GetWheelEnlarge())
        self.__valuedict['wheelreduce']= str(self.__options.GetWheelReduce())
        self.__valuedict['sensitive']= str(self.__options.GetSensitive())
        self.__valuedict['maxpenstylescale']= str(self.__options.GetMaxPenStyleScale())
        self.__valuedict['maxuserscale']= str(self.__options.GetMaxUserScale())
        self.__valuedict['scsize']= str(self.__options.GetSmartCursSize())
        self.__valuedict['scpensize']= str(self.__options.GetSmartCursPenSize())
        self.__valuedict['sccolpoint']= self.__options.GetSmartCursColPoint()
        self.__valuedict['sccolsegm']= self.__options.GetSmartCursColSegm()
        self.__valuedict['sccolelem']= self.__options.GetSmartCursColElem()
        self.__valuedict['scmarkpensize']= str(self.__options.GetSmartCursMarkPenSize())
        self.__valuedict['prevcol']= self.__options.GetPreviewColor()
        self.__valuedict['prevpensize']= str(self.__options.GetPreviewPenSize())
        self.__valuedict['prevpenstyle']= self.__options.GetPreviewPenStyle()
        self.__valuedict['stcoordw']= str(self.__options.GetStCoordW())
        self.__valuedict['selcol']= self.__options.GetSelCol()

        w, h= self.__options.GetDlgAttrSize()
        self.__valuedict['dlgattrsize']= str(w)+ '/' +str(h)

        w0, w1= self.__options.GetOptListWidths()
        self.__valuedict['optlistwidths']= str(w0)+ ','+ str(w1)

        w0, w1, w2, w3, w4, w5, w6, w7, w8, w9, w10, w11, w12, w13, w14, w15= self.__options.GetAttrListWidths()
        self.__valuedict['attrlistwidths']= str(w0)+ ','+ str(w1)+ ','+ str(w2)+ ','+ str(w3)+ ','+ str(w4)+ ','+ str(w5)+ ','+ str(w6)+ ','+ str(w7)+ ','+ str(w8)+ ','+ str(w9)+ ','+ str(w10)+ ','+ str(w11)+ ','+ str(w12)+ ','+ str(w13)+ ','+ str(w14)+ ','+ str(w15)

        w, h= self.__options.GetDlgHelpSize()
        self.__valuedict['dlghelpsize']= str(w)+ '/' +str(h)


    def SetListCtrlNew(self):
        # Set the list new, clear all items
        self.__list.DeleteAllItems()

        #get items from SortedListDictionairy and add to listcontrol
        items = self.GetSortedListDict().items()
        for key, data in items:
            index = self.__list.InsertStringItem(sys.maxint, data[0])
            self.__list.SetStringItem(index, 1, data[1])
            self.__list.SetItemData(index, key)


    def OnItemSelected(self, event):
        # Dobuleclick on item in listcontrol, edit item
        self.EditOption()


    def OnEdit(self, event):
        # click on edit button, edit item
        self.EditOption()


    def EditOption(self):
        # edit opiton: get index, search selected option
        index= self.__list.GetFocusedItem()
        label= self.__list.GetItemText(index)
        for key, data in self.__labeldict.items():
            if data== label:
                seloption= key

        if seloption== 'framesize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['framesize'], _(u'Please enter the size of the program window, width/height'), self.__valuedict['framesize'])
            if text!= '':
                # clicked ok, get size (w, h), convert and check
                text= self.ConvWindowSize(text, 400)
                if text!= '':
                    # size is correct
                    self.__valuedict['framesize']= text

        elif seloption== 'gdiborder':
            # get string from list selection dialogue
            text= self.__dlg.ListSelection(self.__labeldict['gdiborder'], self.__gdi.GetBordersDict().keys(), self.__valuedict['gdiborder'])
            if text!= '':
                # clicked ok
                self.__valuedict['gdiborder']= text

        elif seloption== 'dezplcs':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['dezplcs'], _(u'Please enter the number of dezimal places'), self.__valuedict['dezplcs'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['dezplcs']= self.__valchck.Int(text, str(self.__options.GetDezPlcs()), 0)

        elif seloption== 'cursorwin':
            # get string from list selection dialogue
            text= self.__dlg.ListSelection(self.__labeldict['cursorwin'], self.__gdi.GetCursorsDict().keys(), self.__valuedict['cursorwin'])
            if text!= '':
                # clicked ok
                self.__valuedict['cursorwin']= text

        elif seloption== 'cursordraw':
            # get string from list selection dialogue
            text= self.__dlg.ListSelection(self.__labeldict['cursordraw'], self.__gdi.GetCursorsDict().keys(), self.__valuedict['cursordraw'])
            if text!= '':
                # clicked ok
                self.__valuedict['cursordraw']= text

        elif seloption== 'cursorselect':
            # get string from list selection dialogue
            text= self.__dlg.ListSelection(self.__labeldict['cursorselect'], self.__gdi.GetCursorsDict().keys(), self.__valuedict['cursorselect'])
            if text!= '':
                # clicked ok
                self.__valuedict['cursorselect']= text

        elif seloption== 'dlgborder':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['dlgborder'], _(u'Please enter the distance between the widgets in the user interface'), self.__valuedict['dlgborder'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['dlgborder']= self.__valchck.Int(text, str(self.__options.GetDlgBorder()), 1)

        elif seloption== 'dlgoptsize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['dlgoptsize'], _(u'Please enter the size of the settings dialogue, width/height'), self.__valuedict['dlgoptsize'])
            if text!= '':
                # clicked ok, get size (w, h), convert and check
                text= self.ConvWindowSize(text, 200)
                if text!= '':
                    # size is correct
                    self.__valuedict['dlgoptsize']= text

        elif seloption== 'sbrange':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['sbrange'], _(u'Please enter the range of the scroll bars'), self.__valuedict['sbrange'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['sbrange']= self.__valchck.Int(text, str(self.__options.GetSbRange()), 10)

        elif seloption== 'sbsize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['sbsize'], _(u'Please enter the size of the scroll bar slider'), self.__valuedict['sbsize'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['sbsize']= self.__valchck.Int(text, str(self.__options.GetSbSize()), 1)

        elif seloption== 'wheelenlarge':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['wheelenlarge'], _(u'Please enter the magnification using the mouse wheel'), self.__valuedict['wheelenlarge'])
            if text!= '':
                # clicked ok, get float
                self.__valuedict['wheelenlarge']= self.__valchck.Float(text, str(self.__options.GetWheelEnlarge()), 1.01)

        elif seloption== 'wheelreduce':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['wheelreduce'], _(u'Please enter the reduction using the mouse wheel'), self.__valuedict['wheelreduce'])
            if text!= '':
                # clicked ok, get float
                self.__valuedict['wheelreduce']= self.__valchck.Float(text, str(self.__options.GetWheelReduce()), 0.01)
                if float(self.__valuedict['wheelreduce'])>= 1:
                    self.__valuedict['wheelreduce']= '0.99'

        elif seloption== 'sensitive':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['sensitive'], _(u'Please enter the size of the smart cursor'), self.__valuedict['sensitive'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['sensitive']= self.__valchck.Int(text, str(self.__options.GetSensitive()), 1)

        elif seloption== 'maxpenstylescale':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['maxpenstylescale'], _(u'Please specify the highest magnification to show linetypes'), self.__valuedict['maxpenstylescale'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['maxpenstylescale']= self.__valchck.Int(text, str(self.__options.GetMaxPenStyleScale()), 2)

        elif seloption== 'maxuserscale':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['maxuserscale'], _(u'Please enter the highest magnification for drawing'), self.__valuedict['maxuserscale'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['maxuserscale']= self.__valchck.Int(text, str(self.__options.GetMaxUserScale()), 2)

        elif seloption== 'scsize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['scsize'], _(u'Please enter the size of the marker symbol'), self.__valuedict['scsize'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['scsize']= self.__valchck.Int(text, str(self.__options.GetSmartCursSize()), 2)

        elif seloption== 'scpensize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['scpensize'], _(u'Please enter the strength of the marking pen'), self.__valuedict['scpensize'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['scpensize']= self.__valchck.Int(text, str(self.__options.GetSmartCursPenSize()), 1)

        elif seloption== 'sccolpoint':
            # get color as html string from color selection dialogue
            colour= self.__dlg.ColorDialog(self.__valuedict['sccolpoint'])
            if colour!= '':
                # clicked ok, get colour html string
                self.__valuedict['sccolpoint']= colour

        elif seloption== 'sccolsegm':
            # get color as html string from color selection dialogue
            colour= self.__dlg.ColorDialog(self.__valuedict['sccolsegm'])
            if colour!= '':
                # clicked ok, get colour html string
                self.__valuedict['sccolsegm']= colour

        elif seloption== 'sccolelem':
            # get color as html string from color selection dialogue
            colour= self.__dlg.ColorDialog(self.__valuedict['sccolelem'])
            if colour!= '':
                # clicked ok, get colour html string
                self.__valuedict['sccolelem']= colour

        elif seloption== 'scmarkpensize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['scmarkpensize'], _(u'Please enter the magnification of the line width'), self.__valuedict['scmarkpensize'])
            if text!= '':
                # clicked ok, get float
                self.__valuedict['scmarkpensize']= self.__valchck.Float(text, str(self.__options.GetSmartCursMarkPenSize()), 1.0)

        elif seloption== 'prevcol':
            # get color as html string from color selection dialogue
            colour= self.__dlg.ColorDialog(self.__valuedict['prevcol'])
            if colour!= '':
                # clicked ok, get colour html string
                self.__valuedict['prevcol']= colour

        elif seloption== 'prevpensize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['prevpensize'], _(u'Please enter the line width for the preview'), self.__valuedict['prevpensize'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['prevpensize']= self.__valchck.Int(text, str(self.__options.GetPreviewPenSize()), 0)

        elif seloption== 'prevpenstyle':
            # get string from list selection dialogue, get list from dictionairy in GraphicPanel
            prevpenstylelist= self.__gdi.GetPenStyleDict().keys()

            # delete 'wx.USER_DASH' from list
            for i in range(len(prevpenstylelist)):
                if prevpenstylelist[i]== 'wx.USER_DASH':
                    del prevpenstylelist[i]
                    break

            text= self.__dlg.ListSelection(self.__labeldict['prevpenstyle'], prevpenstylelist, self.__valuedict['prevpenstyle'])
            if text!= '':
                # clicked ok
                self.__valuedict['prevpenstyle']= text

        elif seloption== 'stcoordw':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['stcoordw'], _(u'Please enter the width of the coordinate fields in the status bar'), self.__valuedict['stcoordw'])
            if text!= '':
                # clicked ok, get integer
                self.__valuedict['stcoordw']= self.__valchck.Int(text, str(self.__options.GetStCoordW()), 20)

        elif seloption== 'selcol':
            # get color as html string from color selection dialogue
            colour= self.__dlg.ColorDialog(self.__valuedict['selcol'])
            if colour!= '':
                # clicked ok, get colour html string
                self.__valuedict['selcol']= colour

        elif seloption== 'dlgattrsize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['dlgattrsize'], _(u'Please enter the size of the attribute dialogue, width/height'), self.__valuedict['dlgattrsize'])
            if text!= '':
                # clicked ok, get size (w, h), convert and check
                text= self.ConvWindowSize(text, 200)
                if text!= '':
                    # size is correct
                    self.__valuedict['dlgattrsize']= text

        elif seloption== 'optlistwidths':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['optlistwidths'], _(u'Please enter the width of the columns in the settings dialogue, width1,width2'), self.__valuedict['optlistwidths'])
            if text!= '':
                # clicked ok, get widths (w1, w2) and check
                widths= self.__valchck.StringToList(text, ',', False)
                if len(widths)> 1:
                    if widths[0]< 40:
                        widths[0]= 40
                    if widths[1]< 40:
                        widths[1]= 40
                    self.__valuedict['optlistwidths']= str(widths[0])+ ',' +str(widths[1])

        elif seloption== 'attrlistwidths':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['attrlistwidths'], _(u'Please enter the width of the columns in the attribute dialogue, each 4 values for colour, layer, line style and line width'), self.__valuedict['attrlistwidths'])
            if text!= '':
                # clicked ok, get widths (w0, w1, w2, w3, w4, w5, w6, w7, w8, w9, w10, w11, w12, w13, w14, w15) and check
                widths= self.__valchck.StringToList(text, ',', False)
                if len(widths)> 15:
                    if widths[0]< 40:
                        widths[0]= 40
                    if widths[1]< 40:
                        widths[1]= 40
                    if widths[2]< 40:
                        widths[2]= 40
                    if widths[3]< 40:
                        widths[3]= 40
                    if widths[4]< 40:
                        widths[4]= 40
                    if widths[5]< 40:
                        widths[5]= 40
                    if widths[6]< 40:
                        widths[6]= 40
                    if widths[7]< 40:
                        widths[7]= 40
                    if widths[8]< 40:
                        widths[8]= 40
                    if widths[9]< 40:
                        widths[9]= 40
                    if widths[10]< 40:
                        widths[10]= 40
                    if widths[11]< 40:
                        widths[11]= 40
                    if widths[12]< 40:
                        widths[12]= 40
                    if widths[13]< 40:
                        widths[13]= 40
                    if widths[14]< 40:
                        widths[14]= 40
                    if widths[15]< 40:
                        widths[15]= 40
                    self.__valuedict['attrlistwidths']= str(widths[0])+ ',' +str(widths[1])+ ',' +str(widths[2])+ ',' +str(widths[3])+ ',' +str(widths[4])+ ',' +str(widths[5])+ ',' +str(widths[6])+ ',' +str(widths[7])+ ',' +str(widths[8])+ ',' +str(widths[9])+ ',' +str(widths[10])+ ',' +str(widths[11])+ ',' +str(widths[12])+ ',' +str(widths[13])+ ',' +str(widths[14])+ ',' +str(widths[15])

        elif seloption== 'dlghelpsize':
            # get text from text entry dialogue
            text= self.__dlg.TextDialog(self.__labeldict['dlghelpsize'], _(u'Please enter the size of the help dialogue, width/height'), self.__valuedict['dlghelpsize'])
            if text!= '':
                # clicked ok, get size (w, h), convert and check
                text= self.ConvWindowSize(text, 200)
                if text!= '':
                    # size is correct
                    self.__valuedict['dlghelpsize']= text

        else:
            # option not found, no action
            return

        self.__list.DeleteItem(index)

        self.__list.InsertStringItem(index, self.__labeldict[seloption])
        self.__list.SetStringItem(index, 1, self.__valuedict[seloption])
        self.__list.SetItemData(index, index)


    def GetSortedListDict(self):
        # Returns a dictionairy for SortedListCtrl
        sortedlistdict= {}
        index= 0
        for i in self.__orderlist:
            sortedlistdict[index]= (self.__labeldict[i], self.__valuedict[i])
            index+= 1
        return(sortedlistdict)


    def OnDialogueClose(self, event):
        # last commando before close the dialogue, read dialogue size and widths of the columns
        wdlg, hdlg= self.GetSize()
        colw1= self.__list.GetColumnWidth(0)
        colw2= self.__list.GetColumnWidth(1)

        # set sizes to options
        self.__options.SetDlgOptSize(wdlg, hdlg)
        self.__options.SetOptListWidths(colw1, colw2)

        event.Skip()


    def DialogueSize(self):
        wdlg, hdlg= self.__options.GetDlgOptSize()

        # get size from parent
        wparent, hparent= self.__parent.GetSize()

        # dialog size control
        if hdlg> hparent:
            # correct height to parent size
            hdlg= hparent

        if wdlg> wparent:
            # correct width to parent size
            wdlg= wparent

        return(wdlg, hdlg)


    def ConvWindowSize(self, text, minimum):
        # converts a window size entry into a string, check size minimum and size maximum
        size= self.__valchck.StringToList(text, '/', False)
        if len(size)> 1:
            wdisp, hdisp= wx.GetDisplaySize()
            if size[0]< minimum:
                size[0]= minimum
            elif size[0]> wdisp:
                size[0]= wdisp
            if size[1]< minimum:
                size[1]= minimum
            elif size[1]> hdisp:
                size[1]= hdisp
            return(str(size[0])+ '/' +str(size[1]))
        else:
            return('')


class TestFrame(wx.Frame):
    # object for testing

    def __init__(self):
        wx.Frame.__init__(self, None, title= u'Testcode LorzeDlgOptions', size= (600, 600))

        self.__options= LorzeOptions()
        self.__valchck= LorzeValueCheck()
        self.__dlg= LorzeDlgHelper(self)
        self.__geohelp= LorzeGeoHelp(self.__options)
        self.__log= LorzeLog()
        self.__attribute= LorzeAttribute(self.__options, self.__log, self.__valchck)
        self.__drawman= LorzeDrawManager(self.__attribute, self.__log, self.__geohelp)
        self.__selection= LorzeSelection(self.__drawman, self.__geohelp)
        self.__gdi= GraphicPanel(self, self.__options, self.__geohelp, self.__log, self.__valchck, self.__attribute, self.__drawman, self.__selection)

        buttontest= wx.Button(self, label= u'Test')
        buttonexit= wx.Button(self, label= u'Exit')

        vbox= wx.BoxSizer(wx.VERTICAL)
        vbox.Add(buttontest)
        vbox.Add(buttonexit)
        self.SetSizer(vbox)

        buttontest.Bind(wx.EVT_BUTTON, self.OnTest)
        buttonexit.Bind(wx.EVT_BUTTON, self.OnExit)

        self.Center()
        self.Show()


    def OnTest(self, event):
        dlg= LorzeDlgOptions(self, self.__options, self.__valchck, self.__dlg, self.__gdi)
        dlg.ShowModal()
        restart= dlg.GetRestart()
        dlg.Destroy()
        if restart== True:
            print(self.__options.GetFrameTitle())
            print(self.__options.GetFrameSize())
            print(self.__options.GetDezPlcs())
            print(self.__options.GetGDIBorder())
            print(self.__options.GetCursorWin())
            print(self.__options.GetCursorDraw())
            print(self.__options.GetCursorSelect())
            print(self.__options.GetDlgBorder())
            print(self.__options.GetDlgOptSize())
            print(self.__options.GetSbRange())
            print(self.__options.GetSbSize())
            print(self.__options.GetWheelEnlarge())
            print(self.__options.GetWheelReduce())
            print(self.__options.GetSensitive())
            print(self.__options.GetMaxPenStyleScale())
            print(self.__options.GetMaxUserScale())
            print(self.__options.GetSmartCursSize())
            print(self.__options.GetSmartCursPenSize())
            print(self.__options.GetSmartCursColPoint())
            print(self.__options.GetSmartCursColSegm())
            print(self.__options.GetSmartCursColElem())
            print(self.__options.GetSmartCursMarkPenSize())
            print(self.__options.GetPreviewColor())
            print(self.__options.GetPreviewPenSize())
            print(self.__options.GetPreviewPenStyle())
            print(self.__options.GetSelCol())
            print(self.__options.GetDlgAttrSize())
            print(self.__options.GetOptListWidths())
            print(self.__options.GetAttrListWidths())
            print(self.__options.GetDlgHelpSize())


    def OnExit(self, event):
        self.Close()


    def GetGdiSize(self):
        # dummy for the method, return framesize
        return(self.GetSize())


if __name__== '__main__':
    from lrzoptions import LorzeOptions
    from lrzselect import LorzeSelection
    from lrzgeohelp import LorzeGeoHelp
    from lrzdrawing import LorzeDrawing
    from lrzlog import LorzeLog
    from lrzvalchck import LorzeValueCheck
    from lrzattribute import LorzeAttribute
    from lrzdrawmanager import LorzeDrawManager
    from lrzdlghelper import LorzeDlgHelper
    from lrzgdipnl import GraphicPanel

    app= wx.App()

    # internationalization
    wxloc= wx.Locale()
    wxloc.AddCatalogLookupPathPrefix('./in18')
    # get system language ('xx_XX', 'CHARSET')
    wxlang= locale.getdefaultlocale()
    wxlang= wxlang[0][:2]
    # select translations
    if wxlang== 'de':
        wxloc.AddCatalog('lorze_de')

    frame= TestFrame()
    app.MainLoop()

