#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

from lrzdrawline import DrawLine


class LorzeDrawing():
    # Drawing object for LORZE erasandcad

    def __init__(self, attribute, log, geohelp):
        # LorzeAttribute
        self.__attribute= attribute

        # log object
        self.__log= log

        # geometry helper object
        self.__geohelp= geohelp

        # string for lorzelog
        self.__logtext= ''

        # float: outline rectangle x1, y1, x2, y2
        self.__recx, self.__recy, self.__recw, self.__rech= 0.0, 0.0, 0.0, 0.0

        # dictionairy: drawing elements, key= id, value= drawing element
        self.__elements= {}

        # lists of free ids, has the list only one value: from this ongoing, B= block, C= circle, E= ellipse, H= hatch, K= thickness, L= line, M= measure, O= clone, P= point, T= text, V= view
        self.__newidB= [0]
        self.__newidC= [0]
        self.__newidE= [0]
        self.__newidH= [0]
        self.__newidK= [0]
        self.__newidL= [0]
        self.__newidM= [0]
        self.__newidO= [0]
        self.__newidP= [0]
        self.__newidT= [0]
        self.__newidV= [0]

        # dictionairy, for indexing lists with free ids: B= block, C= circle,  E= ellipse, H= hatch, K= thickness, L= line, M= measure, O= clone, P= point, T= text, V= view
        self.__newids= {'B':self.__newidB, 'C':self.__newidC, 'E':self.__newidE, 'H':self.__newidH, 'K':self.__newidK, 'L':self.__newidL, 'M':self.__newidM, 'O':self.__newidO, 'P':self.__newidP, 'T':self.__newidT, 'V':self.__newidV}


    def SetRec(self, recx, recy, recw, rech):
        # set outline rectangle (float)
        self.__recx= float(recx)
        self.__recy= float(recy)
        self.__recw= float(recw)
        self.__rech= float(rech)


    def GetRec(self):
        # get outline rectangle (float)
        return(self.__recx, self.__recy, self.__recw, self.__rech)


    def Add(self, addlist):
        # adding drawing elements (list) / list: [('typ', attributes), ..] / typ: B (block), C (circle), E (ellipse), H (hatch), K (thickness), L (line), M (measure), O (clone), P (point), T (text), V (view) / attributes: (color, layer, style, width, coords) - coords: (x1, y1, x2, y2 ..)

        for i in addlist:
            # get typ & attributes
            typ, attributes= i
            color, layer, style, width, coords= attributes

            # logtext
            self.__logtext= 'LorzeDrawing.Add([('+ typ+ ', ('+ color+ ', '+ layer+ ', '+ style+ ', '+ width+ ', ('
            for i in coords:
                if i!= 0:
                    self.__logtext+= ', '+ str(i)
                else:
                    self.__logtext+= str(i)
            self.__logtext+= '))]'

            if typ== 'B':
                # add block
                pass

            elif typ== 'C':
                # add circle
                pass

            elif typ== 'E':
                # add ellipse
                pass

            elif typ== 'H':
                # add hatch
                pass

            elif typ== 'K':
                # add thickness
                pass

            elif typ== 'L':
                # add line
                line= DrawLine(self.__geohelp)
                lrzid= self.GetFreeId('L')
                self.__logtext+= '|New ID: '+ lrzid

                line= self.SetBaseAttributes(line, lrzid, color, layer, style, width)
                line.SetCoords(coords[0], coords[1], coords[2], coords[3])
                self.__elements[lrzid]= line

                self.__log.AddEntry(self.__logtext)

            elif typ== 'M':
                # add measure
                pass

            elif typ== 'O':
                # add clone
                pass

            elif typ== 'P':
                # add point
                pass

            elif typ== 'T':
                # add text
                pass

            elif typ== 'V':
                # add view
                pass


    def Update(self, updatelist):
        # updateing drawing elements (list) / updatelist: [(lorze-id, new_attributes), ..] / new_attributes: (color, layer, style, width, coords) - coords: (x1, y1, x2, y2 ..)
        for i in updatelist:
            # get id & attributes
            lrzid, attributes= i
            color, layer, style, width, coords= attributes

            # logtext
            self.__logtext= 'LorzeDrawing.Update([('+ lrzid+ ', ('+ color+ ', '+ layer+ ', '+ style+ ', '+ width+ ', ('
            for i in coords:
                if i!= 0:
                    self.__logtext+= ', '+ str(i)
                else:
                    self.__logtext+= str(i)
            self.__logtext+= '))]'

            typ= lrzid[0]

            if typ== 'B':
                # update block
                pass

            elif typ== 'C':
                # update circle
                pass

            elif typ== 'E':
                # update ellipse
                pass

            elif typ== 'H':
                # update hatch
                pass

            elif typ== 'K':
                # update thickness
                pass

            elif typ== 'L':
                # update line
                line= self.__elements[lrzid]
                line= self.SetBaseAttributes(line, lrzid, color, layer, style, width)
                line.SetCoords(coords[0], coords[1], coords[2], coords[3])
                self.__elements[lrzid]= line

                self.__log.AddEntry(self.__logtext)

            elif typ== 'M':
                # update measure
                pass

            elif typ== 'O':
                # update clone
                pass

            elif typ== 'P':
                # update point
                pass

            elif typ== 'T':
                # update text
                pass

            elif typ== 'V':
                # update view
                pass


    def Del(self, idlist):
        # delete drawing elements (list)
        for i in idlist:
            if i in self.__elements:
                # element with id is in the dictionairy
                del self.__elements[i]
                self.SetFreeId(i)
                self.__log.AddEntry('OK|LorzeDrawing.Del('+ i+ ')')
            else:
                # element with not in the dictionairy
                self.__log.AddEntry('FAIL|Id not in dictionary|LorzeDrawing.Del('+ i+ ')')


    def Get(self, idlist):
        # get drawing elements (list)
        elements= []
        for i in idlist:
            elements.append(self.__elements[i])
        return(elements)


    def GetElements(self):
        # get dictionairy with elements (dictionairy)
        return(self.__elements)


    def GetFreeId(self, typ):
        # get free id (list)
        if len(self.__newids[typ])== 1:
            # only one id: reading id, increase one
            lrzid= self.__newids[typ][0]
            self.__newids[typ][0]= lrzid+ 1
        else:
            # a few ids: reading id, then delete
            lrzid= self.__newids[typ][0]
            del self.__newids[typ][0]

        # 1st char is the type, the other are the number
        return(typ+ str(lrzid))


    def SetFreeId(self, lrzid):
        # mark id as free (string), 1st char is the type
        typ= lrzid[0]

        # the other chars ar a number
        value= int(lrzid[1:])

        # set id as free
        self.__newids[typ].insert(0, value)


    def SetBaseAttributes(self, element, lrzid, color, layer, style, width):
        # set basis attributes (LorzeDrawElement, attributes)
        element.SetId(lrzid)

        # default drawing attributes and all keys of the attribute dictionairys
        drawdefcolor= self.__attribute.GetDefaultColor()
        drawdeflayer= self.__attribute.GetDefaultLayer()
        drawdefstyle= self.__attribute.GetDefaultStyle()
        drawdefwidth= self.__attribute.GetDefaultWidth()
        colorkeys, layerkeys, stylekeys, widthkeys= self.__attribute.GetAllAttrNames()

        # color
        if color in colorkeys:
            element.SetColor(color)
        else:
            element.SetColor(drawdefcolor)
            self.__logtext+= '|color not in dictionairy, set default'

        #layer
        if layer in layerkeys:
            element.SetLayer(layer)
        else:
            element.SetLayer(drawdeflayer)
            self.__logtext+= '|layer not in dictionairy, set default'

        # style
        if style in stylekeys:
            element.SetStyle(style)
        else:
            element.SetStyle(drawdefstyle)
            self.__logtext+= '|style not in dictionairy, set default'

        # width
        if width in widthkeys:
            element.SetWidth(width)
        else:
            element.SetWidth(drawdefwidth)
            self.__logtext+= '|width not in dictionairy, set default'

        self.__logtext= 'OK|'+ self.__logtext

        return(element)


    def GetUsedAttributes(self):
        # get amounts of used attributes, define amounts
        color= set()
        layer= set()
        style= set()
        width= set()

        # get attributes
        for lrzid, lrzelem in self.__elements.items():
            color.add(lrzelem.GetColor())
            layer.add(lrzelem.GetLayer())
            style.add(lrzelem.GetStyle())
            width.add(lrzelem.GetWidth())

        return(color, layer, style, width)


    def CheckAttrDict(self):
        # check all attributes in connection with the dictionairys, get default drawing attributes and all keys of the attribute dictionairys
        drawdefcolor= self.__attribute.GetDefaultColor()
        drawdeflayer= self.__attribute.GetDefaultLayer()
        drawdefstyle= self.__attribute.GetDefaultStyle()
        drawdefwidth= self.__attribute.GetDefaultWidth()
        colorkeys, layerkeys, stylekeys, widthkeys= self.__attribute.GetAllAttrNames()

        for lrzid, lrzelem in self.__elements.items():
            # get attributes
            color= lrzelem.GetColor()
            layer= lrzelem.GetLayer()
            style= lrzelem.GetStyle()
            width= lrzelem.GetWidth()

            # set default color, string for lorzelog
            logtext= 'LorzeDrawing.CheckAttrDict()'

            # check attributes, if they not exists
            if color not in colorkeys:
                lrzelem.SetColor(drawdefcolor)
                self.__log.AddEntry('FAIL|colour not in dictionairy: '+ color+ '|'+ logtext+ '|Set default: '+ drawdefcolor)

            if layer not in layerkeys:
                lrzelem.SetLayer(drawdeflayer)
                self.__log.AddEntry('FAIL|layer not in dictionairy: '+ layer+ '|'+ logtext+ '|Set default: '+ drawdeflayer)

            if style not in stylekeys:
                lrzelem.SetStyle(drawdefstyle)
                self.__log.AddEntry('FAIL|style not in dictionairy: '+ style+ '|'+ logtext+ '|Set default: '+ drawdefstyle)

            if width not in widthkeys:
                lrzelem.SetWidth(drawdefwidth)
                self.__log.AddEntry('FAIL|width not in dictionairy: '+ width+ '|'+ logtext+ '|Set default: '+ drawdefwidth)


if __name__== '__main__':
    from lrzoptions import LorzeOptions
    from lrzlog import LorzeLog
    from lrzvalchck import LorzeValueCheck
    from lrzattribute import LorzeAttribute
    from lrzgeohelp import LorzeGeoHelp

    options= LorzeOptions()
    log= LorzeLog()
    valchck= LorzeValueCheck()
    attribute= LorzeAttribute(options, log, valchck)
    geohelp= LorzeGeoHelp(options)

    drawing= LorzeDrawing(attribute, log, geohelp)
    drawing.SetRec(1, 2, 3, 4)
    print('Rec', drawing.GetRec())
    print('')
    print('Add 2 lines; Red, Test 1, Solid, 0.18 mm, 0, 0, 10, 10; White, Test 2, Dashed 1, 0.25 mm, 10, 10, 0, 0')
    line1= ('L', ('Red', 'Test 1', 'Solid', '0.18 mm', (0, 0, 10, 10)))
    line2= ('L', ('White', 'Test 2', 'Dashed 1', '0.25 mm', (10, 10, 0, 0)))
    drawing.Add([line1, line2])
    a= drawing.GetElements()
    for i in a.keys():
        print (i, a[i].GetId(), a[i].GetColor(), a[i].GetLayer(), a[i].GetStyle(), a[i].GetWidth(), a[i].GetCoords())
    print('')
    print('Get with ids: L1')
    for i in drawing.Get(['L1']):
        print (i.GetId(), i.GetColor(),i.GetLayer(), i.GetStyle(), i.GetWidth(), i.GetCoords())
    print('')
    print('Update lines with ids: L0, L1; White, error, Dashed 1, error, 0.1, 0.1, 10.1, 10.1; error, Test 1, error, 0.18 mm, 11, 11, 1, 1')
    update1= ('L0', ('White', 'error', 'Dashed 1', 'error', (0.1, 0.1, 10.1, 10.1)))
    update2= ('L1', ('error', 'Test 1', 'error', '0.18 mm', (11, 11, 1, 1)))
    drawing.Update([update1, update2])
    a= drawing.GetElements()
    for i in a.keys():
        print (i, a[i].GetId(), a[i].GetColor(), a[i].GetLayer(), a[i].GetStyle(), a[i].GetWidth(), a[i].GetCoords())
    print('')
    print('Add 2 lines; Red, Test 2, Dashed 1, 0.18 mm; White, Test 1, Solid, 0.25 mm / remove 2 lines; L0, L2')
    line3= ('L', ('Red', 'Test 2', 'Dashed 1', '0.18 mm', (12, 12, 2, 2)))
    line4= ('L', ('White', 'Test 1', 'Solid', '0.25 mm', (13, 13, 3, 3)))
    drawing.Add([line3, line4])
    drawing.Del(['L0', 'L2'])
    print('After deleting element L0 and L2')
    a= drawing.GetElements()
    for i in a.keys():
        print (i, a[i].GetId(), a[i].GetColor(), a[i].GetLayer(), a[i].GetStyle(), a[i].GetWidth(), a[i].GetCoords())
    print('')
    print('Add 2 line; error, error, error, error, 14, 14, 4, 4; error, error, error, error, 14, 14, 4, 4')
    line5= ('L', ('error', 'error', 'error', 'error', (14, 14, 4, 4)))
    drawing.Add([line5, line5])
    a= drawing.GetElements()
    for i in a.keys():
        print (i, a[i].GetId(), a[i].GetColor(), a[i].GetLayer(), a[i].GetStyle(), a[i].GetWidth(), a[i].GetCoords())
    print('')
    drawing.Del(['L5'])
    print('Trying to delete element L5')
    print('')
    print('Get used attributes')
    print(drawing.GetUsedAttributes())
    print('')
    print('check attribute dictionairy')
    drawing.CheckAttrDict()
    print('')
    print('LorzeLog')
    for i in log.GetLog():
        print(i)
