#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.


class DrawLine():
    # line object of LORZE erasandcad

    def __init__(self, geohelp):
        # geometry helper object
        self.__geohelp= geohelp

        # id, string
        self.__lrzid= ''

        # coordinates, float, x1, y1, x2, y2
        self.__x1, self.__y1, self.__x2, self.__y2= 0.0, 0.0, 0.0, 0.0

        # colour, string
        self.__color= ''

        # layer, string
        self.__layer= ''

        # line style, string
        self.__style= ''

        # line width, string
        self.__width= ''


    def SetId(self, lrzid):
        self.__lrzid= lrzid


    def GetId(self):
        return(self.__lrzid)


    def SetCoords(self, x1, y1, x2, y2):
        self.__x1= float(x1)
        self.__y1= float(y1)
        self.__x2= float(x2)
        self.__y2= float(y2)


    def GetCoords(self):
        return(self.__x1, self.__y1, self.__x2, self.__y2)


    def GetRec(self):
        # get outline rectangle, x, y, w, h
        return(self.__geohelp.RectXYtoWH(self.__x1, self.__y1, self.__x2, self.__y2))


    def SetColor(self, color):
        self.__color= (color)


    def GetColor(self):
        return(self.__color)


    def SetLayer(self, layer):
        self.__layer= layer


    def GetLayer(self):
        return(self.__layer)


    def SetStyle(self, style):
        self.__style= style


    def GetStyle(self):
        return(self.__style)


    def SetWidth(self, width):
        self.__width= width


    def GetWidth(self):
        return(self.__width)


    def GetX1(self):
        return(self.__x1)


    def GetY1(self):
        return(self.__y1)


    def GetX2(self):
        return(self.__x2)


    def GetY2(self ):
        return(self.__y2)


if __name__== '__main__':
    from lrzgeohelp import LorzeGeoHelp
    from lrzoptions import LorzeOptions

    options= LorzeOptions()
    geohelp= LorzeGeoHelp(options)

    line= DrawLine(geohelp)
    line.SetId('L123456789')
    line.SetCoords(-20, -30, 20, 30)
    line.SetColor('Black')
    line.SetLayer('Test')
    line.SetStyle('Solid')
    line.SetWidth('1 mm')

    print('Element-ID:', line.GetId())
    print('coordinates:', line.GetCoords())
    print('X1:', line.GetX1())
    print('Y1:', line.GetY1())
    print('X2:', line.GetX2())
    print('Y2:', line.GetY2())
    print('outline rectangle (x, y, w, h):', line.GetRec())
    print('colour:', line.GetColor())
    print('layer:', line.GetLayer())
    print('penstyle:', line.GetStyle())
    print('pensize:', line.GetWidth())
