#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

from lrzdrawing import LorzeDrawing


class LorzeDrawManager():
    # manager for drawing objects in LORZE erasandcad

    def __init__(self, attribute, log, geohelp):
        # LorzeAttribute
        self.__attribute= attribute

        # log object
        self.__log= log

        # geometry helper object
        self.__geohelp= geohelp

        # default drawing, text
        self.__defdraw= ''

        # dictionairy with LorzeDrawing()
        self.__drawings= {}


    def GetDefaultDraw(self):
        return(self.__defdraw)


    def SetDefaultDraw(self, drawkey):
        # set default drawing
        drawlist= self.__drawings.keys()

        # string for lorzelog
        logtext= 'LorzeAttribute.SetDefaultDraw('+ drawkey+ ')'

        if drawkey in drawlist:
            self.__log.AddEntry('OK|'+ logtext+ '|Old= '+ self.__defdraw)
            self.__defdraw= drawkey
        else:
            if len(drawlist)> 0:
                logtext+= '|Old= '+ self.__defdraw
                self.__defdraw= drawlist[0]
                self.__log.AddEntry('OK|'+ logtext+ '|drawing not exist, set '+ self.__defdraw)
            else:
                self.__log.AddEntry('FAIL|no drawing available|'+ logtext)


    def AddDrawing(self, key):
        # add new drawing to drawings dictionairy with key
        self.__drawings[key]= LorzeDrawing(self.__attribute, self.__log, self.__geohelp)
        self.__log.AddEntry('OK|LorzeDrawManager.AddDrawing('+ key+ ')')


    def GetAllDrawings(self):
        # return drawings dictionairy
        return(self.__drawings)


    def RemoveDrawing(self, key):
        # remove drawing with the caption of the dictionairy
        if key in self.__drawings:
            del self.__drawings[key]
            self.__log.AddEntry('OK|LorzeDrawManager.RemoveDrawing('+ key+ ')')
        else:
            self.__log.AddEntry('FAIL|Drawing not in dictionairy|LorzeDrawManager.RemoveDrawing('+ key+ ')')


    def GetUsedAttr(self):
        # Get lists of uesd attributes from all drawings / dictionairys to store attributes and in which drawing they are used / key= attribute name, value= drawing
        usedcolordict= {}
        usedlayerdict= {}
        usedstyledict= {}
        usedwidthdict= {}

        for key, draw in self.__drawings.items():
            # each LorzeDrawing, get amounts of used attributes
            color, layer, style, width= draw.GetUsedAttributes()

            for i in color:
                if i not in usedcolordict:
                    usedcolordict[i]= key
                else:
                    usedcolordict[i]= usedcolordict[i]+ ', '+ key

            for i in layer:
                if i not in usedlayerdict:
                    usedlayerdict[i]= key
                else:
                    usedlayerdict[i]= usedlayerdict[i]+ ', '+ key

            for i in style:
                if i not in usedstyledict:
                    usedstyledict[i]= key
                else:
                    usedstyledict[i]= usedstyledict[i]+ ', '+ key

            for i in width:
                if i not in usedwidthdict:
                    usedwidthdict[i]= key
                else:
                    usedwidthdict[i]= usedwidthdict[i]+ ', '+ key

        # return used dictionairys
        return(usedcolordict, usedlayerdict, usedstyledict, usedwidthdict)


    def GetDrawing(self, key):
        # return drawing with key
        if key in self.__drawings:
            # key is in the dictionairy
            return(self.__drawings[key])
        else:
            # key is not in the dictionairy
            self.__log.AddEntry('FAIL|Drawing not in dictionairy|LorzeDrawManager.GetDrawing('+ key+ ')')



if __name__== '__main__':
    from lrzoptions import LorzeOptions
    from lrzlog import LorzeLog
    from lrzvalchck import LorzeValueCheck
    from lrzattribute import LorzeAttribute
    from lrzgeohelp import LorzeGeoHelp

    options= LorzeOptions()
    log= LorzeLog()
    valchck= LorzeValueCheck()
    attribute= LorzeAttribute(options, log, valchck)
    geohelp= LorzeGeoHelp(options)

    drawman= LorzeDrawManager(attribute, log, geohelp)

    print('')
    print('LorzeDrawManager')
    print('----------------')
    print('')
    print('Set default drawing test, and then add drawing test, get drawings, get default drawing')
    drawman.SetDefaultDraw('test')
    drawman.AddDrawing('test')
    print(drawman.GetAllDrawings())
    print(drawman.GetDefaultDraw())
    print('')
    print('Add 2 new drawings, test 2 & test 3, get drawings')
    drawman.AddDrawing('test 2')
    drawman.AddDrawing('test 3')
    print(drawman.GetAllDrawings())
    print('')
    print('Remove drawing test2, trying to remove drawing error, get drawings')
    drawman.RemoveDrawing('test 2')
    drawman.RemoveDrawing('error')
    print(drawman.GetAllDrawings())

    color= [('Red', '255, 0, 0', 'base color'), ('Blue', '0, 0, 255', 'base color'), ('Green', '0, 128, 0', 'base color'), ('Yellow', '255, 255, 0', 'base color'), ('Gray', '128, 128, 128', 'base color'), ('White', '0, 0, 0', 'base color')]
    layer= [('Layer 1', 'default'), ('Layer 2', 'default'), ('Layer 3', 'default'), ('Layer 4', 'default'), ('Layer 5', 'default'), ('Layer 6', 'default')]
    style= [('Solid', 'SOLID', 'standard'), ('Dot', 'DOT', 'standard'), ('Long dash', 'LONG_DASH', 'standard'), ('Short dash', 'SHORT_DASH', 'standard'), ('Dot dash', 'DOT_DASH', 'standard'), ('Axis 13mm', '4, 2, 1, 2, 4', 'standard')]
    width= [('0.13mm', '0.13', 'Rotring'), ('0.18mm', '0.18', 'Rotring'), ('0.25mm', '0.25', 'Rotring'), ('0.35mm', '0.35', 'Rotring'), ('0.50mm', '0.50', 'Rotring'), ('0.70mm', '0.70', 'Rotring')]
    for i in color:
        attribute.AddColorEntry(i[0], i[1], i[2])
    for i in layer:
        attribute.AddLayerEntry(i[0], i[1])
    for i in style:
        attribute.AddStyleEntry(i[0], i[1], i[2])
    for i in width:
        attribute.AddWidthEntry(i[0], i[1], i[2])

    line1= ('L', ('Red', 'Layer 1', 'Solid', '0.13mm', (1, 1, 10, 10)))
    line2= ('L', ('Blue', 'Layer 2', 'Dot', '0.18mm', (2, 2, 20, 20)))
    line3= ('L', ('Green', 'Layer 3', 'Long dash', '0.25mm', (3, 3, 30, 30)))
    line4= ('L', ('Yellow', 'Layer 4', 'Short dash', '0.35mm', (4, 4, 40, 40)))
    line5= ('L', ('Gray', 'Layer 5', 'Dot dash', '0.50mm', (5, 5, 50, 50)))
    line6= ('L', ('White', 'Layer 6', 'Axis 13mm', '0.70mm', (6, 6, 60, 60)))

    print('')
    print('Trying to get drawing error')
    drawman.GetDrawing('error')
    print('')
    print('Add 3 lines to drawing test and 3 lines to drawing test 3')
    drawman.SetDefaultDraw('test')
    drawman.GetDrawing(drawman.GetDefaultDraw()).Add([line1, line2, line3])
    drawman.SetDefaultDraw('test 2')
    drawman.SetDefaultDraw('test 3')
    drawman.GetDrawing(drawman.GetDefaultDraw()).Add([line4, line5, line6])
    print('')
    print('GetUsedAttr of all drawings')
    colordict, layerdict, styledict, widthdict= drawman.GetUsedAttr()
    print(colordict, layerdict, styledict, widthdict)

    print('')
    print('LorzeLog')
    for i in log.GetLog():
        print(i)


