#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx
from lrzsmartcurs import SmartCursor


class GraphicPanel(wx.Panel):
    # Graphic panel to the GUI of LORZE erasandcad

    def __init__(self, parent, options, geohelp, log, valchck, attribute, drawman, selection):
        # subclass
        wx.Panel.__init__(self, parent)

        # parent class: wx.Frame
        self.__parent= parent

        # LorzeOptions() of parent class
        self.__options= options

        # geometrie helper object
        self.__geohelp= geohelp

        # log object
        self.__log= log

        # LorzeValueCheck
        self.__valchck= valchck

        # LorzeAttribute
        self.__attribute= attribute

        # manager for drawings
        self.__drawman= drawman

        # selection object
        self.__selection= selection

        # wx.Cursors: dictionairy to index
        self.__cursorsdict= {'wx.CURSOR_ARROW': wx.CURSOR_ARROW, 'wx.CURSOR_RIGHT_ARROW': wx.CURSOR_RIGHT_ARROW, 'wx.CURSOR_BLANK': wx.CURSOR_BLANK, 'wx.CURSOR_BULLSEYE': wx.CURSOR_BULLSEYE, 'wx.CURSOR_CHAR': wx.CURSOR_CHAR, 'wx.CURSOR_CROSS': wx.CURSOR_CROSS, 'wx.CURSOR_HAND': wx.CURSOR_HAND, 'wx.CURSOR_IBEAM': wx.CURSOR_IBEAM, 'wx.CURSOR_LEFT_BUTTON': wx.CURSOR_LEFT_BUTTON, 'wx.CURSOR_MAGNIFIER': wx.CURSOR_MAGNIFIER, 'wx.CURSOR_MIDDLE_BUTTON': wx.CURSOR_MIDDLE_BUTTON, 'wx.CURSOR_NO_ENTRY': wx.CURSOR_NO_ENTRY, 'wx.CURSOR_PAINT_BRUSH': wx.CURSOR_PAINT_BRUSH, 'wx.CURSOR_PENCIL': wx.CURSOR_PENCIL, 'wx.CURSOR_POINT_LEFT': wx.CURSOR_POINT_LEFT, 'wx.CURSOR_POINT_RIGHT': wx.CURSOR_POINT_RIGHT, 'wx.CURSOR_QUESTION_ARROW': wx.CURSOR_QUESTION_ARROW, 'wx.CURSOR_RIGHT_BUTTON': wx.CURSOR_RIGHT_BUTTON, 'wx.CURSOR_SIZENESW': wx.CURSOR_SIZENESW, 'wx.CURSOR_SIZENS': wx.CURSOR_SIZENS, 'wx.CURSOR_SIZENWSE': wx.CURSOR_SIZENWSE, 'wx.CURSOR_SIZEWE': wx.CURSOR_SIZEWE, 'wx.CURSOR_SIZING': wx.CURSOR_SIZING, 'wx.CURSOR_SPRAYCAN': wx.CURSOR_SPRAYCAN, 'wx.CURSOR_WAIT': wx.CURSOR_WAIT, 'wx.CURSOR_WATCH': wx.CURSOR_WATCH, 'wx.CURSOR_ARROWWAIT': wx.CURSOR_ARROWWAIT, 'wx.CURSOR_DEFAULT': wx.CURSOR_DEFAULT}

        # wx.Border: dictionairy to index
        self.__bordersdict= {'wx.SIMPLE_BORDER': wx.SIMPLE_BORDER, 'wx.RAISED_BORDER': wx.RAISED_BORDER, 'wx.SUNKEN_BORDER': wx.SUNKEN_BORDER, 'wx.NO_BORDER': wx.NO_BORDER}

        # wx.Pen styles: dictionairy to index
        self.__penstyldict= {'wx.SOLID':wx.SOLID, 'wx.DOT':wx.DOT, 'wx.LONG_DASH':wx.LONG_DASH, 'wx.SHORT_DASH':wx.SHORT_DASH, 'wx.DOT_DASH':wx.DOT_DASH, 'wx.USER_DASH':wx.USER_DASH}

        # panel clicked, x & y
        self.__xclick, self.__yclick= 0, 0

        # origin x & y for drawing client
        self.__originx, self.__originy= 0, 0

        # origin x & y for pan with mouse-wheel
        self.__originxpan, self.__originypan= 0, 0

        # x & y position scrollbar
        self.__scrollposx, self.__scrollposy= 0, 0

        # factor for line width
        self.__linewidthfactor= 1

        # drawing x & y centre, pixel on the panel
        self.__drawcentx, self.__drawcenty= 0, 0

        # pixels per millimeter, get self.__ppmm
        self.GetPpmm()

        # scale factor for drawing
        self.__userscale= self.__ppmm

        # border graphic panel
        border= self.__bordersdict[self.__options.GetGDIBorder()]

        # smart cursor object, pixel size smart cursor & smart cursor flag
        self.__smartcurs= SmartCursor(self.__geohelp)
        self.__smartflag= False

        # flag
        self.__showlinestyle= True

        # mouse x- and y- coordinates in user units
        self.__mousex, self.__mousey= 0.0, 0.0

        # preview attributes: counter for clicks and coordinates
        self.__prevclick= 0
        self.__prevx1, self.__prevy1, self.__prevx2, self.__prevy2= 0, 0, 0, 0

        # mouse action, shift key, left dragging
        self.__action= ''
        self.__shift= False

        # bindings
        self.Bind(wx.EVT_MOTION, self.OnMouseMove)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.Bind(wx.EVT_MIDDLE_DOWN, self.OnMiddleDown)
        self.Bind(wx.EVT_MOUSEWHEEL, self.OnWheel)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_ENTER_WINDOW, self.EnterGdi)
        self.Bind(wx.EVT_LEAVE_WINDOW, self.LeaveGdi)
        self.Bind(wx.EVT_SCROLLWIN, self.OnScroll)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)

        # scrollbars
        self.SetScrollbar(wx.VERTICAL, 0, self.__options.GetSbSize(), self.__options.GetSbRange())
        self.SetScrollbar(wx.HORIZONTAL, 0, self.__options.GetSbSize(), self.__options.GetSbRange())


    def OnLeftDown(self, event):
        # left mouse buttuon, click is no dragging

        if self.__prevclick== 0:
            # 1st time clicked
            if self.__action== 'elementsel':
                # elementselection
                if self.__smartcurs.GetType()!= '':
                    # smart cursor found, get lorze id & drawing key
                    lrzid= self.__smartcurs.GetLrzId()
                    drawkey= self.__smartcurs.GetKey()
                    if event.ShiftDown():
                        # shift is pressed
                        self.__selection.AddRemove(lrzid, drawkey)
                    else:
                        # shift is not pressed
                        self.__selection.Clear()
                        self.__selection.AddRemove(lrzid, drawkey)
                    self.Refresh()
                else:
                    # not clicked on element, selection with rect, set 1st point
                    self.__prevclick+= 1
                    self.__prevx1, self.__prevy1= self.__mousex, self.__mousey
            else:
                # other commands, set 1st point
                self.__prevclick+= 1
                self.__prevx1, self.__prevy1= self.__mousex, self.__mousey

        elif self.__prevclick== 1:
            # 2nd time clicked
            self.__prevclick+= 1
            self.__prevx2, self.__prevy2= self.__mousex, self.__mousey

            if self.__action in ('elementsel', 'pointsel', 'line'):
                # 2 click action, 2nd point
                if self.__action== 'elementsel':
                    # select elements and set action elementsel again
                    if event.ShiftDown():
                        # shift is pressed
                        self.__selection.ElemSelRect(self.__prevx1, self.__prevy1, self.__prevx2, self.__prevy2)
                    else:
                        # shift is not pressed
                        self.__selection.Clear()
                        self.__selection.ElemSelRect(self.__prevx1, self.__prevy1, self.__prevx2, self.__prevy2)
                    self.SetAction('elementsel')

                elif self.__action== 'pointsel':
                    pass
                    # select points and set action pointsel again
                    # self.__selection.PointSelRect(x1, y1, x2, y2)
                    # self.SetAction('pointsel')

                elif self.__action== 'line':
                    # Add with addlist: [('typ', attributes), ..]
                    self.__drawman.GetDrawing(self.__drawman.GetDefaultDraw()).Add([('L', (self.__attribute.GetDefaultColor(), self.__attribute.GetDefaultLayer(), self.__attribute.GetDefaultStyle(), self.__attribute.GetDefaultWidth(), (self.__prevx1, self.__prevy1, self.__prevx2, self.__prevy2)))])

                    self.SetAction('line')
                self.Refresh()
                # event.AltDown()
                # event.ControlDown()


    def OnRightDown(self, event):
        # right mouse button
        pass


    def OnMiddleDown(self, event):
        # middle mouse button, mouse-position
        self.__xclick, self.__yclick= event.GetPosition()

        # old origin x & y
        self.__originxpan, self.__originypan= self.__originx, self.__originy


    def OnWheel(self, event):
        # mouse wheel
        wheel= event.GetWheelRotation()
        if wheel > 0:
            self.__userscale*= self.__options.GetWheelEnlarge()
            if self.__userscale> self.__options.GetMaxUserScale():
                self.__userscale= self.__options.GetMaxUserScale()
        else:
            self.__userscale*= self.__options.GetWheelReduce()
        self.Refresh()


    def OnMouseMove(self, event):
        # mouse movement, mouse-position, pixel on the panel
        xpanel, ypanel= event.GetPosition()
        paintflag= False

        if event.Dragging() and event.MiddleIsDown():
            # dragging with wheel (middle), move x and y origin
            self.__originx= self.__originxpan+ xpanel- self.__xclick
            self.__originy= self.__originypan+ ypanel- self.__yclick
            paintflag= True

        elif event.Dragging() and event.RightIsDown():
            # dragging with right
            pass

        else:
            # no dragging, drawing client, set origin & axis
            dc= wx.ClientDC(self)
            dc.SetDeviceOrigin(self.__originx, self.__originy)
            dc.SetAxisOrientation(True, True)

            # calculate coordinates for smart curosr
            scpx= round(dc.DeviceToLogicalX(xpanel)/ self.__userscale, self.__options.GetDezPlcs())
            scpy= round(dc.DeviceToLogicalY(ypanel)/ self.__userscale, self.__options.GetDezPlcs())
            sctolerance= round(self.__options.GetSensitive()/ self.__userscale, self.__options.GetDezPlcs())
            found= self.__smartcurs.Get(scpx, scpy, sctolerance)

            if found== True:
                # position from smart cursor
                self.__mousex, self.__mousey= self.__smartcurs.GetCoord()

                # set smartflag True: Refresh Panel when smart cursor not found
                self.__smartflag= True
                paintflag= True
            else:
                # position from mouse
                self.__mousex= round(dc.DeviceToLogicalX(xpanel)/ self.__userscale, self.__options.GetDezPlcs())
                self.__mousey= round(dc.DeviceToLogicalY(ypanel)/ self.__userscale, self.__options.GetDezPlcs())
                if self.__smartflag== True:
                    # set smartflag False: Not Refresh Panel when not found
                    self.__smartflag= None
                    paintflag= True

            if event.ShiftDown()== True and self.__prevclick> 0 and self.__action not in ('elementsel', 'pointsel'):
                # orthogonal drawing
                w= abs(self.__mousex- self.__prevx1)
                h= abs(self.__mousey- self.__prevy1)
                if w< h:
                    # 90 or 270 degrees
                    self.__mousex= self.__prevx1
                else:
                    # 0 or 180 degrees
                    self.__mousey= self.__prevy1

            # actualize status bar information
            self.__parent.SetStatusMouse(self.__mousex, self.__mousey)

        if paintflag== True or self.__prevclick> 0:
            # on changes paint event
            self.Refresh()

        #if event.Dragging() and event.LeftIsDown():
            # dragging with left


    def OnPaint(self, event):
        # drawing graphic panel, drawing client
        dc= wx.PaintDC(self)

        # set origin
        dc.SetDeviceOrigin(self.__originx, self.__originy)

        # set axis
        dc.SetAxisOrientation(True, True)

        # panel width, height
        scW, scH= self.__parent.GetGdiSize()

        # screen rect x1, x2, y1, y2
        scrx= round(dc.DeviceToLogicalX(0)/ self.__userscale, self.__options.GetDezPlcs())
        scry= round(dc.DeviceToLogicalY(scH)/ self.__userscale, self.__options.GetDezPlcs())
        scx2= round(dc.DeviceToLogicalX(scW)/ self.__userscale, self.__options.GetDezPlcs())
        scy2= round(dc.DeviceToLogicalY(0)/ self.__userscale, self.__options.GetDezPlcs())
        scrw= scx2- scrx
        scrh= scy2- scry

        # check userscale for penstyles (problems with paint)
        if self.__userscale> self.__options.GetMaxPenStyleScale():
            # ignore penstyles, all paint in wx.SOLID
            self.__showlinestyle= False

        else:
            # show penstyles
            self.__showlinestyle= True

        # amounts for x & y points
        xpoints= set()
        ypoints= set()

        for drawkey, drawing in self.__drawman.GetAllDrawings().items():
            # each LorzeDrawing, get drawing elements

            for lrzid, lrzelem in drawing.GetElements().items():
                # get rec coordinates and add to points
                ex, ey, ew, eh= lrzelem.GetRec()
                xpoints.add(ex)
                xpoints.add(ex+ ew)
                ypoints.add(ey)
                ypoints.add(ey+ eh)

                if self.__geohelp.RectInRect((scrx, scry, scrw, scrh), (ex, ey, ew, eh)):
                    # element rect in display rect, add element to smartcursor
                    self.__smartcurs.AddElement(lrzelem, drawkey)

                    # paint element
                    self.PaintDrawing(dc, lrzelem, drawkey)

        # center of the drawing elements
        if len(xpoints)> 0:
            minxpoints= min(xpoints)
            maxxpoints= max(xpoints)
            minypoints= min(ypoints)
            maxypoints= max(ypoints)
            cxpoints= ((maxxpoints- minxpoints)/ 2)+ minxpoints
            cypoints= ((maxypoints- minypoints)/ 2)+ minypoints

            # x & y centre in pixels
            self.__drawcentx= dc.LogicalToDeviceX(cxpoints/ self.__userscale)
            self.__drawcenty= dc.LogicalToDeviceY(cypoints/ self.__userscale)
        else:
            self.__drawcentx= 0
            self.__drawcenty= 0

        # paint selection
        self.PaintSelect(dc)

        # smart cursor
        if self.__smartcurs.GetType()!= '':
            # smart cursor found, paint in SmartCursor()
            self.PaintSmartCursor(dc)
            pass

        # preview
        self.PaintPreview(dc)

        # set up scrollbars
        self.SetScrollBars()


    def EnterGdi(self, event):
        # cursor is on the graphic panel
        if self.__action== 'elementsel' or self.__action== 'pointsel':
            self.SetCursor(wx.StockCursor(self.__cursorsdict[self.__options.GetCursorSelect()]))
        else:
            self.SetCursor(wx.StockCursor(self.__cursorsdict[self.__options.GetCursorDraw()]))
        self.SetFocus()


    def LeaveGdi(self, event):
        # cursor is leaving the graphic panel
        self.SetCursor(wx.StockCursor(self.__cursorsdict[self.__options.GetCursorWin()]))


    def SetScrollBars(self):
        # set scrollbars to the panel, width, height in pixel
        wdev, hdev= self.__parent.GetGdiSize()
        wdev= float(wdev)
        hdev= float(hdev)

        # set scrollpos to drawing centre
        self.__scrollposx= self.__drawcentx* (self.__options.GetSbRange()/ wdev)
        self.__scrollposy= self.__drawcenty* (self.__options.GetSbRange()/ hdev)

        # check under- and overflow
        if self.__scrollposx< 0:
            self.__scrollposx= 0
        elif self.__scrollposx> self.__options.GetSbRange():
            self.__scrollposx= self.__options.GetSbRange()
        if self.__scrollposy< 0:
            self.__scrollposy= 0
        elif self.__scrollposy> self.__options.GetSbRange():
            self.__scrollposy= self.__options.GetSbRange()

        # set scrollbars
        self.SetScrollPos(wx.HORIZONTAL, self.__scrollposx, True)
        self.SetScrollPos(wx.VERTICAL, self.__scrollposy, True)


    def OnScroll(self, event):
        # scrollbar event, value of the event
        value= event.GetPosition()

        # orientation of the event
        orientation= event.GetOrientation()

        # width, height in pixel
        wdev, hdev= self.__parent.GetGdiSize()
        wdev= float(wdev)
        hdev= float(hdev)

        if orientation== wx.HORIZONTAL:
            # horizontal scrollbar
            self.__originx+= (value- self.__scrollposx)* (self.__options.GetSbRange()/ wdev)

        else:
            # vertical scrollbar
            self.__originy+= (value- self.__scrollposy)* (self.__options.GetSbRange()/ hdev)

        # actualize graphic panel
        self.Refresh()


    def GetPpmm(self):
        # get pixels per mm, drawing client
        dc= wx.ClientDC(self)

        # pixels per inch
        ppi= float(dc.GetPPI()[0])

        # pixels per mm
        self.__ppmm= ppi/ 25.4


    def OnKeyDown(self, event):
        key= event.GetKeyCode()

        if key== wx.WXK_ESCAPE:
            # esc: return to select/ unselect elements
            self.__parent.CheckMenu('elementsel')

        elif key== wx.WXK_TAB:
            # tabulator: toggle coordinates
            pass


    def SetAction(self, action):
        # set action in GraphicPanel and set preview attributes new, set action
        self.__action= action

        # preview attributes: counter for clicks and coordinates
        self.__prevclick= 0
        self.__prevx1, self.__prevy1, self.__prevx2, self.__prevy2= 0, 0, 0, 0

        if self.__action== 'elementsel':
            # element selection
            self.__parent.SetStatusInfo(u'Select/ deselect elements')

        elif self.__action== 'pointsel':
            # point selection
            self.__parent.SetStatusInfo(u'Select/ deselect points')

        elif self.__action== 'line':
            # draw lines
            self.__parent.SetStatusInfo(u'Draw line: Enter 1st point')


    def GetAction(self):
        return (self.__action)


    def PaintDrawing(self, dc, lrzelem, drawkey):
        # paint lorze drawing elements
        drawing= self.__drawman.GetDrawing(drawkey)

        if self.__showlinestyle== True:
            # show line styles
            pen= wx.Pen(self.__attribute.GetWxColor(lrzelem.GetColor()), self.__attribute.GetWxWidth(lrzelem.GetWidth())* self.__userscale* self.__linewidthfactor, self.__penstyldict[self.__attribute.GetWxStyle(lrzelem.GetStyle())[0]])

            if self.__attribute.GetWxStyle(lrzelem.GetStyle())[0]== 'wx.USER_DASH':
                # user penstyle
                pen.SetDashes(self.__attribute.GetWxStyle(lrzelem.GetStyle())[1])

            dc.SetPen(pen)

        else:
            # don't show penstyles
            dc.SetPen(wx.Pen(self.__attribute.GetWxColor(lrzelem.GetColor()), self.__attribute.GetWxWidth(lrzelem.GetWidth())* self.__userscale* self.__linewidthfactor, wx.SOLID))

        if lrzelem.GetId()[0]== 'L':
            # line
            dc.DrawLine(lrzelem.GetX1()* self.__userscale,
            lrzelem.GetY1()* self.__userscale, lrzelem.GetX2()* self.__userscale, lrzelem.GetY2()* self.__userscale)


    def PaintSmartCursor(self, dc):
        # paint point marker, set half mark size in mm
        sh= self.__options.GetSmartCursSize()/ 2/ self.__userscale

        # x1, x2, y1, y2 of the mark in mm
        x1= (self.__smartcurs.GetX()- sh)* self.__userscale
        x2= (self.__smartcurs.GetX()+ sh)* self.__userscale
        y1= (self.__smartcurs.GetY()- sh)* self.__userscale
        y2= (self.__smartcurs.GetY()+ sh)* self.__userscale

        # get element with the smart cursor
        drawing= self.__drawman.GetDrawing(self.__smartcurs.GetKey())
        elementsdict= drawing.GetElements()
        element= elementsdict[self.__smartcurs.GetLrzId()]

        # pen marked element
        dc.SetPen(wx.Pen(self.__options.GetSmartCursColElem(), self.__attribute.GetWxWidth(element.GetWidth())* self.__userscale* self.__linewidthfactor* self.__options.GetSmartCursMarkPenSize(),  wx.SOLID))

        # paint marked element
        if element.GetId()[0]== 'L':
            # paint line
            dc.DrawLine(element.GetX1()* self.__userscale,  element.GetY1()* self.__userscale, element.GetX2()* self.__userscale, element.GetY2()* self.__userscale)

        if self.__smartcurs.GetType()== 'P':
            # pen point mark
            dc.SetPen(wx.Pen(self.__options.GetSmartCursColPoint(), self.__options.GetSmartCursPenSize(), wx.SOLID))
        elif self.__smartcurs.GetType()== 'S':
            # pen segment mark
            dc.SetPen(wx.Pen(self.__options.GetSmartCursColSegm(), self.__options.GetSmartCursPenSize(), wx.SOLID))

        # point mark
        dc.DrawLine(x1, y1, x2, y2)
        dc.DrawLine(x1, y2, x2, y1)


    def PaintPreview(self, dc):
        # paint preview, set pen
        dc.SetPen(wx.Pen(self.__options.GetPreviewColor(), self.__options.GetPreviewPenSize(), self.__penstyldict[self.__options.GetPreviewPenStyle()]))

        # set brush
        dc.SetBrush(wx.Brush('#000000', wx.TRANSPARENT))

        # select & paint preview
        if self.__action== 'elementsel':
            if self.__prevclick== 1:
                # 1st point of rect, rect coordinates
                if self.__prevx1< self.__mousex:
                    x= self.__prevx1* self.__userscale
                    w= (self.__mousex- self.__prevx1)* self.__userscale
                else:
                    x= self.__mousex* self.__userscale
                    w= (self.__prevx1 - self.__mousex)* self.__userscale

                if self.__prevy1< self.__mousey:
                    y= self.__prevy1* self.__userscale
                    h= (self.__mousey- self.__prevy1)* self.__userscale
                else:
                    y= self.__mousey* self.__userscale
                    h= (self.__prevy1 - self.__mousey)* self.__userscale
                dc.DrawRectangle(x, y, w, h)

        elif self.__action== 'line':
            if self.__prevclick== 1:
                # draw line from 1st point to current position
                dc.DrawLine(self.__prevx1* self.__userscale, self.__prevy1* self.__userscale, self.__mousex* self.__userscale, self.__mousey* self.__userscale)


    def PaintSelect(self, dc):
        # paint selected elements in wxpython drawing client
        for i in self.__selection.GetElements():
            # paint all elements
            lrzid, drawkey= i
            drawing= self.__drawman.GetDrawing(drawkey)
            elementsdict= drawing.GetElements()
            element= elementsdict[lrzid]

            if self.__showlinestyle== True:
                # show penstyles
                pen= wx.Pen(self.__options.GetSelCol(), self.__attribute.GetWxWidth(element.GetWidth())* self.__userscale* self.__linewidthfactor, self.__penstyldict[self.__attribute.GetWxStyle(element.GetStyle())[0]])

                if self.__attribute.GetWxStyle(element.GetStyle())[0]== 'wx.USER_DASH':
                    # user penstyle
                    pen.SetDashes(self.__attribute.GetWxStyle(element.GetStyle())[1])

                dc.SetPen(pen)
            else:
                # don't show penstyles
                dc.SetPen(wx.Pen(self.__options.GetSelCol(), self.__attribute.GetWxWidth(element.GetWidth())* self.__userscale* self.__linewidthfactor, wx.SOLID))

            if element.GetId()[0]== 'L':
                # line
                dc.DrawLine(element.GetX1()* self.__userscale, element.GetY1()* self.__userscale, element.GetX2()* self.__userscale, element.GetY2()* self.__userscale)


    def GetBordersDict(self):
        return(self.__bordersdict)


    def GetCursorsDict(self):
        return(self.__cursorsdict)


    def GetPenStyleDict(self):
        return(self.__penstyldict)


    def GetLog(self):
        return(self.__log.GetLog())


class TestFrame(wx.Frame):
    # object for testing
    def __init__(self):
        self.__options= LorzeOptions()
        wx.Frame.__init__(self, None, title= u'Testcode GraphicPanel', size= self.__options.GetFrameSize())

        self.__statusbar= self.CreateStatusBar()
        self.__statusmouse= ''
        self.__statusinfo= ''
        self.RefreshStatus()

        self.__valchck= LorzeValueCheck()
        self.__geohelp= LorzeGeoHelp(self.__options)
        self.__log= LorzeLog()
        self.__attribute= LorzeAttribute(self.__options, self.__log, self.__valchck)
        self.__drawman= LorzeDrawManager(self.__attribute, self.__log, self.__geohelp)
        self.__selection= LorzeSelection(self.__drawman, self.__geohelp)

        self.__gdi= GraphicPanel(self, self.__options, self.__geohelp, self.__log, self.__valchck, self.__attribute, self.__drawman, self.__selection)

        self.__drawman.AddDrawing('test')
        color= [('Red', '255, 0, 0', 'base color'), ('Blue', '0, 0, 255', 'base color'), ('Green', '0, 128, 0', 'base color'), ('Yellow', '255, 255, 0', 'base color'), ('Gray', '128, 128, 128', 'base color'), ('White', '0, 0, 0', 'base color')]
        layer= [('Layer 1', 'default'), ('Layer 2', 'default'), ('Layer 3', 'default'), ('Layer 4', 'default'), ('Layer 5', 'default'), ('Layer 6', 'default')]
        style= [('Solid', 'SOLID', 'standard'), ('Dot', 'DOT', 'standard'), ('Long dash', 'LONG_DASH', 'standard'), ('Short dash', 'SHORT_DASH', 'standard'), ('Dot dash', 'DOT_DASH', 'standard'), ('Axis 13mm', '4, 2, 1, 2, 4', 'standard')]
        width= [('0.13mm', '0.13', 'Rotring'), ('0.18mm', '0.18', 'Rotring'), ('0.25mm', '0.25', 'Rotring'), ('0.35mm', '0.35', 'Rotring'), ('0.50mm', '0.50', 'Rotring'), ('0.70mm', '0.70', 'Rotring')]
        for i in color:
            self.__attribute.AddColorEntry(i[0], i[1], i[2])
        for i in layer:
            self.__attribute.AddLayerEntry(i[0], i[1])
        for i in style:
            self.__attribute.AddStyleEntry(i[0], i[1], i[2])
        for i in width:
            self.__attribute.AddWidthEntry(i[0], i[1], i[2])
        line1= ('L', ('Red', 'Layer 1', 'Solid', '0.13mm', (0, 0, 10, 10)))
        line2= ('L', ('Blue', 'Layer 2', 'Dot', '0.18mm', (0, 0, -10, 10)))
        line3= ('L', ('Green', 'Layer 3', 'Long dash', '0.25mm', (0, 0, -10, -10)))
        line4= ('L', ('Yellow', 'Layer 4', 'Short dash', '0.35mm', (0, 0, 10, -10)))
        line5= ('L', ('Gray', 'Layer 5', 'Dot dash', '0.50mm', (-15, 0, 15, 0)))
        line6= ('L', ('White', 'Layer 6', 'Axis 13mm', '0.70mm', (0, -15, 0, 15)))
        self.__drawman.SetDefaultDraw('test')
        self.__drawman.GetDrawing(self.__drawman.GetDefaultDraw()).Add([line1, line2, line3, line4, line5, line6])
        print('LorzeLog')
        for i in self.__log.GetLog():
            print(i)


        self.__vbox= wx.BoxSizer(wx.VERTICAL)
        self.__vbox.Add(self.__gdi, 1, wx.EXPAND)
        self.SetSizer(self.__vbox)

        self.Center()
        self.Show()

        self.CheckMenu('elementsel')


    def SetStatusMouse(self, x, y):
        self.__statusmouse= 'X: '+ str(x)+ ' / Y: '+ str(y)
        self.RefreshStatus()


    def SetStatusInfo(self, info):
        self.__statusinfo= info
        self.RefreshStatus()


    def RefreshStatus(self):
        text= self.__statusmouse+ ' || '+ self.__statusinfo
        self.__statusbar.SetStatusText(text)


    def GetGdiSize(self):
        child= self.__vbox.GetChildren()
        return(child[0].GetSize())


    def CheckMenu(self, cmd):
        self.__gdi.SetAction(cmd)


if __name__== '__main__':
    from lrzoptions import LorzeOptions
    from lrzselect import LorzeSelection
    from lrzgeohelp import LorzeGeoHelp
    from lrzdrawing import LorzeDrawing
    from lrzlog import LorzeLog
    from lrzvalchck import LorzeValueCheck
    from lrzattribute import LorzeAttribute
    from lrzdrawmanager import LorzeDrawManager

    app= wx.App()
    frame= TestFrame()
    app.MainLoop()

