#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of "LORZE erasandcad"

# "LORZE erasandcad" is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# "LORZE erasandcad" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx, locale
from lrzoptions import LorzeOptions
from lrzgdipnl import GraphicPanel
from lrzrestart import LorzeRestart
from lrzdlgoptions import LorzeDlgOptions
from lrzloadsave import LorzeLoadSave
from lrzdlgloadsave import LorzeDlgLoadSave
from lrzdrawing import LorzeDrawing
from lrzvalchck import LorzeValueCheck
from lrzselect import LorzeSelection
from lrzgeohelp import LorzeGeoHelp
from lrzattribute import LorzeAttribute
from lrzlog import LorzeLog
from lrzdrawmanager import LorzeDrawManager
from lrzdlghelper import LorzeDlgHelper
from lrzdlgattribut import LorzeDlgAttribut
from lrzdlgabout import LorzeAbout
from lrzdlghelp import LorzeHelp

# name for translations texts
_= wx.GetTranslation # name for translations texts


class LorzeGUI(wx.Frame):
    # GUI for LORZE erasandcad

    def __init__(self, restart):
        # restart object
        self.__restart= restart

        if self.__restart.GetRestart()== False:
            # LorzeOptions() / add current path to instpaths
            self.__options= LorzeOptions()
            self.__options.AddInstPath()

        else:
            # dictionairy with datas
            data= self.__restart.GetData()

            # LorzeOptions() of datas
            self.__options= data['options']

            # next time no restart
            self.__restart.SetRestart(False)

        # subclass
        wx.Frame.__init__(self, None, title= self.__options.GetFrameTitle(), size= self.__options.GetFrameSize())

        # icon
        self.SetIcon(wx.Icon('../Icons/LORZE_64.png', wx.BITMAP_TYPE_PNG))

        # dictionairy menu titles
        mnutit= {'file':_(u'&File'), 'edit':_(u'&Edit'), 'tool':_(u'&Tools'), 'modify':_(u'&Modify'), 'options':_(u'&Options'), 'help':_(u'&Help')}
         #'view':_(u'&View'), 'geometry':_(u'&Geometry')

        # dictionairy menu layout
        mnulay= {'file':('new', 'sep', 'open', 'save', 'saveas', 'sep', 'close', 'sep', 'exit'),  'edit':('delete', 'sep'), 'tool':('elementsel', 'pointsel', 'sep', 'line', 'circle', 'text', 'point'), 'modify':('attributes', 'sizes', 'sep', 'move', 'duplicate', 'mirror', 'rotate', 'sep', 'scale'), 'options':('program', 'sep'), 'help':('about', 'sep', 'helpwin')}
        #'view':(), 'geometry':()

        # dictionairy toolbar layout
        toollayHa= ('new', 'open', 'save', 'saveas')
        self.__toollayVa= ('elementsel', 'pointsel', 'line', 'circle', 'text', 'point')

        # dictionairy menu commands text
        mnutxt= {'new':_(u'&New\tCtrl+N'), 'open':_(u'&Open\tCtrl+O'), 'save':_(u'&Save\tCtrl+S'), 'saveas':_(u'Save &as..\tCtrl+T'), 'close':_(u'&Close\tCtrl+W'), 'exit':_(u'&Exit\tCtrl+Q'), 'delete':_(u'&Delete\tDelete'), 'elementsel':_(u'&Select elements\t1'), 'pointsel':_(u'S&elect points\t2'), 'line':_(u'&Line\tL'), 'circle':_(u'&Circle\tC'), 'text':_(u'&Text\tT'), 'point':_(u'&Point\tP'), 'attributes':_(u'&Attributes\tA'), 'sizes':_(u'&Sizes\tSpace'), 'move':_(u'&Move\tM'), 'duplicate':_(u'D&uplicate\tD'), 'mirror':_(u'M&irror\tN'), 'rotate':_(u'&Rotate\tR'), 'scale':_(u'S&cale\tS'), 'program':_(u'&Program\tCtrl+2'), 'about':_(u'&About'), 'helpwin':_(u'&Help window')}

        # dictionairy menu & toolbar commands help
        mnuhlp= {'new':_(u'New drawing'), 'open':_(u'Open drawing'), 'save':_(u'Save drawing'), 'saveas':_(u'Save drawing with a new name'), 'close':_(u'Close drawing'), 'exit':_(u'Terminate LORZE'), 'delete':_(u'Delete elements'), 'elementsel':_(u'Select elements'), 'pointsel':_(u'Select points'), 'line':_(u'Draw a line'), 'circle':_(u'Draw a circle'), 'text':_(u'Insert a text'), 'point':_(u'Draw a point'), 'attributes':_(u'Edit element attributes'), 'sizes':_(u'Edit element sizes'), 'move':_(u'Move elements'), 'duplicate':_(u'Duplicate elements'), 'mirror':_(u'Mirror elements'), 'rotate':_(u'rotate elements'), 'scale':_(u'scale elements'), 'program':_(u'Progam options'), 'about':_(u'Informations about LORZE erasandcad'), 'helpwin':_(u'Help to LORZE erasandcad')}

        # dictionairy icon names
        mnupic= {'new':'document-new.png', 'open':'document-open.png', 'save':'document-save.png', 'saveas':'document-save-as.png', 'close':'',
        'exit':'system-log-out.png', 'delete':'edit-delete.png', 'elementsel':'lorze_elemsel.png', 'pointsel':'lorze_pntsel.png', 'line':'lorze_line.png', 'circle':'lorze_circle.png', 'text':'lorze_text.png', 'point':'lorze_point.png', 'attributes':'', 'sizes':'', 'move':'', 'duplicate':'', 'mirror':'', 'rotate':'', 'scale':'', 'program':'preferences-system.png', 'about':'dialog-information.png', 'helpwin':'help-browser.png'}

        # width of statusbar field for coordinate
        stcoordw= self.__options.GetStCoordW()

        # path to menu icons and toolbar icons
        mnupicpath= '../Icons/22x22/'
        toolpicpath= '../Icons/32x32/'

        # geometrie helper object
        self.__geohelp= LorzeGeoHelp(self.__options)

        # log object
        self.__log= LorzeLog()

        # LorzeValueCheck
        self.__valchck= LorzeValueCheck()

        # attribute
        self.__attribute= LorzeAttribute(self.__options, self.__log, self.__valchck)

        # drawings manager
        self.__drawman= LorzeDrawManager(self.__attribute, self.__log, self.__geohelp)

        # selection object
        self.__selection= LorzeSelection(self.__drawman, self.__geohelp)

        # Dialog Helper
        self.__dlg= LorzeDlgHelper(self)

        # graphic-panel
        self.__gdi= GraphicPanel(self, self.__options, self.__geohelp, self.__log, self.__valchck, self.__attribute, self.__drawman, self.__selection)

        # add new drawing and set default
        self.__drawman.AddDrawing('Drawing')
        self.__drawman.SetDefaultDraw('Drawing')

        # dictionairy menus, wx.Menu
        lrzmenu= {}

        # dictionairy menu-item, wx.MenuItem
        lrzmenuitem= {}

        # dictionairy toolbar, wx.ToolBar
        toolitem= {}

        # dictionairy toolbar, Checktools
        self.__checktool= {}

        # menubar
        menubar= wx.MenuBar()

        # creating menu
        for i in ['file', 'edit', 'tool', 'modify', 'options', 'help']:
            # item in menubar
            lrzmenu[i]= wx.Menu()

            for j in mnulay[i]:
                if j!= 'sep':
                    # menuitem
                    lrzmenuitem[j]= wx.MenuItem(lrzmenu[i], wx.ID_ANY, mnutxt[j], mnuhlp[j])

                    if mnupic[j]!= '':
                        # menu-icon
                        lrzmenuitem[j].SetBitmap(wx.Bitmap(mnupicpath+ mnupic[j]))

                    # appending at the menu
                    lrzmenu[i].AppendItem(lrzmenuitem[j])

                else:
                    # seperation
                    lrzmenu[i].AppendSeparator()

            menubar.Append(lrzmenu[i], mnutit[i])

        # dictionairy bindings
        bindcmd= {'new':self.OnNew, 'open':self.OnOpen, 'save':self.OnSave, 'saveas':self.OnSaveAs, 'close':self.OnClose, 'exit':self.OnExit, 'delete':self.OnDelete, 'elementsel':self.OnElementSel, 'pointsel':self.OnPointSel, 'line':self.OnLine, 'circle':self.OnCircle, 'text':self.OnText, 'point':self.OnPoint, 'attributes':self.OnAttributes, 'sizes':self.OnSizes, 'move':self.OnMove, 'duplicate':self.OnDuplicate, 'mirror':self.OnMirror, 'rotate':self.OnRotate, 'scale':self.OnScale, 'program':self.OnProgram, 'about':self.OnAbout, 'helpwin':self.OnHelp}

        # menu bindings
        for i in ['file', 'edit', 'tool', 'modify', 'options', 'help']:
            for j in mnulay[i]:
                if j!= 'sep':
                    self.Bind(wx.EVT_MENU, bindcmd[j], lrzmenuitem[j])

        # horizontal toolbar
        toolbarHa= wx.ToolBar(self, style= wx.TB_HORIZONTAL)

        for i in toollayHa:
            # toolbar-item
            toolitem[i]= toolbarHa.AddLabelTool(wx.ID_ANY, '', wx.Bitmap(toolpicpath+ mnupic[i]), longHelp= mnuhlp[i])

        toolbarHa.Realize()

        # vertical toolbar
        self.__toolbarVa= wx.ToolBar(self, style= wx.TB_VERTICAL)

        for i in self.__toollayVa:
            # toolbar-item
            self.__checktool[i]= self.__toolbarVa.AddCheckLabelTool(wx.ID_ANY, '', wx.Bitmap(toolpicpath+ mnupic[i]), longHelp= mnuhlp[i])

        self.__toolbarVa.Realize()

        # toolbar bindings
        for i in toollayHa:
            self.Bind(wx.EVT_TOOL, bindcmd[i], toolitem[i])
        for i in self.__toollayVa:
            self.Bind(wx.EVT_TOOL, bindcmd[i], self.__checktool[i])

        # exit bindings
        self.Bind(wx.EVT_CLOSE, self.OnSystemClose)

        # layouting with sizers
        vbox= wx.BoxSizer(wx.VERTICAL)
        self.__hbox= wx.BoxSizer(wx.HORIZONTAL)
        vbox.Add(toolbarHa, 0, wx.EXPAND)
        self.__hbox.Add(self.__toolbarVa, 0, wx.EXPAND)
        self.__hbox.Add(self.__gdi, 1, wx.EXPAND)
        vbox.Add(self.__hbox,1, wx.EXPAND)
        self.SetSizer(vbox)

        # create MenuBar
        self.SetMenuBar(menubar)

        # create Statusbar & attribute for statusbar info: string
        self.__statusbar= self.CreateStatusBar()
        self.__statusbar.SetFieldsCount(3)
        self.__statusbar.SetStatusWidths([-1, stcoordw, stcoordw])
        self.__statusinfo= ''

        # centre window
        self.Center()

        # show window
        self.Show()

        # set elementselection for start
        self.CheckMenu('elementsel')


    def OnNew(self, event):
        # new
        self.NotAvailable()


    def OnOpen(self, event):
        # open
        # load/save dialog
        lsdlg= LorzeDlgLoadSave(self, self.__options)

        # THE PATH IS ALREADY UNDER CONSTRUCTION
        lsdlg.SetPath= ''

        # load with flag
        dlgflag= lsdlg.LoadDrawing()
        if dlgflag== True:
            # load success
            self.NotAvailable()


    def OnSave(self, event):
        # save
        self.NotAvailable()


    def OnSaveAs(self, event):
        # save as
        # load/save dialog
        lsdlg= LorzeDlgLoadSave(self, self.__options)

        # DATAS ARE ALREADY UNDER CONSTRUCTION
        lsdlg.SetData(None)

        # THE PATH IS ALREADY UNDER CONSTRUCTION
        lsdlg.SetPath= ''

        # save with flag
        dlgflag= lsdlg.SaveAsDrawing()
        if dlgflag== True:
            # save success
            self.NotAvailable()


    def OnClose(self, event):
        # close
        self.NotAvailable()


    def OnExit(self, event):
        # terminate
        self.Close()


    def OnDelete(self, event):
        # delete
        self.NotAvailable()


    def OnElementSel(self, event):
        # element selection
        self.CheckMenu('elementsel')


    def OnPointSel(self, event):
        # point selection
        self.CheckMenu('pointsel')


    def OnLine(self, event):
        # draw a line
        self.CheckMenu('line')


    def OnCircle(self, event):
        # draw a circle
        self.CheckMenu('circle')


    def OnText(self, event):
        # insert a text
        self.CheckMenu('text')


    def OnPoint(self, event):
        # draw a point
        self.CheckMenu('point')


    def OnAttributes(self, event):
        # alement-attributes
        dlg= LorzeDlgAttribut(self, self.__options, self.__valchck, self.__dlg, self.__gdi)
        dlg.ShowModal()
        dlg.Destroy()


    def OnSizes(self, event):
        # alement-attributes
        self.NotAvailable()


    def OnMove(self, event):
        # move
        self.NotAvailable()


    def OnDuplicate(self, event):
        # duplicate
        self.NotAvailable()


    def OnMirror(self, event):
        # mirror
        self.NotAvailable()


    def OnRotate(self, event):
        # rotate
        self.NotAvailable()


    def OnScale(self, event):
        # scaling
        self.NotAvailable()


    def OnProgram(self, event):
        # program-options
        dlg= LorzeDlgOptions(self, self.__options, self.__valchck, self.__dlg, self.__gdi)
        dlg.ShowModal()

        # get restart, Ture or False
        restart= dlg.GetRestart()
        dlg.Destroy()
        if restart== True:
            # restart
            self.RestartGui()


    def OnAbout(self, event):
        # about dialogue
        LorzeAbout(self.__options)


    def OnHelp(self, event):
        dlg= LorzeHelp(self, self.__options)
        dlg.ShowModal()
        dlg.Destroy()


    def SetStatusMouse(self, x, y):
        # set mouse-coordinates to the statusbar, field 1 & 2,  set information to the field 0
        self.__statusbar.SetStatusText('X: '+ str(x), 1)
        self.__statusbar.SetStatusText('Y: '+ str(y), 2)
        self.__statusbar.SetStatusText(self.__statusinfo, 0)


    def SetStatusInfo(self, text):
        # set information to attribute
        self.__statusinfo= text


    def OnSystemClose(self, event):
        # last commando before close the window, set actual window size in options
        w, h= self.GetSize()
        self.__options.SetFrameSize(w, h)

        if self.__restart.GetRestart()!= True:
            # only when no restart
            self.__options.SaveOptions()

        event.Skip()


    def RestartGui(self):
        # restart gui, save datas in object restart
        # dictionary to store datas
        data={}

        # store options
        data['options']= self.__options

        # set restart
        self.__restart.SetData(data)
        self.__restart.SetRestart(True)
        self.Close()


    def GetGdiSize(self):
        # returns the size of the gdi panel, self.__gdi
        child= self.__hbox.GetChildren()
        return(child[1].GetSize())


    def NotAvailable(self):
        # show info box: This command is not available at the moment
        dlg= wx.MessageBox(_(u'This command is not available at the moment'), _(u'Information'))


    def CheckMenu(self, cmd):
        # control all checkitems in menu and toolbar, set action in gdi panel
        # set all checktoollabel unpressed
        for i in self.__toollayVa:
            wxid= self.__checktool[i].GetId()
            self.__toolbarVa.ToggleTool(wxid, False)

        #set pressed chectoollabel
        wxid= self.__checktool[cmd].GetId()
        self.__toolbarVa.ToggleTool(wxid, True)

        self.__gdi.SetAction(cmd)


if __name__== '__main__':
    # LorzeRestart() for changing options
    restart= LorzeRestart()

    # wxpython app
    app= wx.App()

    # internationalization of LorzeGUI
    wxloc= wx.Locale()
    wxloc.AddCatalogLookupPathPrefix('./in18')
    # get system language ('xx_XX', 'CHARSET')
    wxlang= locale.getdefaultlocale()
    wxlang= wxlang[0][:2]
    # get system language ('xx_XX', 'CHARSET') and select translation
    if locale.getdefaultlocale()[0][:2]== 'de':
        wxloc.AddCatalog('lorze_de')

    # wx.frame, main loop
    frame= LorzeGUI(restart)
    app.MainLoop()

    # check for restart
    while restart.GetRestart()== True:
        # restart gui
        frame= LorzeGUI(restart)
        app.MainLoop()

