#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of "LORZE erasandcad"

# "LORZE erasandcad" is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# "LORZE erasandcad" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import wx, os


class LorzeOptions():
    # Lorze options

    def __init__(self): # initialize LorzeOptions
        self.__lrzcfg = wx.Config('lorze_0-2') # wx.Config object to load options from system settings

        # frametitle: window title, string
        # instpaths: installation paths, string, 'path/inst1,path/inst2'
        # framesize: main window size, (w, h)
        # gdiborder: grafik panel border, string, key for self.__bordersdict
        # dezplcs: number of decimal places, integer
        # cursorwin: standard-cursor, string, key for self.__cursorsdict
        # cursordraw: cursor graphic-panel in draw modus, string, key for self.__cursorsdict
        # cursorselect: cursor graphic-panel in select modus, string, key for self.__cursorsdict
        # dlgborder: distance in dialogs between widgets, integer
        # dlgoptsize: options dialog size, (w, h)
        # sbrange: GraphicPANEL() scrollbar range, integer
        # sbsize: GraphicPANEL() size scrollbar slider, integer
        # wheelenlarge: mouse wheel factor enlargement, float
        # wheelreduce: mouse wheel factor reduction, float
        # sensitive: pixelsize smart cursor, integer
        # maxpenstylescale: maximum scale factor (pen), integer
        # maxuserscale: maximum userscale, integer
        # scsize: smart cursor mark size in pixel, integer
        # scpensize: smart cursor mark pen size in pixel, integer
        # sccolpoint: smart cursor colour point, string, html format
        # sccolsegm: smart cursor colour segment, string, html format
        # sccolelem: smart cursor colour element, string, html format
        # scmarkpensize: smart cursor mark pen size factor, float
        # prevcol: preview color, string, html format
        # prevpensize: preview pen size, integer
        # prevpenstyle: preview pen style, string
        # stcoordw: statusbar field with for coordinates, integer
        # selcol: color selected elements, string, html format
        # dlgattrsize: attribute dialogue size (w, h)
        # optlistwidths: column width option values
        # attrlistwidths: column width attribute values
        # defcolorname: default drawing colour name, string
        # defcolorvalue: default drawing colour value, string, 'r, g, b'
        # defcolorlabel: default drawing color label, string
        # deflayername: default drawing layer name, string
        # deflayerlabel: default drawing layer label, string
        # defstylename: default drawing line style name, string
        # defstylevalue: default drawing line style value, string, list: 'USER_DASH', '10, 20, 10'
        # defstylelabel: default drawing line style label, string
        # defwidthname: default drawing line width name, string
        # defwidthvalue: default drawing line width value, Float
        # defwidthlabel: default drawing line width label, string
        # dlghelpsize: help dialogue size (w, h)

        # default values: dictionairy
        self.__defval= {'framesizew':800, 'framesizeh':600, 'gdiborder':'wx.SUNKEN_BORDER', 'dezplcs':6, 'cursorwin':'wx.CURSOR_DEFAULT', 'cursordraw':'wx.CURSOR_CROSS', 'cursorselect':'wx.CURSOR_DEFAULT', 'dlgborder':5, 'dlgoptsizew':450, 'dlgoptsizeh':600, 'sbrange':200, 'sbsize':10, 'wheelenlarge':1.2, 'wheelreduce':0.8, 'sensitive':10, 'maxpenstylescale':50, 'maxuserscale':1000, 'scsize':20, 'scpensize':2, 'sccolpoint':'#000000', 'sccolsegm':'#0AE2F2', 'sccolelem':'#0000FF', 'scmarkpensize':1.5, 'prevcol':'#000000', 'prevpensize':1, 'prevpenstyle':'wx.SOLID', 'stcoordw': 150, 'selcol': '#00FF00', 'dlgattrsizew':600, 'dlgattrsizeh':500, 'optlistwidth0':290, 'optlistwidth1':120, 'attrlistwidth0':220, 'attrlistwidth1':120, 'attrlistwidth2':120, 'attrlistwidth3':120, 'attrlistwidth4':220, 'attrlistwidth5':120, 'attrlistwidth6':120, 'attrlistwidth7':120, 'attrlistwidth8':220, 'attrlistwidth9':120, 'attrlistwidth10':120, 'attrlistwidth11':120, 'attrlistwidth12':220, 'attrlistwidth13':120, 'attrlistwidth14':120, 'attrlistwidth15':120, 'defcolorname':'Black', 'defcolorvalue':'0, 0, 0', 'defcolorlabel':'Default colour', 'deflayername':'System', 'deflayerlabel': 'Default layer', 'defstylename':'Solid', 'defstylevalue': 'SOLID', 'defstylelabel':'Default line style', 'defwidthname':'1mm', 'defwidthvalue':'1.0', 'defwidthlabel':'Default line width', 'dlghelpsizew':600, 'dlghelpsizeh':600}

        # frametitle are not editable
        self.__frametitle= u'LORZE erasandcad 0.2 / alpha 2'

        # instpaths
        cwd= os.getcwd()
        self.__instpaths= self.__lrzcfg.Read('instpaths', cwd)

        # set options from 'lorzecfg'
        w= self.__lrzcfg.ReadInt('framesizew', self.__defval['framesizew'])
        h= self.__lrzcfg.ReadInt('framesizeh', self.__defval['framesizeh'])
        self.__framesize= (w, h)

        self.__gdiborder= self.__lrzcfg.Read('gdiborder', self.__defval['gdiborder'])
        self.__dezplcs= self.__lrzcfg.ReadInt('dezplcs', self.__defval['dezplcs'])
        self.__cursorwin= self.__lrzcfg.Read('cursorwin', self.__defval['cursorwin'])
        self.__cursordraw= self.__lrzcfg.Read('cursordraw', self.__defval['cursordraw'])
        self.__cursorselect= self.__lrzcfg.Read('cursorselect', self.__defval['cursorselect'])
        self.__dlgborder= self.__lrzcfg.ReadInt('dlgborder', self.__defval['dlgborder'])

        w= self.__lrzcfg.ReadInt('dlgoptsizew', self.__defval['dlgoptsizew'])
        h= self.__lrzcfg.ReadInt('dlgoptsizeh', self.__defval['dlgoptsizeh'])
        self.__dlgoptsize= (w, h)

        self.__sbrange= self.__lrzcfg.ReadInt('sbrange', self.__defval['sbrange'])
        self.__sbsize= self.__lrzcfg.ReadInt('sbsize', self.__defval['sbsize'])
        self.__wheelenlarge= self.__lrzcfg.ReadFloat('wheelenlarge', self.__defval['wheelenlarge'])
        self.__wheelreduce= self.__lrzcfg.ReadFloat('wheelreduce', self.__defval['wheelreduce'])
        self.__sensitive= self.__lrzcfg.ReadInt('sensitive', self.__defval['sensitive'])
        self.__maxpenstylescale= self.__lrzcfg.ReadInt('maxpenstylescale', self.__defval['maxpenstylescale'])
        self.__maxuserscale= self.__lrzcfg.ReadInt('maxuserscale', self.__defval['maxuserscale'])
        self.__scsize= self.__lrzcfg.ReadInt('scsize', self.__defval['scsize'])
        self.__scpensize= self.__lrzcfg.ReadInt('scpensize', self.__defval['scpensize'])
        self.__sccolpoint= self.__lrzcfg.Read('sccolpoint', self.__defval['sccolpoint'])
        self.__sccolsegm= self.__lrzcfg.Read('sccolsegm', self.__defval['sccolsegm'])
        self.__sccolelem= self.__lrzcfg.Read('sccolelem', self.__defval['sccolelem'])
        self.__scmarkpensize= self.__lrzcfg.ReadFloat('scmarkpensize', self.__defval['scmarkpensize'])
        self.__prevcol= self.__lrzcfg.Read('prevcol', self.__defval['prevcol'])
        self.__prevpensize= self.__lrzcfg.ReadInt('prevpensize', self.__defval['prevpensize'])
        self.__prevpenstyle= self.__lrzcfg.Read('prevpenstyle', self.__defval['prevpenstyle'])
        self.__stcoordw= self.__lrzcfg.ReadInt('stcoordw', self.__defval['stcoordw'])
        self.__selcol= self.__lrzcfg.Read('selcol', self.__defval['selcol'])

        w= self.__lrzcfg.ReadInt('dlgattrsizew', self.__defval['dlgattrsizew'])
        h= self.__lrzcfg.ReadInt('dlgattrsizeh', self.__defval['dlgattrsizeh'])
        self.__dlgattrsize= (w, h)

        w1= self.__lrzcfg.ReadInt('optlistwidth0', self.__defval['optlistwidth0'])
        w2= self.__lrzcfg.ReadInt('optlistwidth1', self.__defval['optlistwidth1'])
        self.__optlistwidths= (w1, w2)

        w0= self.__lrzcfg.ReadInt('attrlistwidth0', self.__defval['attrlistwidth0'])
        w1= self.__lrzcfg.ReadInt('attrlistwidth1', self.__defval['attrlistwidth1'])
        w2= self.__lrzcfg.ReadInt('attrlistwidth2', self.__defval['attrlistwidth2'])
        w3= self.__lrzcfg.ReadInt('attrlistwidth3', self.__defval['attrlistwidth3'])
        w4= self.__lrzcfg.ReadInt('attrlistwidth4', self.__defval['attrlistwidth4'])
        w5= self.__lrzcfg.ReadInt('attrlistwidth5', self.__defval['attrlistwidth5'])
        w6= self.__lrzcfg.ReadInt('attrlistwidth6', self.__defval['attrlistwidth6'])
        w7= self.__lrzcfg.ReadInt('attrlistwidth7', self.__defval['attrlistwidth7'])
        w8= self.__lrzcfg.ReadInt('attrlistwidth8', self.__defval['attrlistwidth8'])
        w9= self.__lrzcfg.ReadInt('attrlistwidth9', self.__defval['attrlistwidth9'])
        w10= self.__lrzcfg.ReadInt('attrlistwidth10', self.__defval['attrlistwidth10'])
        w11= self.__lrzcfg.ReadInt('attrlistwidth11', self.__defval['attrlistwidth11'])
        w12= self.__lrzcfg.ReadInt('attrlistwidth12', self.__defval['attrlistwidth12'])
        w13= self.__lrzcfg.ReadInt('attrlistwidth13', self.__defval['attrlistwidth13'])
        w14= self.__lrzcfg.ReadInt('attrlistwidth14', self.__defval['attrlistwidth14'])
        w15= self.__lrzcfg.ReadInt('attrlistwidth15', self.__defval['attrlistwidth15'])
        self.__attrlistwidths= (w0, w1, w2, w3, w4, w5, w6, w7, w8, w9, w10, w11, w12, w13, w14, w15)

        name= self.__lrzcfg.Read('defcolorname', self.__defval['defcolorname'])
        value= self.__lrzcfg.Read('defcolorvalue', self.__defval['defcolorvalue'])
        label= self.__lrzcfg.Read('defcolorlabel', self.__defval['defcolorlabel'])
        self.__drawdefcolor= (name, value, label)

        name= self.__lrzcfg.Read('deflayername', self.__defval['deflayername'])
        label= self.__lrzcfg.Read('deflayerlabel', self.__defval['deflayerlabel'])
        self.__drawdeflayer= (name, label)

        name= self.__lrzcfg.Read('defstylename', self.__defval['defstylename'])
        value= self.__lrzcfg.Read('defstylevalue', self.__defval['defstylevalue'])
        label= self.__lrzcfg.Read('defstylelabel', self.__defval['defstylelabel'])
        self.__drawdefstyle= (name, value, label)

        name= self.__lrzcfg.Read('defwidthname', self.__defval['defwidthname'])
        value= self.__lrzcfg.Read('defwidthvalue', self.__defval['defwidthvalue'])
        label= self.__lrzcfg.Read('defwidthlabel', self.__defval['defwidthlabel'])
        self.__drawdefwidth= (name, value, label)

        w= self.__lrzcfg.ReadInt('dlghelpsizew', self.__defval['dlghelpsizew'])
        h= self.__lrzcfg.ReadInt('dlghelpsizeh', self.__defval['dlghelpsizeh'])
        self.__dlghelpsize= (w, h)


    def SaveOptions(self):
        self.__lrzcfg.Write('instpaths', self.__instpaths)
        self.__lrzcfg.WriteInt('framesizew', self.__framesize[0])
        self.__lrzcfg.WriteInt('framesizeh', self.__framesize[1])
        self.__lrzcfg.Write('gdiborder', self.__gdiborder)
        self.__lrzcfg.WriteInt('dezplcs', self.__dezplcs)
        self.__lrzcfg.Write('cursorwin', self.__cursorwin)
        self.__lrzcfg.Write('cursordraw', self.__cursordraw)
        self.__lrzcfg.Write('cursorselect', self.__cursorselect)
        self.__lrzcfg.WriteInt('dlgborder', self.__dlgborder)
        self.__lrzcfg.WriteInt('dlgoptsizew', self.__dlgoptsize[0])
        self.__lrzcfg.WriteInt('dlgoptsizeh', self.__dlgoptsize[1])
        self.__lrzcfg.WriteInt('sbrange', self.__sbrange)
        self.__lrzcfg.WriteInt('sbsize', self.__sbsize)
        self.__lrzcfg.WriteFloat('wheelenlarge', self.__wheelenlarge)
        self.__lrzcfg.WriteFloat('wheelreduce', self.__wheelreduce)
        self.__lrzcfg.WriteInt('sensitive', self.__sensitive)
        self.__lrzcfg.WriteInt('maxpenstylescale', self.__maxpenstylescale)
        self.__lrzcfg.WriteInt('maxuserscale', self.__maxuserscale)
        self.__lrzcfg.WriteInt('scsize', self.__scsize)
        self.__lrzcfg.WriteInt('scpensize', self.__scpensize)
        self.__lrzcfg.Write('sccolpoint', self.__sccolpoint)
        self.__lrzcfg.Write('sccolsegm', self.__sccolsegm)
        self.__lrzcfg.Write('sccolelem', self.__sccolelem)
        self.__lrzcfg.WriteFloat('scmarkpensize', self.__scmarkpensize)
        self.__lrzcfg.Write('prevcol', self.__prevcol)
        self.__lrzcfg.WriteInt('prevpensize', self.__prevpensize)
        self.__lrzcfg.Write('prevpenstyle', self.__prevpenstyle)
        self.__lrzcfg.WriteInt('stcoordw', self.__stcoordw)
        self.__lrzcfg.Write('selcol', self.__selcol)

        self.__lrzcfg.WriteInt('dlgattrsizew', self.__dlgattrsize[0])
        self.__lrzcfg.WriteInt('dlgattrsizeh', self.__dlgattrsize[1])

        self.__lrzcfg.WriteInt('optlistwidth0', self.__optlistwidths[0])
        self.__lrzcfg.WriteInt('optlistwidth1', self.__optlistwidths[1])

        self.__lrzcfg.WriteInt('attrlistwidth0', self.__attrlistwidths[0])
        self.__lrzcfg.WriteInt('attrlistwidth1', self.__attrlistwidths[1])
        self.__lrzcfg.WriteInt('attrlistwidth2', self.__attrlistwidths[2])
        self.__lrzcfg.WriteInt('attrlistwidth3', self.__attrlistwidths[3])
        self.__lrzcfg.WriteInt('attrlistwidth4', self.__attrlistwidths[4])
        self.__lrzcfg.WriteInt('attrlistwidth5', self.__attrlistwidths[5])
        self.__lrzcfg.WriteInt('attrlistwidth6', self.__attrlistwidths[6])
        self.__lrzcfg.WriteInt('attrlistwidth7', self.__attrlistwidths[7])
        self.__lrzcfg.WriteInt('attrlistwidth8', self.__attrlistwidths[8])
        self.__lrzcfg.WriteInt('attrlistwidth9', self.__attrlistwidths[9])
        self.__lrzcfg.WriteInt('attrlistwidth10', self.__attrlistwidths[10])
        self.__lrzcfg.WriteInt('attrlistwidth11', self.__attrlistwidths[11])
        self.__lrzcfg.WriteInt('attrlistwidth12', self.__attrlistwidths[12])
        self.__lrzcfg.WriteInt('attrlistwidth13', self.__attrlistwidths[13])
        self.__lrzcfg.WriteInt('attrlistwidth14', self.__attrlistwidths[14])
        self.__lrzcfg.WriteInt('attrlistwidth15', self.__attrlistwidths[15])

        self.__lrzcfg.Write('defcolorname', self.__drawdefcolor[0])
        self.__lrzcfg.Write('defcolorvalue', self.__drawdefcolor[1])
        self.__lrzcfg.Write('defcolorlabel', self.__drawdefcolor[2])

        self.__lrzcfg.Write('deflayername', self.__drawdeflayer[0])
        self.__lrzcfg.Write('deflayerlabel', self.__drawdeflayer[1])

        self.__lrzcfg.Write('defstylename', self.__drawdefstyle[0])
        self.__lrzcfg.Write('defstylevalue', self.__drawdefstyle[1])
        self.__lrzcfg.Write('defstylelabel', self.__drawdefstyle[2])

        self.__lrzcfg.Write('defwidthname', self.__drawdefwidth[0])
        self.__lrzcfg.Write('defwidthvalue', self.__drawdefwidth[1])
        self.__lrzcfg.Write('defwidthlabel', self.__drawdefwidth[2])

        self.__lrzcfg.WriteInt('dlghelpsizew', self.__dlghelpsize[0])
        self.__lrzcfg.WriteInt('dlghelpsizeh', self.__dlghelpsize[1])


    def Reset(self):
        cwd= os.getcwd()
        self.__instpaths= self.__lrzcfg.Read('instpaths', cwd)

        self.__framesize= ((self.__defval['framesizew'], self.__defval['framesizeh']))
        self.__gdiborder= self.__defval['gdiborder']
        self.__dezplcs= self.__defval['dezplcs']
        self.__cursorwin= self.__defval['cursorwin']
        self.__cursordraw= self.__defval['cursordraw']
        self.__cursorselect= self.__defval['cursorselect']
        self.__dlgborder= self.__defval['dlgborder']
        self.__dlgoptsize= ((self.__defval['dlgoptsizew'], self.__defval['dlgoptsizeh']))
        self.__sbrange= self.__defval['sbrange']
        self.__sbsize= self.__defval['sbsize']
        self.__wheelenlarge= self.__defval['wheelenlarge']
        self.__wheelreduce= self.__defval['wheelreduce']
        self.__sensitive= self.__defval['sensitive']
        self.__maxpenstylescale= self.__defval['maxpenstylescale']
        self.__maxuserscale= self.__defval['maxuserscale']
        self.__scsize= self.__defval['scsize']
        self.__scpensize= self.__defval['scpensize']
        self.__sccolpoint= self.__defval['sccolpoint']
        self.__sccolsegm= self.__defval['sccolsegm']
        self.__sccolelem= self.__defval['sccolelem']
        self.__scmarkpensize= self.__defval['scmarkpensize']
        self.__prevcol= self.__defval['prevcol']
        self.__prevpensize= self.__defval['prevpensize']
        self.__prevpenstyle= self.__defval['prevpenstyle']
        self.__stcoordw= self.__defval['stcoordw']
        self.__selcol= self.__defval['selcol']
        self.__dlgattrsize= (self.__defval['dlgattrsizew'], self.__defval['dlgattrsizeh'])
        self.__optlistwidths= (self.__defval['optlistwidth0'], self.__defval['optlistwidth1'])
        self.__attrlistwidths= (self.__defval['attrlistwidth0'], self.__defval['attrlistwidth1'], self.__defval['attrlistwidth2'], self.__defval['attrlistwidth3'], self.__defval['attrlistwidth4'], self.__defval['attrlistwidth5'], self.__defval['attrlistwidth6'], self.__defval['attrlistwidth7'], self.__defval['attrlistwidth8'], self.__defval['attrlistwidth9'], self.__defval['attrlistwidth10'], self.__defval['attrlistwidth11'], self.__defval['attrlistwidth12'], self.__defval['attrlistwidth13'], self.__defval['attrlistwidth14'], self.__defval['attrlistwidth15'])
        self.__drawdefcolor= (self.__defval['defcolorname'], self.__defval['defcolorvalue'], self.__defval['defcolorlabel'])
        self.__drawdeflayer= (self.__defval['deflayername'], self.__defval['deflayerlabel'])
        self.__drawdefstyle= (self.__defval['defstylename'], self.__defval['defstylevalue'], self.__defval['defstylelabel'])
        self.__drawdefwidth= (self.__defval['defwidthname'], self.__defval['defwidthvalue'], self.__defval['defwidthlabel'])
        self.__dlghelpsize= (self.__defval['dlghelpsizew'], self.__defval['dlghelpsizeh'])


    def SetFrameSize(self, width, height):
        self.__framesize= (width, height)


    def GetFrameSize(self):
        return(self.__framesize)


    def GetFrameTitle(self):
        return(self.__frametitle)


    def SetGDIBorder(self, gdiborder):
        self.__gdiborder= str(gdiborder)


    def GetGDIBorder(self):
        return(self.__gdiborder)


    def SetDezPlcs(self, dezplcs):
        self.__dezplcs= int(dezplcs)


    def GetDezPlcs(self):
        return(self.__dezplcs)


    def SetCursorDraw(self, cursordraw):
        self.__cursordraw= str(cursordraw)


    def GetCursorDraw(self):
        return (self.__cursordraw)


    def SetCursorSelect(self, cursorselect):
        self.__cursorselect= str(cursorselect)


    def GetCursorSelect(self):
        return (self.__cursorselect)


    def SetCursorWin(self, cursorwin):
        self.__cursorwin= str(cursorwin)


    def GetCursorWin(self):
        return (self.__cursorwin)


    def SetDlgBorder(self, dlgborder):
        self.__dlgborder= int(dlgborder)


    def GetDlgBorder(self):
        return(self.__dlgborder)


    def SetDlgOptSize(self, width, height):
        self.__dlgoptsize= (width, height)


    def GetDlgOptSize(self):
        return(self.__dlgoptsize)


    def SetSbRange(self, sbrange):
        self.__sbrange= int(sbrange)


    def GetSbRange(self):
        return(self.__sbrange)


    def SetSbSize(self, sbsize):
        self.__sbsize= int(sbsize)


    def GetSbSize(self):
        return(self.__sbsize)


    def SetWheelEnlarge(self, wheelenlarge):
        self.__wheelenlarge= float(wheelenlarge)


    def GetWheelEnlarge(self):
        return(self.__wheelenlarge)


    def SetWheelReduce(self, wheelreduce):
        self.__wheelreduce= float(wheelreduce)


    def GetWheelReduce(self):
        return(self.__wheelreduce)


    def SetSensitive(self, sensitive):
        self.__sensitive= int(sensitive)


    def GetSensitive(self):
        return(self.__sensitive)


    def SetMaxPenStyleScale(self, maxpenstylescale):
        self.__maxpenstylescale= int(maxpenstylescale)


    def GetMaxPenStyleScale(self):
        return(self.__maxpenstylescale)


    def SetMaxUserScale(self, maxuserscale):
        self.__maxuserscale= int(maxuserscale)


    def GetMaxUserScale(self):
        return(self.__maxuserscale)


    def SetSmartCursSize(self, scsize):
        self.__scsize= int(scsize)


    def GetSmartCursSize(self):
        return(self.__scsize)


    def SetSmartCursPenSize(self, scpensize):
        self.__scpensize= int(scpensize)


    def GetSmartCursPenSize(self):
        return(self.__scpensize)


    def SetSmartCursColPoint(self, sccolpoint):
        self.__sccolpoint= str(sccolpoint)


    def GetSmartCursColPoint(self):
        return(self.__sccolpoint)


    def SetSmartCursColSegm(self, sccolsegm):
        self.__sccolsegm= str(sccolsegm)


    def GetSmartCursColSegm(self):
        return(self.__sccolsegm)


    def SetSmartCursColElem(self, sccolelem):
        self.__sccolelem= str(sccolelem)


    def GetSmartCursColElem(self):
        return(self.__sccolelem)


    def SetSmartCursMarkPenSize(self, scmarkpensize):
        self.__scmarkpensize= float(scmarkpensize)


    def GetSmartCursMarkPenSize(self):
        return(self.__scmarkpensize)


    def SetPreviewColor(self, prevcol):
        self.__prevcol= str(prevcol)


    def GetPreviewColor(self):
        return(self.__prevcol)


    def SetPreviewPenSize(self, prevpensize):
        self.__prevpensize= int(prevpensize)


    def GetPreviewPenSize(self):
        return(self.__prevpensize)


    def SetPreviewPenStyle(self, prevpenstyle):
        self.__prevpenstyle= str(prevpenstyle)


    def GetPreviewPenStyle(self):
        return (self.__prevpenstyle)


    def SetStCoordW(self, stcoordw):
        self.__stcoordw= int(stcoordw)


    def GetStCoordW(self):
        return (self.__stcoordw)


    def SetSelCol(self, selcol):
        self.__selcol= str(selcol)


    def GetSelCol(self):
        return(self.__selcol)


    def SetDlgAttrSize(self, width, height):
        self.__dlgattrsize= (width, height)


    def GetDlgAttrSize(self):
        return(self.__dlgattrsize)


    def SetOptListWidths(self, w1, w2):
        self.__optlistwidths= (w1, w2)


    def GetOptListWidths(self):
        return(self.__optlistwidths)


    def SetAttrListWidths(self, w0, w1, w2, w3, w4, w5, w6, w7, w8, w9, w10, w11, w12, w13, w14, w15):
        self.__attrlistwidths= (w0, w1, w2, w3, w4, w5, w6, w7, w8, w9, w10, w11, w12, w13, w14, w15)


    def GetAttrListWidths(self):
        return(self.__attrlistwidths)


    def SetDrawDefColor(self, name, value, label):
        self.__drawdefcolor= (name, value, label)


    def SetDrawDefLayer(self, name, label):
        self.__drawdeflayer= (name, label)


    def SetDrawDefStyle(self, name, value, label):
        self.__drawdefstyle= (name, value, label)


    def SetDrawDefWidth(self, name, value, label):
        self.__drawdefwidth= (name, value, label)


    def GetDrawDefColor(self):
        return(self.__drawdefcolor)


    def GetDrawDefLayer(self):
        return(self.__drawdeflayer)


    def GetDrawDefStyle(self):
        return(self.__drawdefstyle)


    def GetDrawDefWidth(self):
        return(self.__drawdefwidth)


    def SetDlgHelpSize(self, width, height):
        self.__dlghelpsize= (width, height)


    def GetDlgHelpSize(self):
        return(self.__dlghelpsize)


    def GetInstPaths(self):
        return(self.__instpaths)


    def AddInstPath(self):
        instpath= os.getcwd()
        if instpath not in self.__instpaths:
            if self.__instpaths== '':
                self.__instpaths= instpath
            else:
                self.__instpaths+= ','+ instpath


if __name__== '__main__':
    options= LorzeOptions()

    # wx for load and save configurations
    app= wx.App()

    # test, flag, True= test the object, False= reset lorze options
    test= False

    if test== True:
        # test LorzeOptions()
        print('Values after initialization')
        print(options.GetFrameTitle())
        print(options.GetInstPaths())
        print(options.GetFrameSize())
        print(options.GetFrameTitle())
        print(options.GetGDIBorder())
        print(options.GetDezPlcs())
        print(options.GetCursorDraw())
        print(options.GetCursorSelect())
        print(options.GetCursorWin())
        print(options.GetDlgBorder())
        print(options.GetDlgOptSize())
        print(options.GetSbRange())
        print(options.GetSbSize())
        print(options.GetWheelEnlarge())
        print(options.GetWheelReduce())
        print(options.GetSensitive())
        print(options.GetMaxPenStyleScale())
        print(options.GetMaxUserScale())
        print(options.GetSmartCursSize())
        print(options.GetSmartCursPenSize())
        print(options.GetSmartCursColPoint())
        print(options.GetSmartCursColSegm())
        print(options.GetSmartCursColElem())
        print(options.GetSmartCursMarkPenSize())
        print(options.GetPreviewColor())
        print(options.GetPreviewPenSize())
        print(options.GetPreviewPenStyle())
        print(options.GetStCoordW())
        print(options.GetSelCol())
        print(options.GetDlgAttrSize())
        print(options.GetOptListWidths())
        print(options.GetAttrListWidths())
        print(options.GetDrawDefColor())
        print(options.GetDrawDefLayer())
        print(options.GetDrawDefStyle())
        print(options.GetDrawDefWidth())
        print(options.GetDlgHelpSize())

        print('Set new values')
        options.AddInstPath()
        options.SetFrameSize(100, 200)
        options.SetGDIBorder('wx.RAISED_BORDER')
        options.SetDezPlcs(3)
        options.SetCursorDraw('wx.CURSOR_SPRAYCAN')
        options.SetCursorSelect('wx.CURSOR_IBEAM')
        options.SetCursorWin('wx.CURSOR_BULLSEYE')
        options.SetDlgBorder(7)
        options.SetDlgOptSize(222, 333)
        options.SetSbRange(444)
        options.SetSbSize(55)
        options.SetWheelEnlarge(7.7)
        options.SetWheelReduce(0.55)
        options.SetSensitive(18)
        options.SetMaxPenStyleScale(66)
        options.SetMaxUserScale(1111)
        options.SetSmartCursSize(22)
        options.SetSmartCursPenSize(4)
        options.SetSmartCursColPoint('#FF0000')
        options.SetSmartCursColSegm('#FFFFFF')
        options.SetSmartCursColElem('#00FF00')
        options.SetSmartCursMarkPenSize(9.9)
        options.SetPreviewColor('#FFFFFA')
        options.SetPreviewPenSize(6)
        options.SetPreviewPenStyle('wx.DOT')
        options.SetStCoordW(222)
        options.SetSelCol('#ABCDEF')
        options.SetDlgAttrSize(444, 555)
        options.SetOptListWidths(123, 124)
        options.SetAttrListWidths(100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115)
        options.SetDrawDefColor('White', '255, 255, 255', 'Colour test')
        options.SetDrawDefLayer('Default', 'Layer test')
        options.SetDrawDefStyle('Dot', 'DOT', 'Line style test')
        options.SetDrawDefWidth('2 mm', '2.0', 'Line width test')
        options.SetDlgHelpSize(888, 999)

        print(options.GetFrameTitle())
        print('Added new path', options.GetInstPaths())
        print('Frameszize should be 100, 200 / it is', options.GetFrameSize())
        print('Frametitle was not modified / it is', options.GetFrameTitle())
        print('GDIBorder should be wx.RAISED_BORDER / it is', options.GetGDIBorder())
        print('DezPlcs should be 3 / it is', options.GetDezPlcs())
        print('CursorDraw should be wx.CURSOR_SPRAYCAN / it is', options.GetCursorDraw())
        print('CursorSelect should be wx.CURSOR_IBEAM / it is', options.GetCursorSelect())
        print('CursorWin should be wx.CURSOR_BULLSEYE / it is', options.GetCursorWin())
        print('Dialog border should be 7 / it is', options.GetDlgBorder())
        print('Options-dialog size should be 222, 333 / it is', options.GetDlgOptSize())
        print('Scrollable range should be 444 / it is', options.GetSbRange())
        print('Size scrollbar slider should be 55 / it is', options.GetSbSize())
        print('Enlarge factor mouse whee should be 7.7 / it is', options.GetWheelEnlarge())
        print('Reduce factor mouse wheel should be 0.55 / it is', options.GetWheelReduce())
        print('Pixel size smart cursor should be 18 / it is', options.GetSensitive())
        print('Maximum scale factor penstyles should be 66 / it is', options.GetMaxPenStyleScale())
        print('Maximum userscale should be 1111 / it is', options.GetMaxUserScale())
        print('Smart cursor mark size should be 22 / it is', options.GetSmartCursSize())
        print('Smart cursor mark pen size should be 4 / it is', options.GetSmartCursPenSize())
        print('Smart cursor color point mark should be #FF0000 / it is', options.GetSmartCursColPoint())
        print('Smart cursor color point mark should be #FFFFFF / it is', options.GetSmartCursColSegm())
        print('Smart cursor color point mark should be #00FF00 / it is', options.GetSmartCursColElem())
        print('Smart cur. pen size fact. marked elem. should be 9.9 / it is', options.GetSmartCursMarkPenSize())
        print('Preview color shuld be #FFFFFA / it is', options.GetPreviewColor())
        print('Preview pen size should be 6 / it is', options.GetPreviewPenSize())
        print('Preview pen style should be wx.DOT / it is', options.GetPreviewPenStyle())
        print('Width field coordinates should be 222 / it is', options.GetStCoordW())
        print('Selectet elements color should be #ABCDEF / it is', options.GetSelCol())
        print('Attribute dialog size should be 444, 555 / it is', options.GetDlgAttrSize())
        print('List width option values should be 123, 124 / it is', options.GetOptListWidths())
        print('List width attributes values should be 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115 / it is', options.GetAttrListWidths())
        print('Default drawing colour should be White, 255, 255, 255, Colour test / it is', options.GetDrawDefColor())
        print('Default drawing layer should be Default, Layer test / it is', options.GetDrawDefLayer())
        print('Default drawing line style should be Dot, DOT, Line style test / it is', options.GetDrawDefStyle())
        print('Default drawing line width should be 2 mm, 2.0, Line width test / it is', options.GetDrawDefWidth())
        print('Help dialog size should be 888, 999 / it is', options.GetDlgHelpSize())

        print(u'Save new options')
        options.SaveOptions()
        print(u'Restart LorzeOptions to load')

    else:
        # reset & save options
        print('reset & save options .. done.')
        options.Reset()
        options.SaveOptions()

