#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

class LorzeSelection():
    # Selection object for LORZE erasandcad

    def __init__(self, drawman, geohelp):
        # parent, GraphicPanel()
        self.__drawman= drawman

        # rect coordinates: x1, y1, x2, y2
        self.__rx, self.__ry, self.__rw, self.__rh= 0.0, 0.0, 0.0, 0.0

        # geohelp object
        self.__geohelp= geohelp

        # set elements
        self.Clear()


    def Clear(self):
        # clear selection: list with selected elements, [(lrzid, drawkey), ..]
        self.__elements= []


    def AddRemove(self, lrzid, drawkey):
        # add a drawing element if its not added, remove a drawing element if its added
        found= None
        for i in range(len(self.__elements)):
            # search if element is added
            ilrzid, idrawkey= self.__elements[i]
            if ilrzid== lrzid and idrawkey== drawkey:
                # element added, set index to found
                found= i
                break
        if found== None:
            # element not added: add element
            self.__elements.append((lrzid, drawkey))
        else:
            # element added: remove element
            del self.__elements[found]


    def GetElements(self):
        # return selection: list with selected elements, [(lrzid, drawkey), ..]
        return(self.__elements)


    def ElemSelRect(self, x1, y1, x2, y2):
        # select all elements of the active drawing in a rect, convert x1, y1, x2, y2 to x, y, w, h
        self.__rx, self.__ry, self.__rw, self.__rh= self.__geohelp.RectXYtoWH(x1, y1, x2, y2)

        # dictionairy with drawings
        for drawkey, drawing in self.__drawman.GetAllDrawings().items():

            # each LorzeDrawing, get drawing elements
            for lrzid, lrzelem in drawing.GetElements().items():
                # get rec coordinates and add to points

                if lrzelem.GetId()[0]== 'L':
                    # line
                    if self.GetLine(lrzelem)== True:
                        self.AddRemove(lrzelem.GetId(), drawkey)


    def GetLine(self, element):
        # select line with a rect, selection attribute
        select= False

        # rect of the line, coordinates of the line
        x, y, w, h= element.GetRec()
        lx1, ly1, lx2, ly2= element.GetCoords()

        if self.__geohelp.RectInRect((self.__rx, self.__ry, self.__rw, self.__rh), (x, y, w, h)):
            # element in rect
            if self.__geohelp.PointInRect((lx1, ly1), (self.__rx, self.__ry, self.__rw, self.__rh)):
                # point 1 in rect
                select= True
            elif self.__geohelp.PointInRect((lx2, ly2), (self.__rx, self.__ry, self.__rw, self.__rh)):
                # point 2 in rect
                select= True
            else:
                # search in segment, methode IntersSegm (segment1, [line]), return [(x, y, flag), ..], list with all segments of the rect
                segments= [(self.__rx, self.__ry, self.__rx, self.__ry+ self.__rh), (self.__rx+ self.__rw, self.__ry, self.__rx+ self.__rw, self.__ry+ self.__rh), (self.__rx, self.__ry, self.__rx+ self.__rw, self.__ry), (self.__rx, self.__ry+ self.__rh, self.__rx+ self.__rw, self.__ry+ self.__rh)]

                # get intersection with all 4 segments
                for i in segments:
                    inters= self.__geohelp.IntersSegm(i, [(lx1, ly1, lx2, ly2)])
                    if len(inters)> 0:
                        # intersections found
                        if inters[0][2]== True:
                            select= True
                            break

        return(select)


    def GetSelectAttr(self):
        # Get lists of used attributes in selection / get elements from selection, [(lrzid, drawkey), ..]
        selectionlist= self.__sel.GetElements()

        # dictionairys to store attributes and in which drawing they are used / key= attribute name, value= drawing
        usedcolordict= {}
        usedlayerdict= {}
        usedstyledict= {}
        usedwidthdict= {}

        # define amounts
        color= set()
        layer= set()
        style= set()
        width= set()

        # search in selectionlist
        for i in selectionlist:
            lrzid, drawkey= i

            # get first element of elementlist
            lrzelem= self.__drawings[drawkey].Get([lrzid])[0]

            color= lrzelem.GetColor()
            if color not in usedcolordict:
                    usedcolordict[color]= drawkey
            else:
                if drawkey not in usedcolordict[color]:
                    usedcolordict[color]= usedcolordict[color]+ ','+ drawkey

            layer= lrzelem.GetLayer()
            if layer not in usedlayerdict:
                    usedlayerdict[layer]= drawkey
            else:
                if drawkey not in usedlayerdict[layer]:
                    usedlayerdict[layer]= usedlayerdict[layer]+ ','+ drawkey

            style= lrzelem.GetStyle()
            if style not in usedstyledict:
                    usedstyledict[style]= drawkey
            else:
                if drawkey not in usedstyledict[style]:
                    usedstyledict[style]= usedstyledict[style]+ ','+ drawkey

            width= lrzelem.GetWidth()
            if width not in usedwidthdict:
                    usedwidthdict[width]= drawkey
            else:
                if drawkey not in usedwidthdict[width]:
                    usedwidthdict[width]= usedwidthdict[width]+ ','+ drawkey

        # return used dictionairys
        return(usedcolordict, usedlayerdict, usedstyledict, usedwidthdict)


if __name__== '__main__':
    from lrzoptions import LorzeOptions
    from lrzlog import LorzeLog
    from lrzvalchck import LorzeValueCheck
    from lrzattribute import LorzeAttribute
    from lrzgeohelp import LorzeGeoHelp
    from lrzdrawmanager import LorzeDrawManager
    from lrzdrawline import DrawLine

    options= LorzeOptions()
    log= LorzeLog()
    valchck= LorzeValueCheck()
    attribute= LorzeAttribute(options, log, valchck)
    geohelp= LorzeGeoHelp(options)
    drawman= LorzeDrawManager(attribute, log, geohelp)

    sel= LorzeSelection(drawman, geohelp)

    print('Clear Selection)')
    line1= DrawLine(geohelp)
    line1.SetId('L1')
    line2= DrawLine(geohelp)
    line2.SetId('L2')
    line3= DrawLine(geohelp)
    line3.SetId('L3')

    sel.Clear()
    print(u'Elements:', sel.GetElements())
    print(u'Select/Deselect: line1 main1, line2 main1, line3 main3')
    sel.AddRemove(line1, 'main1')
    sel.AddRemove(line2, 'main1')
    sel.AddRemove(line3, 'main3')
    print(u'Elements:', sel.GetElements())

    print(u'Select/Deselect: line1 main1, line2 main2')
    sel.AddRemove(line1, 'main1')
    sel.AddRemove(line2, 'main2')
    print(u'Elements:', sel.GetElements())

    print(u'Select/Deselect: line2 main1, line2 main2, line3 main3')
    sel.AddRemove(line2, 'main1')
    sel.AddRemove(line2, 'main2')
    sel.AddRemove(line3, 'main3')
    print(u'Elements:', sel.GetElements())

    print(u'Clear Selection)')
    sel.Clear()
    print(u'Elements:', sel.GetElements())
