#!/usr/bin/python
# -*- coding: utf-8 -*-

# LORZE erasandcad, a 2D CAD with an intuitive user interface, simple and easy.
# http://erasand.jimdo.com/python-programme/lorze/
# (C) 2012, Andreas Ulrich

# This file is part of “LORZE erasandcad“

# “LORZE erasandcad“ is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# “LORZE erasandcad“ is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with LORZE erasandcad.  If not, see <http://www.gnu.org/licenses/>.

import re

class LorzeValueCheck():
    # Lorze value checks for different inputs

    def __init__(self):
        pass


    def Float(self, txtval, txtdef, minimal):
        # check float with regular expressions, txtval: string, value to check, txtdef: string, default value, minimal: float, minimal value
        number= re.compile('[0123456789.]+')
        result= number.findall(txtval)
        if len(result)== 0:
            result= [txtdef]
        if float(result[0])< minimal:
            result[0]= str(minimal)
        return(result[0])


    def Int(self, txtint, txtdef, minimal):
        # check integer with regular expressions, txtint: string, integer to check, txtdef: string, default integer, minimal: integer, minimal integer
        number= re.compile('[0123456789]+')
        result= number.findall(txtint)
        if len(result)== 0:
            result= [txtdef]
        if int(result[0])< minimal:
            result[0]= str(minimal)
        return(result[0])


    def StringToList(self, text, seperator, floatflag):
        # converts a string to a list, definied seperator, floatflag for lists with float, text: string, string to convert, speperator: string, seperation of values, floatflag: True= returns a list with floats, False returns a list with integers
        splitlist= text.split(seperator)
        resultlist= []
        for i in splitlist:
            if floatflag== True:
                number= re.compile('[0123456789.]+')
                result= number.findall(i)
                if len(result)== 0:
                    resultlist.append(0.0)
                else:
                    resultlist.append(float(result[0]))
            else:
                number= re.compile('[0123456789]+')
                result= number.findall(i)
                if len(result)== 0:
                    resultlist.append(0)
                else:
                    resultlist.append(int(result[0]))
        return(resultlist)


    def ListToString(self, valuelist):
        # converts a list to a string without [ and ]
        text= str(valuelist)
        text= text.lstrip('[').rstrip(']')
        return(text)


    def RGBtxtToHTMLcol(self, rgbtext):
        # convert a RGB color text to a HTML color text, example: rgbtext= '0, 0, 0', htmltext= '#000000'
        collist= self.StringToList(rgbtext, ',', False)
        if len(collist)< 3:
            return('#000000')

        htmltext= []
        for i in range(3):
            htmltext.append(hex(collist[i])[2:])
            if len(htmltext[i])< 2:
                htmltext[i]= '0'+ htmltext[i]

        return('#'+ htmltext[0]+ htmltext[1]+ htmltext[2])


    def HTMLcolToRGBtxt(self, htmltext):
        # convert a HTML color text to a RGB color text, example: htmltext= '#000000', rgbtext= '0, 0, 0', int('0xff', 0)
        collist= []
        for i in range(3):
            p= (i* 2)+ 1
            hexcode= htmltext[p:p+ 2]
            decimal= int('0x'+ hexcode, 0)
            collist.append(str(decimal))

        return(collist[0]+ ', '+ collist[1]+ ', '+ collist[2])


    def WxStyleToLorzeText(self, wxstyle):
        # convert a wxPython line style to a Lorze line style, example: wxStyle= ('wx.USER_DASH', [40, 50, 40]), LorzeStyle= '40, 50, 40'
        wxsystem= ('wx.SOLID', 'wx.DOT', 'wx.LONG_DASH', 'wx.SHORT_DASH', 'wx.DOT_DASH')
        typ, userdash= wxstyle
        if typ in wxsystem:
            lrztext= typ[3:]
        elif typ== 'wx.USER_DASH':
            lrztext= self.ListToString(userdash)
        else:
            lrztext= 'SOLID'

        return(lrztext)


    def LorzeTextToWxStyle(self, lrztext):
        # convert a Lorze line style to a wxPython line style, example: LorzeStyle= 'LONG_DASH', wxStyle= ('wx.LONG_DASH', [])
        wxsystem= ('SOLID', 'DOT', 'LONG_DASH', 'SHORT_DASH', 'DOT_DASH')
        userdash= self.StringToList(lrztext, ',', False)
        wxtemp= 'wx.'+ lrztext
        if lrztext in wxsystem:
            wxstyle= ('wx.'+ lrztext, [])
        elif len(userdash)> 0:
            wxstyle= ('wx.USER_DASH', userdash)
        else:
            wxstyle= ('SOLID', [])

        return(wxstyle)


if __name__== '__main__':
    vc= LorzeValueCheck()
    print('Int Check 7, default: 3, minimum 2, return: ', vc.Int('7', '3', 2))
    print('Int Check 1, default: 3, minimum 2, return: ', vc.Int('1', '3', 2))
    print('Int Check 0, default: 3, minimum 2, return: ', vc.Int('0', '3', 2))
    print('Int Check adsavsfd, default: 3, minimumm 2, return: ', vc.Int('adsavsfd', '3', 2))
    print('Float Check 5.9, default: 6.7, minimum 1.1, return: ', vc.Float('5.9', '6.7', 1.1))
    print('Float Check 0.3, default: 6.7, minimum 1.1, return: ', vc.Float('0.3', '6.7', 1.1))
    print('Float Check 0, default: 6.7, minimum 1.1, return: ', vc.Float('0', '6.7', 1.1))
    print('Float Check 9, default: 6.7, minimum 1.1, return: ', vc.Float('9', '6.7', 1.1))
    print('Float Check kdfuri87, default: 6.7, minimum 1.1, return: ', vc.Float('kdfuri87', '6.7', 1.1))
    print('StringToList: "1, 2, 3, 4", seperator= ",", floatflag= False', vc.StringToList('1, 2, 3, 4', ',', False))
    print('StringToList: "1.23, 4.56, 7.89, 10.111213", seperator= ",", floatflag= True', vc.StringToList('1.23, 4.56, 7.89, 10.111213', ',', True))
    print('ListToString: [1, 2, 3, 4]', vc.ListToString([1, 2, 3, 4]))
    print('ListToString: [1.23, 4.56, 7.89, 10.111213]', vc.ListToString([1.23, 4.56, 7.89, 10.111213]))
    print('RGBtxtToHTMLcol: 2, 4, 8', vc.RGBtxtToHTMLcol('2, 4, 8'))
    print('RGBtxtToHTMLcol: 16, 32, 64', vc.RGBtxtToHTMLcol('16, 32, 64'))
    print('HTMLcolToRGBtxt: #102040', vc.HTMLcolToRGBtxt('#102040'))
    print('HTMLcolToRGBtxt: #020408', vc.HTMLcolToRGBtxt('#020408'))
    print('WxStyleToLorzeText: wx.SOLID, []', vc.WxStyleToLorzeText(('wx.SOLID', [])))
    print('WxStyleToLorzeText: wx.USER_DASH, [1, 2, 3]', vc.WxStyleToLorzeText(('wx.USER_DASH', [1, 2, 3])))
    print('WxStyleToLorzeText: error, 0', vc.WxStyleToLorzeText(('error', 0)))
    print('LorzeTextToWxStyle: 1, 2, 3', vc.LorzeTextToWxStyle('1, 2, 3'))
    print('LorzeTextToWxStyle: DOT', vc.LorzeTextToWxStyle('DOT'))
    print('LorzeTextToWxStyle: error', vc.LorzeTextToWxStyle('error'))
